@@ -4,7 +4,7 @@
 /*
  * conffile.h	Defines for the conffile parsing routines.
  *
- * Version:	$Id: conffile.h,v 1.18 2001/02/15 07:26:08 pacman Exp $
+ * Version:	$Id: conffile.h,v 1.20 2004/02/26 19:04:19 aland Exp $
  *
  */
 
@@ -47,16 +47,16 @@ void		cf_pair_free(CONF_PAIR **cp);
 void		cf_section_free(CONF_SECTION **cp);
 int		cf_section_parse(CONF_SECTION *cs, void *base, const CONF_PARSER *variables);
 
-/* JLN -- Newly added */
-		
+CONF_SECTION *conf_read(const char *fromfile, int fromline,
+			const char *conffile, CONF_SECTION *parent);
+
+
 CONF_PAIR	*cf_pair_find(CONF_SECTION *section, const char *name);
 CONF_PAIR	*cf_pair_find_next(CONF_SECTION *section, CONF_PAIR *pair, const char *name);
 CONF_SECTION	*cf_section_find(const char *name);
 CONF_SECTION	*cf_section_sub_find(CONF_SECTION *section, const char *name);
 char 		*cf_section_value_find(CONF_SECTION *section, const char *attr);
 
-int		read_radius_conf_file(void);
-
 char *cf_pair_attr(CONF_PAIR *pair);
 char *cf_pair_value(CONF_PAIR *pair);
 char *cf_section_name1(CONF_SECTION *section);
