@@ -5,7 +5,7 @@
  * libradius.h	Structures and prototypes
  *		for the radius library.
  *
- * Version:	$Id: libradius.h,v 1.60.2.1 2003/09/03 13:03:14 phampson Exp $
+ * Version:	$Id: libradius.h,v 1.76 2004/05/28 07:20:15 phampson Exp $
  *
  */
 
@@ -37,7 +37,7 @@
 # include <sys/time.h>
 # include <time.h>
 #else
-# if HAVE_SYS_TIME_H
+# ifdef HAVE_SYS_TIME_H
 #  include <sys/time.h>
 # else
 #  include <time.h>
@@ -81,6 +81,18 @@
 #  define TAG_ANY               -128   /* minimum signed char */
 #endif
 
+#if defined(__GNUC__)
+# define PRINTF_LIKE(n) __attribute__ ((format(printf, n, n+1)))
+# define NEVER_RETURNS __attribute__ ((noreturn))
+# define UNUSED __attribute__ ((unused))
+# define BLANK_FORMAT " "	/* GCC_LINT whines about empty formats */
+#else
+# define PRINTF_LIKE(n)	/* ignore */
+# define NEVER_RETURNS /* ignore */
+# define UNUSED /* ignore */
+# define BLANK_FORMAT ""
+#endif
+
 typedef struct attr_flags {
 	char			addport;	/* Add port to IP address */
 	char			has_tag;	/* attribute allows tags */
@@ -104,19 +116,16 @@ typedef struct dict_attr {
 	int			type;
 	int			vendor;
         ATTR_FLAGS              flags;
-	struct dict_attr	*next;
 } DICT_ATTR;
 
 typedef struct dict_value {
 	char			name[40];
-	char			attrname[40];
 	int			attr;
 	int			value;
-	struct dict_value	*next;
 } DICT_VALUE;
 
 typedef struct dict_vendor {
-	char			vendorname[40];
+	char			name[40];
 	int			vendorpec;
 	struct dict_vendor	*next;
 } DICT_VENDOR;
@@ -149,7 +158,7 @@ typedef struct radius_packet {
 	u_short			src_port;
 	u_short			dst_port;
 	int			id;
-	int			code;
+	unsigned int		code;
 	uint8_t			vector[AUTH_VECTOR_LEN];
 	time_t			timestamp;
 	int			verified;
@@ -179,7 +188,8 @@ DICT_ATTR	*dict_attrbyvalue(int attr);
 DICT_ATTR	*dict_attrbyname(const char *attr);
 DICT_VALUE	*dict_valbyattr(int attr, int val);
 DICT_VALUE	*dict_valbyname(int attr, const char *val);
-int		dict_vendorname(const char *name);
+int		dict_vendorbyname(const char *name);
+DICT_VENDOR	*dict_vendorbyvalue(int vendor);
 
 /*
  *  Compatibility
@@ -195,6 +205,17 @@ int		dict_vendorname(const char *name);
 /*#define dict_valget	dict_valbyattr almost but not quite*/
 #endif
 
+/* get around diffrent ctime_r styles */
+#ifdef CTIMERSTYLE
+#if CTIMERSTYLE == SOLARISSTYLE
+#define CTIME_R(a,b,c) ctime_r(a,b,c)
+#else
+#define CTIME_R(a,b,c) ctime_r(a,b)
+#endif
+#else
+#define CTIME_R(a,b,c) ctime_r(a,b)
+#endif
+
 /* md5.c */
 
 void		librad_md5_calc(u_char *, u_char *, u_int);
@@ -205,6 +226,12 @@ void lrad_hmac_md5(const unsigned char *text, int text_len,
 		   const unsigned char *key, int key_len,
 		   unsigned char *digest);
 
+/* hmacsha1.c */
+
+void lrad_hmac_sha1(const unsigned char *text, int text_len,
+		    const unsigned char *key, int key_len,
+		    unsigned char *digest);
+
 /* radius.c */
 int		rad_send(RADIUS_PACKET *, const RADIUS_PACKET *, const char *secret);
 RADIUS_PACKET	*rad_recv(int fd);
@@ -214,15 +241,17 @@ void		rad_free(RADIUS_PACKET **);
 int		rad_pwencode(char *encpw, int *len, const char *secret, const char *vector);
 int		rad_pwdecode(char *encpw, int len, const char *secret, const char *vector);
 int		rad_tunnel_pwencode(char *encpw, int *len, const char *secret, const char *vector);
-int		rad_tunnel_pwdecode(char *encpw, int *len, const char *secret, const char *vector);
+int		rad_tunnel_pwdecode(uint8_t *encpw, int *len, const char *secret, const char *vector);
 int		rad_chap_encode(RADIUS_PACKET *packet, char *output, int id, VALUE_PAIR *password);
 
 /* valuepair.c */
 VALUE_PAIR	*paircreate(int attr, int type);
 void		pairfree(VALUE_PAIR **);
+void            pairbasicfree(VALUE_PAIR *pair);
 VALUE_PAIR	*pairfind(VALUE_PAIR *, int);
 void		pairdelete(VALUE_PAIR **, int);
 void		pairadd(VALUE_PAIR **, VALUE_PAIR *);
+void            pairreplace(VALUE_PAIR **first, VALUE_PAIR *add);
 VALUE_PAIR	*paircopy(VALUE_PAIR *vp);
 VALUE_PAIR	*paircopy2(VALUE_PAIR *vp, int attr);
 void		pairmove(VALUE_PAIR **to, VALUE_PAIR **from);
@@ -231,6 +260,7 @@ VALUE_PAIR	*pairparsevalue(VALUE_PAIR *vp, const char *value);
 VALUE_PAIR	*pairmake(const char *attribute, const char *value, int operator);
 VALUE_PAIR	*pairread(char **ptr, LRAD_TOKEN *eol);
 LRAD_TOKEN	userparse(char *buffer, VALUE_PAIR **first_pair);
+VALUE_PAIR     *readvp2(FILE *fp, int *pfiledone, const char *errprefix);
 
 /*
  *	Error functions.
@@ -259,6 +289,10 @@ char *		ip_hostname (char *buf, size_t buflen, uint32_t ipaddr);
 uint32_t	ip_getaddr (const char *);
 char *		ip_ntoa(char *, uint32_t);
 uint32_t	ip_addr(const char *);
+char		*ifid_ntoa(char *buffer, size_t size, uint8_t *ifid);
+uint8_t		*ifid_aton(const char *ifid_str, uint8_t *ifid);
+char		*ipv6_ntoa(char *buffer, size_t size, void *ip6addr);
+int		ipv6_addr(const char *ip6_str, void *ip6addr);
 char		*strNcpy(char *dest, const char *src, int n);
 void		rad_lowercase(char *str);
 void		rad_rmspace(char *str);
@@ -268,7 +302,7 @@ int		rad_unlockfd(int fd, int lock_len);
 
 #ifdef ASCEND_BINARY
 /* filters.c */
-int		filterBinary(VALUE_PAIR *pair, const char *valstr);
+int		ascend_parse_filter(VALUE_PAIR *pair);
 void		print_abinary(VALUE_PAIR *vp, u_char *buffer, int len);
 #endif /*ASCEND_BINARY*/
 
@@ -293,12 +327,26 @@ void lrad_isaac(lrad_randctx *ctx);
 void lrad_randinit(lrad_randctx *ctx, int flag);
 uint32_t lrad_rand(void);	/* like rand(), but better. */
 
-void lrad_lmpwdhash(const unsigned char *password,unsigned char *lmhash);
-void lrad_mschap(const unsigned char *win_password,
-		 const unsigned char *challenge, unsigned char *response);
-
-
 /* crypt wrapper from crypt.c */
 int lrad_crypt_check(const char *key, const char *salt);
 
+/* rbtree.c */
+typedef struct rbtree_t rbtree_t;
+typedef struct rbnode_t rbnode_t;
+
+rbtree_t       *rbtree_create(int (*Compare)(const void *, const void *),
+			       void (*freeNode)(void *),
+			       int replace_flag);
+void		rbtree_free(rbtree_t *tree);
+int		rbtree_insert(rbtree_t *tree, const void *Data);
+void		rbtree_delete(rbtree_t *tree, rbnode_t *Z);
+rbnode_t       *rbtree_find(rbtree_t *tree, const void *Data);
+void	       *rbtree_finddata(rbtree_t *tree, const void *Data);
+int		rbtree_num_elements(rbtree_t *tree);
+void	       *rbtree_node2data(rbtree_t *tree, rbnode_t *node);
+
+/* callback order for walking  */
+typedef enum { PreOrder, InOrder, PostOrder } RBTREE_ORDER;
+int rbtree_walk(rbtree_t *tree, int (*callback)(void *), RBTREE_ORDER order);
+
 #endif /*LIBRADIUS_H*/
