@@ -1,7 +1,7 @@
 /*
  * radius.h	Constants of the radius protocol.
  *
- * Version:	$Id: radius.h,v 1.54 2003/04/21 20:39:57 aland Exp $
+ * Version:	$Id: radius.h,v 1.66 2004/04/30 18:23:08 aland Exp $
  *
  */
 
@@ -12,6 +12,9 @@
 #define PW_TYPE_DATE			3
 #define PW_TYPE_ABINARY			4
 #define PW_TYPE_OCTETS			5
+#define PW_TYPE_IFID			6
+#define PW_TYPE_IPV6ADDR		7
+#define PW_TYPE_IPV6PREFIX		8
 
 #define	PW_AUTHENTICATION_REQUEST	1
 #define	PW_AUTHENTICATION_ACK		2
@@ -43,10 +46,6 @@
 #define	PW_CHAP_PASSWORD		3
 #define	PW_NAS_IP_ADDRESS		4
 #define	PW_NAS_PORT			5
-/*
- *  FIXME: We should remove this...
- */
-#define	PW_NAS_PORT_ID			5
 #define	PW_SERVICE_TYPE			6
 #define	PW_FRAMED_PROTOCOL		7
 #define	PW_FRAMED_IP_ADDRESS		8
@@ -137,6 +136,8 @@
 #define PW_PRE_PROXY_TYPE		1015
 #define PW_POST_PROXY_TYPE		1016
 #define PW_PRE_ACCT_TYPE		1017
+#define PW_EAP_TYPE			1018
+#define PW_EAP_TLS_REQUIRE_CLIENT_CERT	1019
 
 #define PW_USER_CATEGORY		1029
 #define PW_GROUP_NAME			1030
@@ -182,6 +183,8 @@
 #define PW_REWRITE_RULE			1078
 #define PW_SQL_GROUP			1079
 #define PW_RESPONSE_PACKET_TYPE		1080
+#define PW_PACKET_DST_PORT		1081
+#define PW_MS_CHAP_USE_NTLM_AUTH	1082
 
 /*
  *	Integer Translations
@@ -198,7 +201,7 @@
 #define PW_NAS_PROMPT_USER		7
 #define PW_AUTHENTICATE_ONLY		8
 #define PW_CALLBACK_NAS_PROMPT		9
-	      
+
 /*	Framed Protocols	*/
 
 #define	PW_PPP				1
@@ -232,6 +235,7 @@
 #define PW_AUTHTYPE_ACTIVCARD		5
 #define PW_AUTHTYPE_EAP                 6
 #define PW_AUTHTYPE_ACCEPT		254
+#define PW_AUTHTYPE_MS_CHAP		1028
 
 /*	Port Types		*/
 
@@ -260,6 +264,16 @@
  */
 #define PW_FREERADIUS_PROXIED_TO	((VENDORPEC_FREERADIUS<<16)|1)
 
+/*
+ *  Microsoft has vendor code 311.
+ */
+#define PW_MSCHAP_RESPONSE	((311 << 16) | 1)
+#define PW_MSCHAP_ERROR		((311 << 16) | 2)
+#define PW_MSCHAP_CHALLENGE	((311 << 16) | 11)
+#define PW_MSCHAP2_RESPONSE	((311 << 16) | 25)
+#define PW_MSCHAP2_SUCCESS	((311 << 16) | 26)
+
+
 /*
  *  Old nonsense.  Will be deleted ASAP
  */
