@@ -4,15 +4,16 @@
  * request_list.h	Hide the handling of the REQUEST list from
  *			the main server.
  *
- * Version:	$Id: request_list.h,v 1.5 2000/11/20 21:51:47 aland Exp $
+ * Version:	$Id: request_list.h,v 1.9 2004/05/28 14:42:27 aland Exp $
  *
  */
 
 extern int rl_init(void);
 extern void rl_delete(REQUEST *request);
 extern void rl_add(REQUEST *request);
-extern REQUEST *rl_find(REQUEST *request);
-extern REQUEST *rl_find_proxy(REQUEST *request);
+extern REQUEST *rl_find(RADIUS_PACKET *packet);
+extern int rl_add_proxy(REQUEST *request);
+extern REQUEST *rl_find_proxy(RADIUS_PACKET *packet);
 extern REQUEST *rl_next(REQUEST *request);
 extern int rl_num_requests(void);
 
@@ -22,5 +23,6 @@ extern int rl_num_requests(void);
 typedef int (*RL_WALK_FUNC)(REQUEST *, void *);
 
 extern int rl_walk(RL_WALK_FUNC walker, void *data);
+extern struct timeval *rl_clean_list(time_t now);
 
 #endif /* _REQUEST_LIST_H */
