@@ -2,11 +2,26 @@
  * missing.c	Replacements for functions that are or can be
  *		missing on some platforms.
  *
- * Version:	$Id: missing.c,v 1.9 2002/02/25 22:34:28 aland Exp $
+ * Version:	$Id: missing.c,v 1.12 2004/02/26 19:04:21 aland Exp $
  *
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License as published by the Free Software Foundation; either
+ *   version 2.1 of the License, or (at your option) any later version.
+ *
+ *   This library is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+ *   Lesser General Public License for more details.
+ *
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with this library; if not, write to the Free Software
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
+ *
+ * Copyright 2000  The FreeRADIUS server project
  */
 
-static const char rcsid[] = "$Id: missing.c,v 1.9 2002/02/25 22:34:28 aland Exp $";
+static const char rcsid[] = "$Id: missing.c,v 1.12 2004/02/26 19:04:21 aland Exp $";
 
 #include	"autoconf.h"
 
@@ -91,7 +106,7 @@ int gethostname(char *name, int len)
 	char		*h;
 
 	h = getenv("HOSTNAME");
-	if (strlen(h) + 1 > len)
+	if (!h || (strlen(h) + 1 > len))
 		return -1;
 	strcpy(name, h);
 	return 0;
@@ -196,4 +211,4 @@ struct tm *gmtime_r(const time_t *l_clock, struct tm *result)
 
   return result;
 }
-#endif 
+#endif
