@@ -1,11 +1,26 @@
 /*
  * print.c	Routines to print stuff.
  *
- * Version:	$Id: print.c,v 1.19 2002/02/25 22:34:29 aland Exp $
+ * Version:	$Id: print.c,v 1.26 2004/02/26 19:04:21 aland Exp $
  *
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License as published by the Free Software Foundation; either
+ *   version 2.1 of the License, or (at your option) any later version.
+ *
+ *   This library is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+ *   Lesser General Public License for more details.
+ *
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with this library; if not, write to the Free Software
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
+ *
+ * Copyright 2000  The FreeRADIUS server project
  */
 
-static const char rcsid[] = "$Id: print.c,v 1.19 2002/02/25 22:34:29 aland Exp $";
+static const char rcsid[] = "$Id: print.c,v 1.26 2004/02/26 19:04:21 aland Exp $";
 
 #include	"autoconf.h"
 
@@ -95,7 +110,11 @@ int vp_prints_value(char * out, int outlen, VALUE_PAIR *vp, int delimitst)
 
 	switch (vp->type) {
 		case PW_TYPE_STRING:
-			if (vp->attribute == PW_NAS_PORT_ID)
+			/*
+			 *  NAS-Port may have multiple integer values?
+			 *  This is an internal server extension...
+			 */
+			if (vp->attribute == PW_NAS_PORT)
 				a = (char *)vp->strvalue;
 			else {
 				if (delimitst && vp->flags.has_tag) {
@@ -142,9 +161,11 @@ int vp_prints_value(char * out, int outlen, VALUE_PAIR *vp, int delimitst)
 		case PW_TYPE_DATE:
 			t = vp->lvalue;
 			if (delimitst) {
-			  strftime(buf, sizeof(buf), "\"%b %e %Y\"", gmtime_r(&t, &s_tm));
+			  strftime(buf, sizeof(buf), "\"%b %e %Y %H:%M:%S %Z\"",
+				   localtime_r(&t, &s_tm));
 			} else {
-			  strftime(buf, sizeof(buf), "%b %e %Y", gmtime_r(&t, &s_tm));
+			  strftime(buf, sizeof(buf), "%b %e %Y %H:%M:%S %Z",
+				   localtime_r(&t, &s_tm));
 			}
 			a = buf;
 			break;
@@ -174,15 +195,56 @@ int vp_prints_value(char * out, int outlen, VALUE_PAIR *vp, int delimitst)
 		  a = buf;
 		  break;
 
+		case PW_TYPE_IFID:
+			a = ifid_ntoa(buf, sizeof(buf), vp->strvalue);
+			break;
+
+		case PW_TYPE_IPV6ADDR:
+			a = ipv6_ntoa(buf, sizeof(buf), vp->strvalue);
+			break;
+
 		default:
 			a = 0;
 			break;
 	}
 	strNcpy(out, a?a:"UNKNOWN-TYPE", outlen);
-	
+
 	return strlen(out);
 }
 
+/*
+ *  This is a hack, and has to be kept in sync with tokens.h
+ */
+static const char *vp_tokens[] = {
+  "?",				/* T_INVALID */
+  "EOL",			/* T_EOL */
+  "{",
+  "}",
+  "(",
+  ")",
+  ",",
+  ";",
+  "+=",
+  "-=",
+  ":=",
+  "=",
+  "!=",
+  ">=",
+  ">",
+  "<=",
+  "<",
+  "=~",
+  "!~",
+  "=*",
+  "~*",
+  "==",
+  "#",
+  "<BARE-WORD>",
+  "<\"STRING\">",
+  "<'STRING'>",
+  "<`STRING`>"
+};
+
 
 /*
  *	Print one attribute and value into a string.
@@ -190,6 +252,7 @@ int vp_prints_value(char * out, int outlen, VALUE_PAIR *vp, int delimitst)
 int vp_prints(char *out, int outlen, VALUE_PAIR *vp)
 {
 	int		len;
+	const char	*token = NULL;
 
 	out[0] = 0;
 	if (!vp) return 0;
@@ -198,20 +261,28 @@ int vp_prints(char *out, int outlen, VALUE_PAIR *vp)
 		return 0;
 	}
 
+	if ((vp->operator > T_INVALID) &&
+	    (vp->operator < T_TOKEN_LAST)) {
+		token = vp_tokens[vp->operator];
+	} else {
+		token = "<INVALID-TOKEN>";
+	}
+
 	if( vp->flags.has_tag ) {
 
-		snprintf(out, outlen, "%s:%d = ", vp->name, vp->flags.tag);
-		
+		snprintf(out, outlen, "%s:%d %s ", vp->name, vp->flags.tag,
+			 token);
+
 		len = strlen(out);
 		vp_prints_value(out + len, outlen - len, vp, 1);
 
 	} else {
 
-	        snprintf(out, outlen, "%s = ", vp->name);
+	        snprintf(out, outlen, "%s %s ", vp->name, token);
 		len = strlen(out);
 		vp_prints_value(out + len, outlen - len, vp, 1);
 
-	}	  
+	}
 
 	return strlen(out);
 }
