@@ -1,7 +1,7 @@
 /*
  * radius.c	Functions to send/receive radius packets.
  *
- * Version:	$Id: radius.c,v 1.125.2.1 2004/08/30 17:48:31 aland Exp $
+ * Version:	$Id: radius.c,v 1.125.2.3 2004/12/19 20:11:54 aland Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,7 @@
  * Copyright 2000-2003  The FreeRADIUS server project
  */
 
-static const char rcsid[] = "$Id: radius.c,v 1.125.2.1 2004/08/30 17:48:31 aland Exp $";
+static const char rcsid[] = "$Id: radius.c,v 1.125.2.3 2004/12/19 20:11:54 aland Exp $";
 
 #include	"autoconf.h"
 #include	"md5.h"
@@ -2061,6 +2061,7 @@ static void random_vector(uint8_t *vector)
 		lrad_rand_pool.randrsl[2] = (uint32_t) vector;
 
 		lrad_randinit(&lrad_rand_pool, 1);
+		lrad_pool_initialized = 1;
 	}
 
 	lrad_isaac(&lrad_rand_pool);
@@ -2125,6 +2126,7 @@ RADIUS_PACKET *rad_alloc(int newvector)
 	memset(rp, 0, sizeof(RADIUS_PACKET));
 	if (newvector)
 		random_vector(rp->vector);
+	lrad_rand();
 
 	return rp;
 }
