@@ -1,7 +1,7 @@
 /*
  * valuepair.c	Functions to handle VALUE_PAIRs
  *
- * Version:	$Id: valuepair.c,v 1.82 2004/05/06 14:37:21 aland Exp $
+ * Version:	$Id: valuepair.c,v 1.82.2.1 2004/12/03 18:43:32 aland Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,7 @@
  * Copyright 2000  The FreeRADIUS server project
  */
 
-static const char rcsid[] = "$Id: valuepair.c,v 1.82 2004/05/06 14:37:21 aland Exp $";
+static const char rcsid[] = "$Id: valuepair.c,v 1.82.2.1 2004/12/03 18:43:32 aland Exp $";
 
 #include	"autoconf.h"
 
@@ -1222,7 +1222,7 @@ VALUE_PAIR *pairread(char **ptr, LRAD_TOKEN *eol)
 		 */
 	case T_DOUBLE_QUOTED_STRING:
 		p = strchr(value, '%');
-		if (p) {
+		if (p && (p[1] == '{')) {
 			vp = pairmake(attr, NULL, token);
 			if (!vp) {
 				*eol = T_INVALID;
