@@ -1,7 +1,7 @@
 /*
  * auth.c	User authentication.
  *
- * Version:	$Id: auth.c,v 1.136 2004/03/08 21:51:03 aland Exp $
+ * Version:	$Id: auth.c,v 1.136.2.1 2004/12/29 23:08:42 phampson Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Jeff Carneal <jeff@apex.net>
  */
 
-static const char rcsid[] = "$Id: auth.c,v 1.136 2004/03/08 21:51:03 aland Exp $";
+static const char rcsid[] = "$Id: auth.c,v 1.136.2.1 2004/12/29 23:08:42 phampson Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -427,6 +427,31 @@ int rad_postauth(REQUEST *request)
 	return result;
 }
 
+/*
+ *	Before sending an Access-Reject, call the modules in the
+ *	Post-Auth-Type REJECT stanza.
+ */
+static int rad_postauth_reject(REQUEST *request)
+{
+	int		result;
+	VALUE_PAIR	*tmp;
+	DICT_VALUE	*dval;
+
+	dval = dict_valbyname(PW_POST_AUTH_TYPE, "REJECT");
+	if (dval) {
+		/* Overwrite the Post-Auth-Type with the value REJECT */
+		pairdelete(&request->config_items, PW_POST_AUTH_TYPE);
+		tmp = paircreate(PW_POST_AUTH_TYPE, PW_TYPE_INTEGER);
+		tmp->lvalue = dval->value;
+		pairadd(&request->config_items, tmp);
+		result = rad_postauth(request);
+	} else {
+		/* No REJECT stanza */
+		result = RLM_MODULE_OK;
+	}
+	return result;
+}
+
 /*
  *	Process and reply to an authentication request
  *
@@ -501,6 +526,7 @@ int rad_authenticate(REQUEST *request)
 		    (request->proxy_reply->code != PW_ACCESS_CHALLENGE)) {
 			rad_authlog("Login incorrect (Home Server says so)", request, 0);
 			request->reply->code = PW_AUTHENTICATION_REJECT;
+			rad_postauth_reject(request);
 			return RLM_MODULE_REJECT;
 		}
 	}
@@ -777,22 +803,11 @@ autz_redo:
 
 	/*
 	 *	Result should be >= 0 here - if not, it means the user
-	 *	is rejected, so we overwrite the Post-Auth-Type with
-	 *	the value REJECT and call the post-authentication
-	 *	step.
+	 *	is rejected, so we just process post-auth and return.
 	 */
 	if (result < 0) {
-		DICT_VALUE *dval;
-
-		dval = dict_valbyname(PW_POST_AUTH_TYPE, "REJECT");
-		if (dval) {
-			pairdelete(&request->config_items, PW_POST_AUTH_TYPE);
-			tmp = paircreate(PW_POST_AUTH_TYPE, PW_TYPE_INTEGER);
-			tmp->lvalue = dval->value;
-			pairadd(&request->config_items, tmp);
-			rad_postauth(request);
-		}
-		return RLM_MODULE_OK;
+		rad_postauth_reject(request);
+		return RLM_MODULE_REJECT;
 	}
 
 	/*
@@ -899,6 +914,8 @@ autz_redo:
 			rad_authlog("Login incorrect (external check failed)",
 					request, 0);
 
+			rad_postauth_reject(request);
+
 			return RLM_MODULE_REJECT;
 		}
 	}
