@@ -1,7 +1,7 @@
 /*
  * files.c	Read config files into memory.
  *
- * Version:     $Id: client.c,v 1.22 2003/06/15 14:09:32 hartwick Exp $
+ * Version:     $Id: client.c,v 1.27 2004/04/07 16:02:05 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,14 +22,14 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-static const char rcsid[] = "$Id: client.c,v 1.22 2003/06/15 14:09:32 hartwick Exp $";
+static const char rcsid[] = "$Id: client.c,v 1.27 2004/04/07 16:02:05 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
 
 #include <sys/stat.h>
 
-#if HAVE_NETINET_IN_H
+#ifdef HAVE_NETINET_IN_H
 #	include <netinet/in.h>
 #endif
 
@@ -71,6 +71,7 @@ int read_clients_file(const char *file)
 	uint32_t mask;
 	int lineno = 0;
 	char *p;
+	int got_clients = FALSE;
 
 	clients_free(mainconfig.clients);
 	mainconfig.clients = NULL;
@@ -82,7 +83,6 @@ int read_clients_file(const char *file)
 		   done. */
 		return 0;
 	}
-	radlog(L_INFO, "Using deprecated clients file.  Support for this will go away soon.");
 
 	while(fgets(buffer, 256, fp) != NULL) {
 		lineno++;
@@ -96,7 +96,7 @@ int read_clients_file(const char *file)
 		 */
 		p = buffer;
 		while (*p &&
-				((*p == ' ') || (*p == '\t'))) 
+				((*p == ' ') || (*p == '\t')))
 			p++;
 
 		/*
@@ -121,7 +121,7 @@ int read_clients_file(const char *file)
 		mask = ~0;
 
 		if (p) {
-			int i, mask_length;
+			int mask_length;
 
 			*p = '\0';
 			p++;
@@ -129,13 +129,14 @@ int read_clients_file(const char *file)
 			mask_length = atoi(p);
 			if ((mask_length < 0) || (mask_length > 32)) {
 				radlog(L_ERR, "%s[%d]: Invalid value '%s' for IP network mask.",
-						file, lineno, p);
+				       file, lineno, p);
 				return -1;
 			}
 
-			mask = (1 << 31);
-			for (i = 1; i < mask_length; i++) {
-				mask |= (mask >> 1);
+			if (mask_length == 0) {
+				mask = 0;
+			} else {
+				mask = ~0 << (32 - mask_length);
 			}
 		}
 
@@ -144,26 +145,30 @@ int read_clients_file(const char *file)
 		 */
 		if (strlen(hostnm) >= sizeof(c->longname)) {
 			radlog(L_ERR, "%s[%d]: host name of length %d is greater than the allowed maximum of %d.",
-					file, lineno,
-					strlen(hostnm), sizeof(c->longname) - 1);
+			       file, lineno,
+			       (int) strlen(hostnm),
+			       (int) sizeof(c->longname) - 1);
 			return -1;
 		}
 		if (strlen(secret) >= sizeof(c->secret)) {
 			radlog(L_ERR, "%s[%d]: secret of length %d is greater than the allowed maximum of %d.",
-					file, lineno,
-					strlen(secret), sizeof(c->secret) - 1);
+			       file, lineno,
+			       (int) strlen(secret),
+			       (int) sizeof(c->secret) - 1);
 			return -1;
 		}
 		if (strlen(shortnm) > sizeof(c->shortname)) {
 			radlog(L_ERR, "%s[%d]: short name of length %d is greater than the allowed maximum of %d.",
-					file, lineno,
-					strlen(shortnm), sizeof(c->shortname) - 1);
+			       file, lineno,
+			       (int) strlen(shortnm),
+			       (int) sizeof(c->shortname) - 1);
 			return -1;
 		}
-		
+
 		/*
 		 *	It should be OK now, let's create the buffer.
 		 */
+		got_clients = TRUE;
 		c = rad_malloc(sizeof(RADCLIENT));
 		memset(c, 0, sizeof(*c));
 
@@ -216,6 +221,10 @@ int read_clients_file(const char *file)
 	}
 	fclose(fp);
 
+	if (got_clients) {
+		radlog(L_INFO, "Using deprecated clients file.  Support for this will go away soon.");
+	}
+
 	return 0;
 }
 
@@ -244,7 +253,7 @@ RADCLIENT *client_find(uint32_t ipaddr)
  *	Walk the RADCLIENT list displaying the clients.  This function
  *	is for debugging purposes.
  */
-void client_walk(void) 
+void client_walk(void)
 {
 	RADCLIENT *cl;
 	char host_ipaddr[16];
@@ -269,15 +278,16 @@ const char *client_name(uint32_t ipaddr)
 		else
 			return cl->longname;
 	}
-	
-	/* 
-	 * this isn't normally reachable, but if a loggable event happens just 
-	 * after a client list change and a HUP, then we may not know this 
+
+	/*
+	 * this isn't normally reachable, but if a loggable event happens just
+	 * after a client list change and a HUP, then we may not know this
 	 * information any more.
 	 *
 	 * If you see lots of these, then there's something wrong.
 	 */
-	radlog(L_ERR, "Trying to look up name of unknown client %s.\n", ip_ntoa(host_ipaddr, ipaddr));
+	radlog(L_ERR, "Trying to look up name of unknown client %s.\n",
+	       ip_ntoa(host_ipaddr, ipaddr));
 
 	return "UNKNOWN-CLIENT";
 }
