@@ -5,7 +5,7 @@
  *		write a decent parser. I know how to do that, really :)
  *		miquels@cistron.nl
  *
- * Version:	$Id: conffile.c,v 1.93.2.1 2003/07/09 15:55:22 aland Exp $
+ * Version:	$Id: conffile.c,v 1.100 2004/04/06 18:00:57 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#if HAVE_NETINET_IN_H
+#ifdef HAVE_NETINET_IN_H
 #	include <netinet/in.h>
 #endif
 
@@ -43,7 +43,7 @@
 #include "modules.h"
 
 static const char rcsid[] =
-"$Id: conffile.c,v 1.93.2.1 2003/07/09 15:55:22 aland Exp $";
+"$Id: conffile.c,v 1.100 2004/04/06 18:00:57 aland Exp $";
 
 #define xstrdup strdup
 
@@ -148,7 +148,7 @@ static CONF_SECTION *cf_section_alloc(const char *name1, const char *name2,
 {
 	CONF_SECTION	*cs;
 
-	if (name1 == NULL || !name1[0]) 
+	if (name1 == NULL || !name1[0])
 		name1 = "main";
 
 	cs = (CONF_SECTION *)rad_malloc(sizeof(CONF_SECTION));
@@ -181,9 +181,9 @@ void cf_section_free(CONF_SECTION **cs)
 		}
 	}
 
-	if ((*cs)->name1) 
+	if ((*cs)->name1)
 		free((*cs)->name1);
-	if ((*cs)->name2) 
+	if ((*cs)->name2)
 		free((*cs)->name2);
 
 	/*
@@ -203,7 +203,7 @@ void cf_section_free(CONF_SECTION **cs)
 static void cf_item_add(CONF_SECTION *cs, CONF_ITEM *ci_new)
 {
 	CONF_ITEM *ci;
-	
+
 	for (ci = cs->children; ci && ci->next; ci = ci->next)
 		;
 
@@ -217,14 +217,24 @@ static void cf_item_add(CONF_SECTION *cs, CONF_ITEM *ci_new)
  *	Expand the variables in an input string.
  */
 static const char *cf_expand_variables(const char *cf, int *lineno,
-				       CONF_SECTION *cs,
+				       CONF_SECTION *outercs,
 				       char *output, const char *input)
 {
 	char *p;
 	const char *end, *ptr;
 	char name[8192];
-	CONF_PAIR *cpn;
-	CONF_SECTION *outercs;
+	CONF_SECTION *parentcs;
+
+	/*
+	 *	Find the master parent conf section.
+	 *	We can't use mainconfig.config, because we're in the
+	 *	process of re-building it, and it isn't set up yet...
+	 */
+	for (parentcs = outercs;
+	     parentcs->item.parent != NULL;
+	     parentcs = parentcs->item.parent) {
+		/* do nothing */
+	}
 
 	p = output;
 	ptr = input;
@@ -233,6 +243,10 @@ static const char *cf_expand_variables(const char *cf, int *lineno,
 		 *	Ignore anything other than "${"
 		 */
 		if ((*ptr == '$') && (ptr[1] == '{')) {
+			int up;
+			CONF_PAIR *cp;
+			CONF_SECTION *cs;
+
 			/*
 			 *	Look for trailing '}', and log a
 			 *	warning for anything that doesn't match,
@@ -245,35 +259,109 @@ static const char *cf_expand_variables(const char *cf, int *lineno,
 				       cf, *lineno);
 				return NULL;
 			}
-			
+
 			ptr += 2;
-			
-			memcpy(name, ptr, end - ptr);
-			name[end - ptr] = '\0';
-			
-			cpn = cf_pair_find(cs, name);
-			
+
+			cp = NULL;
+			up = 0;
+
 			/*
-			 *	Also look recursively up the section tree,
-			 *	so things like ${confdir} can be defined
-			 *	there and used inside the module config
-			 *	sections.
+			 *	${.foo} means "foo from the current section"
 			 */
-			for (outercs=cs->item.parent; 
-			     (cpn == NULL) && (outercs != NULL);
-			     outercs=outercs->item.parent) {
-				cpn = cf_pair_find(outercs, name);
+			if (*ptr == '.') {
+				up = 1;
+				cs = outercs;
+				ptr++;
+
+				/*
+				 *	${..foo} means "foo from the section
+				 *	enclosing this section" (etc.)
+				 */
+				while (*ptr == '.') {
+					if (cs->item.parent)
+						cs = cs->item.parent;
+					ptr++;
+				}
+
+			} else {
+				const char *q;
+				/*
+				 *	${foo} is local, with
+				 *	main as lower priority
+				 */
+				cs = outercs;
+
+				/*
+				 *	${foo.bar.baz} is always rooted
+				 *	from the top.
+				 */
+				for (q = ptr; *q && q != end; q++) {
+					if (*q == '.') {
+						cs = parentcs;
+						up = 1;
+						break;
+					}
+				}
 			}
-			if (!cpn) {
-				radlog(L_ERR, "%s[%d]: Unknown variable \"%s\"",
-				       cf, *lineno, name);
-				return NULL;
-			}
-			
+
+			while (cp == NULL) {
+				char *q;
+				/*
+				 *	Find the next section.
+				 */
+				for (q = name;
+				     (*ptr != 0) && (*ptr != '.') &&
+					     (ptr != end);
+				     q++, ptr++) {
+					*q = *ptr;
+				}
+				*q = '\0';
+
+				/*
+				 *	The character is a '.', find a
+				 *	section (as the user has given
+				 *	us a subsection to find)
+				 */
+				if (*ptr == '.') {
+					CONF_SECTION *next;
+
+					ptr++;	/* skip the period */
+
+					/*
+					 *	Find the sub-section.
+					 */
+					next = cf_section_sub_find(cs, name);
+					if (next == NULL) {
+						radlog(L_ERR, "config: No such section %s in variable %s", name, input);
+						return NULL;
+					}
+					cs = next;
+
+				} else { /* no period, must be a conf-part */
+					/*
+					 *	Find in the current referenced
+					 *	section.
+					 */
+					cp = cf_pair_find(cs, name);
+					if (cp == NULL) {
+						/*
+						 *	It it was NOT ${..foo}
+						 *	then look in the
+						 *	top-level config items.
+						 */
+						if (!up) cp = cf_pair_find(parentcs, name);
+					}
+					if (cp == NULL) {
+						radlog(L_ERR, "config: No such entry %s for string %s", name, input);
+						return NULL;
+					}
+				}
+			} /* until cp is non-NULL */
+
 			/*
 			 *  Substitute the value of the variable.
 			 */
-			strcpy(p, cpn->value);
+			strcpy(p, cp->value);
 			p += strlen(p);
 			ptr = end + 1;
 
@@ -294,10 +382,10 @@ static const char *cf_expand_variables(const char *cf, int *lineno,
 				       cf, *lineno);
 				return NULL;
 			}
-			
+
 			memcpy(name, ptr, end - ptr);
 			name[end - ptr] = '\0';
-			
+
 			/*
 			 *	Get the environment variable.
 			 *	If none exists, then make it an empty string.
@@ -319,7 +407,7 @@ static const char *cf_expand_variables(const char *cf, int *lineno,
 			*(p++) = *(ptr++);
 		}
 	} /* loop over all of the input string. */
-		
+
 	*p = '\0';
 
 	return output;
@@ -346,7 +434,9 @@ int cf_section_parse(CONF_SECTION *cs, void *base,
 	 */
 	for (i = 0; variables[i].name != NULL; i++) {
 		value = variables[i].dflt;
-		if (base) {
+		if (variables[i].data) {
+			data = variables[i].data; /* prefer this. */
+		} else if (base) {
 			data = ((char *)base) + variables[i].offset;
 		} else {
 			data = variables[i].data;
@@ -356,7 +446,7 @@ int cf_section_parse(CONF_SECTION *cs, void *base,
 		if (cp) {
 			value = cp->value;
 		}
-		
+
 		switch (variables[i].type)
 		{
 		case PW_TYPE_SUBSECTION:
@@ -408,7 +498,7 @@ int cf_section_parse(CONF_SECTION *cs, void *base,
 					variables[i].name,
 					*(int *)data);
 			break;
-			
+
 		case PW_TYPE_STRING_PTR:
 			q = (char **) data;
 			if (*q != NULL) {
@@ -455,14 +545,14 @@ int cf_section_parse(CONF_SECTION *cs, void *base,
 					value, ip_ntoa(buffer, ipaddr));
 			*(uint32_t *) data = ipaddr;
 			break;
-			
+
 		default:
 			radlog(L_ERR, "type %d not supported yet", variables[i].type);
 			return -1;
 			break;
 		} /* switch over variable type */
 	} /* for all variables in the configuration section */
-	
+
 	return 0;
 }
 
@@ -470,8 +560,8 @@ int cf_section_parse(CONF_SECTION *cs, void *base,
  *	Read a part of the config file.
  */
 static CONF_SECTION *cf_section_read(const char *cf, int *lineno, FILE *fp,
-		const char *name1, const char *name2,
-		CONF_SECTION *parent)
+				     const char *name1, const char *name2,
+				     CONF_SECTION *parent)
 {
 	CONF_SECTION *cs, *css;
 	CONF_PAIR *cpn;
@@ -484,7 +574,7 @@ static CONF_SECTION *cf_section_read(const char *cf, int *lineno, FILE *fp,
 	int t1, t2, t3;
 	char *cbuf = buf;
 	int len;
-	
+
 	/*
 	 *	Ensure that the user can't add CONF_SECTIONs
 	 *	with 'internal' names;
@@ -562,7 +652,7 @@ static CONF_SECTION *cf_section_read(const char *cf, int *lineno, FILE *fp,
 		/*
 		 *	Allow for $INCLUDE files
 		 *
-		 *      This *SHOULD* work for any level include.  
+		 *      This *SHOULD* work for any level include.
 		 *      I really really really hate this file.  -cparker
 		 */
 		if (strcasecmp(buf1, "$INCLUDE") == 0) {
@@ -590,7 +680,7 @@ static CONF_SECTION *cf_section_read(const char *cf, int *lineno, FILE *fp,
 			if (is != NULL) {
 				if (is->children != NULL) {
 					CONF_ITEM *ci;
-			
+
 					/*
 					 *	Re-write the parent of the
 					 *	moved children to be the
@@ -648,7 +738,7 @@ static CONF_SECTION *cf_section_read(const char *cf, int *lineno, FILE *fp,
 		 */
 		if (t2 == T_LCBRACE || t3 == T_LCBRACE) {
 			css = cf_section_read(cf, lineno, fp, buf1,
-					t2==T_LCBRACE ? NULL : buf2, cs);
+					      t2==T_LCBRACE ? NULL : buf2, cs);
 			if (css == NULL) {
 				cf_section_free(&cs);
 				return NULL;
@@ -661,7 +751,7 @@ static CONF_SECTION *cf_section_read(const char *cf, int *lineno, FILE *fp,
 		/*
 		 *	Ignore semi-colons.
 		 */
-		if (*buf2 == ';') 
+		if (*buf2 == ';')
 			*buf2 = '\0';
 
 		/*
@@ -669,7 +759,7 @@ static CONF_SECTION *cf_section_read(const char *cf, int *lineno, FILE *fp,
 		 */
 		if (buf1[0] != 0 && buf2[0] == 0 && buf3[0] == 0) {
 			t2 = T_OP_EQ;
-		} else if (buf1[0] == 0 || buf2[0] == 0 || 
+		} else if (buf1[0] == 0 || buf2[0] == 0 ||
 			   (t2 < T_EQSTART || t2 > T_EQEND)) {
 			radlog(L_ERR, "%s[%d]: Line is not in 'attribute = value' format",
 					cf, *lineno);
@@ -721,13 +811,13 @@ static CONF_SECTION *cf_section_read(const char *cf, int *lineno, FILE *fp,
 /*
  *	Read the config file.
  */
-CONF_SECTION *conf_read(const char *fromfile, int fromline, 
+CONF_SECTION *conf_read(const char *fromfile, int fromline,
 			const char *conffile, CONF_SECTION *parent)
 {
 	FILE		*fp;
 	int		lineno = 0;
 	CONF_SECTION	*cs;
-	
+
 	if ((fp = fopen(conffile, "r")) == NULL) {
 		if (fromfile) {
 			radlog(L_ERR|L_CONS, "%s[%d]: Unable to open file \"%s\": %s",
@@ -751,7 +841,7 @@ CONF_SECTION *conf_read(const char *fromfile, int fromline,
 }
 
 
-/* 
+/*
  * Return a CONF_PAIR within a CONF_SECTION.
  */
 CONF_PAIR *cf_pair_find(CONF_SECTION *section, const char *name)
@@ -808,7 +898,7 @@ char *cf_section_name2(CONF_SECTION *section)
 	return (section ? section->name2 : NULL);
 }
 
-/* 
+/*
  * Find a value in a CONF_SECTION
  */
 char *cf_section_value_find(CONF_SECTION *section, const char *attr)
@@ -908,7 +998,7 @@ CONF_SECTION *cf_subsection_find_next(CONF_SECTION *section,
 	for (; ci; ci = ci->next) {
 		if (ci->type != CONF_ITEM_SECTION)
 			continue;
-		if ((name1 == NULL) || 
+		if ((name1 == NULL) ||
 				(strcmp(cf_itemtosection(ci)->name1, name1) == 0))
 			break;
 	}
@@ -951,9 +1041,9 @@ int cf_item_is_section(CONF_ITEM *item)
 
 
 #if 0
-/* 
+/*
  * JMG dump_config tries to dump the config structure in a readable format
- * 
+ *
 */
 
 static int dump_config_section(CONF_SECTION *cs, int indent)
