@@ -1,7 +1,7 @@
 /*
  * exec.c	Execute external programs.
  *
- * Version:	$Id: exec.c,v 1.37.2.1 2003/08/25 19:07:03 aland Exp $
+ * Version:	$Id: exec.c,v 1.42 2004/02/26 19:04:22 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  * Copyright 2000  The FreeRADIUS server project
  * Copyright 2000  Michael J. Hartwick <hartwick@hartwick.com>
  */
-static const char rcsid[] = "$Id: exec.c,v 1.37.2.1 2003/08/25 19:07:03 aland Exp $";
+static const char rcsid[] = "$Id: exec.c,v 1.42 2004/02/26 19:04:22 aland Exp $";
 
 #include "autoconf.h"
 
@@ -32,7 +32,7 @@ static const char rcsid[] = "$Id: exec.c,v 1.37.2.1 2003/08/25 19:07:03 aland Ex
 #include <ctype.h>
 #include <signal.h>
 
-#if HAVE_SYS_WAIT_H
+#ifdef HAVE_SYS_WAIT_H
 #	include <sys/wait.h>
 #endif
 #ifndef WEXITSTATUS
@@ -97,12 +97,12 @@ int radius_exec_program(const char *cmd, REQUEST *request,
 	 */
 	radius_xlat(answer, sizeof(answer), cmd, request, NULL);
 	buf = answer;
-	
+
 	/*
 	 *	Log the command if we are debugging something
 	 */
 	DEBUG("Exec-Program: %s", buf);
-	
+
 	/*
 	 *	Build vector list of arguments and execute.
 	 *
@@ -133,7 +133,7 @@ int radius_exec_program(const char *cmd, REQUEST *request,
 		int envlen;
 		char buffer[1024];
 
-		/*	
+		/*
 		 *	Child process.
 		 *
 		 *	We try to be fail-safe here.  So if ANYTHING
@@ -165,7 +165,7 @@ int radius_exec_program(const char *cmd, REQUEST *request,
 				       strerror(errno));
 				exit(1);
 			}
-			
+
 			/*
 			 *	pd[1] is the FD that the child will write to,
 			 *	so we make it STDOUT.
@@ -209,7 +209,7 @@ int radius_exec_program(const char *cmd, REQUEST *request,
 		 */
 		envlen = 0;
 
-		for (vp = input_pairs; vp && vp->next; vp = vp->next) {
+		for (vp = input_pairs; vp != NULL; vp = vp->next) {
 			/*
 			 *	Hmm... maybe we shouldn't pass the
 			 *	user's password in an environment
@@ -317,7 +317,7 @@ int radius_exec_program(const char *cmd, REQUEST *request,
 	 *	Parse the output, if any.
 	 */
 	if (done) {
-		n = -1;
+		n = T_INVALID;
 		if (output_pairs) {
 			/*
 			 *	For backwards compatibility, first check
