@@ -1,7 +1,7 @@
 /*
  * log.c	Logging module.
  *
- * Version:	$Id: log.c,v 1.34.2.1 2002/12/09 18:47:06 aland Exp $
+ * Version:	$Id: log.c,v 1.41 2004/02/26 19:04:23 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  * Copyright 2001  Chad Miller <cmiller@surfsouth.com>
  */
 
-static const char rcsid[] = "$Id: log.c,v 1.34.2.1 2002/12/09 18:47:06 aland Exp $";
+static const char rcsid[] = "$Id: log.c,v 1.41 2004/02/26 19:04:23 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -37,10 +37,22 @@ static const char rcsid[] = "$Id: log.c,v 1.34.2.1 2002/12/09 18:47:06 aland Exp
 
 #include "radiusd.h"
 
-#if HAVE_SYSLOG_H
+#ifdef HAVE_SYSLOG_H
 #	include <syslog.h>
 #endif
 
+ /*
+ * Logging facility names
+ */
+static LRAD_NAME_NUMBER levels[] = {
+	{ ": Debug: ",          L_DBG   },
+	{ ": Auth: ",           L_AUTH  },
+	{ ": Proxy: ",          L_PROXY },
+	{ ": Info: ",           L_INFO  },
+	{ ": Error: ",          L_ERR   },
+	{ NULL, 0 }
+};
+
 /*
  *	Log the message to the logfile. Include the severity and
  *	a time stamp.
@@ -61,7 +73,15 @@ int vradlog(int lvl, const char *fmt, va_list ap)
 		return 0;
 	}
 
-	if (debug_flag 
+	/*
+	 *	If we don't want any messages, then
+	 *	throw them away.
+	 */
+	if (radlog_dest == RADLOG_NULL) {
+		return 0;
+	}
+
+	if (debug_flag
 	    || (radlog_dest == RADLOG_STDOUT)
 	    || (radlog_dir == NULL)) {
 	        msgfd = stdout;
@@ -70,49 +90,41 @@ int vradlog(int lvl, const char *fmt, va_list ap)
 	        msgfd = stderr;
 
 	} else if (radlog_dest != RADLOG_SYSLOG) {
+		/*
+		 *	No log file set.  It must go to stdout.
+		 */
+		if (!mainconfig.log_file) {
+			msgfd = stdout;
 
-		if ((msgfd = fopen(mainconfig.log_file, "a")) == NULL) {
+			/*
+			 *	Else try to open the file.
+			 */
+		} else if ((msgfd = fopen(mainconfig.log_file, "a")) == NULL) {
 		         fprintf(stderr, "%s: Couldn't open %s for logging: %s\n",
 				 progname, mainconfig.log_file, strerror(errno));
-				
+
 			 fprintf(stderr, "  (");
 			 vfprintf(stderr, fmt, ap);  /* the message that caused the log */
 			 fprintf(stderr, ")\n");
 			 return -1;
 		}
-	
 	}
 
-#if HAVE_SYSLOG_H
+#ifdef HAVE_SYSLOG_H
 	if (radlog_dest == RADLOG_SYSLOG) {
 		*buffer = '\0';
 		len = 0;
 	} else
 #endif
 	{
-		const char *s = ": ";
+		const char *s;
 		time_t timeval;
 
 		timeval = time(NULL);
-		ctime_r(&timeval, buffer);
+		CTIME_R(&timeval, buffer, 8192);
+
+		s = lrad_int2str(levels, (lvl & ~L_CONS), ": ");
 
-		switch(lvl & ~L_CONS) {
-			case L_DBG:
-				s = ": Debug: ";
-				break;
-			case L_AUTH:
-				s = ": Auth: ";
-				break;
-			case L_PROXY:
-				s = ": Proxy: ";
-				break;
-			case L_INFO:
-				s = ": Info: ";
-				break;
-			case L_ERR:
-				s = ": Error: ";
-				break;
-		}
 		strcat(buffer, s);
 		len = strlen(buffer);
 	}
@@ -136,7 +148,7 @@ int vradlog(int lvl, const char *fmt, va_list ap)
 			*p = '?';
 	}
 	strcat(buffer, "\n");
-	
+
 	/*
 	 *   If we're debugging, for small values of debug, then
 	 *   we don't do timestamps.
@@ -152,7 +164,7 @@ int vradlog(int lvl, const char *fmt, va_list ap)
 		p = buffer;
 	}
 
-#if HAVE_SYSLOG_H
+#ifdef HAVE_SYSLOG_H
 	if (radlog_dest != RADLOG_SYSLOG)
 #endif
 	{
@@ -165,7 +177,7 @@ int vradlog(int lvl, const char *fmt, va_list ap)
 			fclose(msgfd);
 		}
 	}
-#if HAVE_SYSLOG_H
+#ifdef HAVE_SYSLOG_H
 	else {			/* it was syslog */
 		switch(lvl & ~L_CONS) {
 			case L_DBG:
@@ -218,16 +230,16 @@ int radlog(int lvl, const char *msg, ...)
 
 /*
  *      Dump a whole list of attributes to DEBUG2
- */                     
+ */
 void vp_listdebug(VALUE_PAIR *vp)
-{       
-        char tmpPair[70];  
+{
+        char tmpPair[70];
         for (; vp; vp = vp->next) {
                 vp_prints(tmpPair, sizeof(tmpPair), vp);
                 DEBUG2("     %s", tmpPair);
-        }               
+        }
 }
-   
 
- 
-        
+
+
+
