@@ -1,7 +1,7 @@
 /*
  * mainconf.c	Handle the server's configuration.
  *
- * Version:	$Id: mainconfig.c,v 1.36.2.2 2004/06/08 15:09:40 aland Exp $
+ * Version:	$Id: mainconfig.c,v 1.36.2.3 2004/09/30 14:49:55 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -50,6 +50,11 @@
 #include <pwd.h>
 
 
+#ifdef WITH_UDPFROMTO
+#include "udpfromto.h"
+#endif
+
+
 struct main_config_t mainconfig;
 
 /*
@@ -846,6 +851,16 @@ static int listen_bind(rad_listen_t *this)
 		return -1;
 	}
 	
+
+#ifdef WITH_UDPFROMTO
+	/*
+	 *	Initialize udpfromto for all sockets.
+	 */
+	if (udpfromto_init(this->fd) != 0) {
+		radlog(L_ERR|L_CONS, "ERROR: udpfromto init failed.");
+	}
+#endif
+
 	sa = (struct sockaddr_in *) &salocal;
 	memset ((char *) sa, '\0', sizeof(salocal));
 	sa->sin_family = AF_INET;
