@@ -1,7 +1,7 @@
 /*
  * proxy.c	Proxy stuff.
  *
- * Version:	$Id: proxy.c,v 1.65.2.1 2003/08/26 12:30:00 phampson Exp $
+ * Version:	$Id: proxy.c,v 1.77 2004/05/28 21:45:07 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,14 +22,14 @@
  * Copyright 2000  Chris Parker <cparker@starnetusa.com>
  */
 
-static const char rcsid[] = "$Id: proxy.c,v 1.65.2.1 2003/08/26 12:30:00 phampson Exp $";
+static const char rcsid[] = "$Id: proxy.c,v 1.77 2004/05/28 21:45:07 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
 
 #include <sys/socket.h>
 
-#if HAVE_NETINET_IN_H
+#ifdef HAVE_NETINET_IN_H
 #	include <netinet/in.h>
 #endif
 
@@ -41,8 +41,7 @@ static const char rcsid[] = "$Id: proxy.c,v 1.65.2.1 2003/08/26 12:30:00 phampso
 #include "radiusd.h"
 #include "rad_assert.h"
 #include "modules.h"
-
-static uint32_t proxy_id = 1;
+#include "request_list.h"
 
 /*
  *	We received a response from a remote radius server.
@@ -53,26 +52,8 @@ static uint32_t proxy_id = 1;
  */
 int proxy_receive(REQUEST *request)
 {
-	VALUE_PAIR *proxypair;
-	VALUE_PAIR *replicatepair;
         int rcode;
 
-	proxypair = pairfind(request->config_items, PW_PROXY_TO_REALM);
-	replicatepair = pairfind(request->config_items, PW_REPLICATE_TO_REALM);
-        if (proxypair) {
-            /* Don't do anything*/
-
-        } else if (replicatepair) {
-            /*
-             *  The request was replicated, so we don't process the response.
-             */
-            return RLM_MODULE_HANDLED;
-
-        } else {
-		radlog(L_PROXY, "Proxy reply to packet with no Realm");
-		return RLM_MODULE_FAIL;
-	}
-
         /*
          *	Delete any reply we had accumulated until now.
 	 */
@@ -137,11 +118,11 @@ static void proxy_addinfo(REQUEST *request)
 static REALM *proxy_realm_ldb(REQUEST *request, const char *realm_name,
 			      int accounting)
 {
-	REALM	*cl;
-	REALM	*array[32];	/* maximum number of load balancing realms */
-	int	size;
+	REALM		*cl, *lb;
+	uint32_t	count;
 
-	size = 0;
+	lb = NULL;
+	count = 0;
 	for (cl = mainconfig.realms; cl; cl = cl->next) {
 		/*
 		 *	Wake up any sleeping realm.
@@ -176,43 +157,44 @@ static REALM *proxy_realm_ldb(REQUEST *request, const char *realm_name,
 		/*
 		 *	Fail-over, pick the first one that matches.
 		 */
-		if ((size == 0) && /* if size > 0, we have round-robin */
+		if ((count == 0) && /* if size > 0, we have round-robin */
 		    (cl->ldflag == 0)) {
 			return cl;
 		}
 
 		/*
-		 *	Else we're doing round-robin.
+		 *	We're doing load-balancing.  Pick a random
+		 *	number, which will be used to determine which
+		 *	home server is chosen.
 		 */
-		array[size] = cl;
-		size++;
+		if (!lb) {
+			lb = cl;
+			count = 1;
+			continue;
+		}
 
 		/*
-		 *	If too many realms are found, then limit
-		 *	the number to 32.
+		 *	Keep track of how many load balancing servers
+		 *	we've gone through.
 		 */
-		if (size >= 32) {
-			break;
+		count++;
+
+		/*
+		 *	See the "camel book" for why this works.
+		 *
+		 *	If (rand(0..n) < 1), pick the current realm.
+		 *	We add a scale factor of 65536, to avoid
+		 *	floating point.
+		 */
+		if ((count * (lrad_rand() & 0xffff)) < (uint32_t) 0x10000) {
+			lb = cl;
 		}
-	}
+	} /* loop over the realms */
 
 	/*
-	 *	WTF?  We didn't find a live realm!
-	 *
-	 *	This means that 'rlm_realm' found a live realm for
-	 *	us, and then it was marked dead before code execution
-	 *	got here.  The only thing to do is to dump the request.
+	 *	Return the load-balanced realm.
 	 */
-	if (size == 0) {
-		return NULL;
-	}
-
-	/*
-	 *	Pick a random realm.  It's not true round-robin,
-	 *	but over a large number of requests, it statistically
-	 *	distributes the requests evenly among the realms.
-	 */
-	return array[(int) (lrad_rand() % size)];
+	return lb;
 }
 
 /*
@@ -226,22 +208,20 @@ static REALM *proxy_realm_ldb(REQUEST *request, const char *realm_name,
 int proxy_send(REQUEST *request)
 {
 	int rcode;
-	VALUE_PAIR *proxypair;
-	VALUE_PAIR *replicatepair;
 	VALUE_PAIR *realmpair;
-	VALUE_PAIR *namepair;
-	VALUE_PAIR *strippednamepair;
+	VALUE_PAIR *strippedname;
 	VALUE_PAIR *delaypair;
-	VALUE_PAIR *vp, *vps;
+	VALUE_PAIR *vp;
 	REALM *realm;
 	char *realmname;
-	int replicating;
 
 	/*
 	 *	Not authentication or accounting.  Stop it.
 	 */
 	if ((request->packet->code != PW_AUTHENTICATION_REQUEST) &&
 	    (request->packet->code != PW_ACCOUNTING_REQUEST)) {
+		DEBUG2("  ERROR: Cannot proxy packets of type %d",
+		       request->packet->code);
 		return RLM_MODULE_FAIL;
 	}
 
@@ -252,23 +232,11 @@ int proxy_send(REQUEST *request)
 	 *	count has been exceeded.  Until then, it should not
 	 *	be eligible for the time-based cleanup.  --Pac. */
 
-	/* Look for proxy/replicate signs */
-	/* FIXME - What to do if multiple Proxy-To/Replicate-To attrs are
-	 * set...  Log an error? Actually replicate to multiple places? That
-	 * would be cool. For now though, I'll just take the first one and
-	 * ignore the rest. */
-	proxypair = pairfind(request->config_items, PW_PROXY_TO_REALM);
-	replicatepair = pairfind(request->config_items, PW_REPLICATE_TO_REALM);
-	if (proxypair) {
-		realmpair = proxypair;
-		replicating = 0;
-	} else if (replicatepair) {
-		realmpair = replicatepair;
-		replicating = 1;
-	} else {
+	realmpair = pairfind(request->config_items, PW_PROXY_TO_REALM);
+	if (!realmpair) {
 		/*
-		 *	Neither proxy or replicate attributes are set,
-		 *	so we can exit from the proxy code.
+		 *	Not proxying, so we can exit from the proxy
+		 *	code.
 		 */
 		return RLM_MODULE_NOOP;
 	}
@@ -303,6 +271,8 @@ int proxy_send(REQUEST *request)
 	realm = proxy_realm_ldb(request, realmname,
 				(request->packet->code == PW_ACCOUNTING_REQUEST));
 	if (realm == NULL) {
+		DEBUG2("  ERROR: Failed to find live home server for realm %s",
+		       realmname);
 		return RLM_MODULE_FAIL;
 	}
 
@@ -311,77 +281,127 @@ int proxy_send(REQUEST *request)
 	 */
 	pairadd(&request->packet->vps,
 		pairmake("Realm", realm->realm, T_OP_EQ));
-	
+
 	/*
 	 *	Access-Request: look for LOCAL realm.
 	 *	Accounting-Request: look for LOCAL realm.
 	 */
 	if (((request->packet->code == PW_AUTHENTICATION_REQUEST) &&
 	     (realm->ipaddr == htonl(INADDR_NONE))) ||
-	    ((request->packet->code == PW_ACCOUNTING_REQUEST) &&	    
+	    ((request->packet->code == PW_ACCOUNTING_REQUEST) &&
 	     (realm->acct_ipaddr == htonl(INADDR_NONE)))) {
+		DEBUG2(" WARNING: Cancelling proxy to Realm %s, as the realm is local.",
+		       realm->realm);
 		return RLM_MODULE_NOOP;
 	}
+
+	/*
+	 *	Allocate the proxy packet, only if it wasn't already
+	 *	allocated by a module.  This check is mainly to support
+	 *	the proxying of EAP-TTLS and EAP-PEAP tunneled requests.
+	 *
+	 *	In those cases, the EAP module creates a "fake"
+	 *	request, and recursively passes it through the
+	 *	authentication stage of the server.  The module then
+	 *	checks if the request was supposed to be proxied, and
+	 *	if so, creates a proxy packet from the TUNNELED request,
+	 *	and not from the EAP request outside of the tunnel.
+	 *
+	 *	The proxy then works like normal, except that the response
+	 *	packet is "eaten" by the EAP module, and encapsulated into
+	 *	an EAP packet.
+	 */
+	if (!request->proxy) {
+		/*
+		 *	Now build a new RADIUS_PACKET.
+		 *
+		 *	FIXME: it could be that the id wraps around
+		 *	too fast if we have a lot of requests, it
+		 *	might be better to keep a seperate ID value
+		 *	per remote server.
+		 *
+		 *	OTOH the remote radius server should be smart
+		 *	enough to compare _both_ ID and vector.
+		 *	Right?
+		 */
+		if ((request->proxy = rad_alloc(TRUE)) == NULL) {
+			radlog(L_ERR|L_CONS, "no memory");
+			exit(1);
+		}
+
+		/*
+		 *	We now massage the attributes to be proxied...
+		 */
+
+		/*
+		 *	Copy the request, then look up name and
+		 *	plain-text password in the copy.
+		 *
+		 *	Note that the User-Name attribute is the
+		 *	*original* as sent over by the client.  The
+		 *	Stripped-User-Name attribute is the one hacked
+		 *	through the 'hints' file.
+		 */
+		request->proxy->vps =  paircopy(request->packet->vps);
+	}
+
+	/*
+	 *	Strip the name, if told to.
+	 *
+	 *	Doing it here catches the case of proxied tunneled
+	 *	requests.
+	 */
+	if ((strippedname = pairfind(request->proxy->vps, PW_STRIPPED_USER_NAME)) != NULL) {
+		/*
+		 *	If there's a Stripped-User-Name attribute in
+		 *	the request, then use THAT as the User-Name
+		 *	for the proxied request, instead of the
+		 *	original name.
+		 *
+		 *	This is done by making a copy of the
+		 *	Stripped-User-Name attribute, turning it into
+		 *	a User-Name attribute, deleting the
+		 *	Stripped-User-Name and User-Name attributes
+		 *	from the vps list, and making the new
+		 *	User-Name the head of the vps list.
+		 */
+		vp = pairfind(request->proxy->vps, PW_USER_NAME);
+		if (!vp) {
+			vp = paircreate(PW_USER_NAME, PW_TYPE_STRING);
+			if (!vp) {
+				radlog(L_ERR|L_CONS, "no memory");
+				exit(1);
+			}
+			vp->next = request->proxy->vps;
+			request->proxy->vps = vp;
+		}
+		memcpy(vp->strvalue, strippedname->strvalue,
+		       sizeof(vp->strvalue));
+		vp->length = strippedname->length;
+
+		/*
+		 *	Do NOT delete Stripped-User-Name.
+		 */
+	}
 	
 	/*
-	 *	Copy the request, then look up
-	 *	name and plain-text password in the copy.
-	 *
-	 *	Note that the User-Name attribute is the *original*
-	 *	as sent over by the client.  The Stripped-User-Name
-	 *	attribute is the one hacked through the 'hints' file.
+	 *	If there is no PW_CHAP_CHALLENGE attribute but
+	 *	there is a PW_CHAP_PASSWORD we need to add it
+	 *	since we can't use the request authenticator
+	 *	anymore - we changed it.
 	 */
-	vps = paircopy(request->packet->vps);
-	namepair = pairfind(vps, PW_USER_NAME);
-	strippednamepair = pairfind(vps, PW_STRIPPED_USER_NAME);
-
-	/*
-	 *	If there's a Stripped-User-Name attribute in the
-	 *	request, then use THAT as the User-Name for the
-	 *	proxied request, instead of the original name.
-	 *
-	 *	This is done by making a copy of the Stripped-User-Name
-	 *	attribute, turning it into a User-Name attribute,
-	 *	deleting the Stripped-User-Name and User-Name attributes
-	 *	from the vps list, and making the new User-Name
-	 *	the head of the vps list.
-	 */
-	if (strippednamepair) {
-		vp = paircreate(PW_USER_NAME, PW_TYPE_STRING);
+	if (pairfind(request->proxy->vps, PW_CHAP_PASSWORD) &&
+	    pairfind(request->proxy->vps, PW_CHAP_CHALLENGE) == NULL) {
+		vp = paircreate(PW_CHAP_CHALLENGE, PW_TYPE_STRING);
 		if (!vp) {
 			radlog(L_ERR|L_CONS, "no memory");
 			exit(1);
 		}
-		memcpy(vp->strvalue, strippednamepair->strvalue,
-				sizeof(vp->strvalue));
-		vp->length = strippednamepair->length;
-		pairdelete(&vps, PW_USER_NAME);
-		pairdelete(&vps, PW_STRIPPED_USER_NAME);
-		vp->next = vps;
-		namepair = vp;
-		vps = vp;
+		vp->length = AUTH_VECTOR_LEN;
+		memcpy(vp->strvalue, request->packet->vector, AUTH_VECTOR_LEN);
+		pairadd(&(request->proxy->vps), vp);
 	}
 
-	/*
-	 *	Now build a new RADIUS_PACKET and send it.
-	 *
-	 *	FIXME: it could be that the id wraps around too fast if
-	 *	we have a lot of requests, it might be better to keep
-	 *	a seperate ID value per remote server.
-	 *
-	 *	OTOH the remote radius server should be smart enough to
-	 *	compare _both_ ID and vector. Right ?
-	 */
-	if ((request->proxy = rad_alloc(TRUE)) == NULL) {
-		radlog(L_ERR|L_CONS, "no memory");
-		exit(1);
-	}
-
-	/*
-	 *	Proxied requests get sent out the proxy FD ONLY.
-	 */
-	request->proxy->sockfd = proxyfd;
-
 	request->proxy->code = request->packet->code;
 	if (request->packet->code == PW_AUTHENTICATION_REQUEST) {
 		request->proxy->dst_port = realm->auth_port;
@@ -390,48 +410,23 @@ int proxy_send(REQUEST *request)
 		request->proxy->dst_port = realm->acct_port;
 		request->proxy->dst_ipaddr = realm->acct_ipaddr;
 	}
-	rad_assert(request->proxy->vps == NULL);
-	request->proxy->vps = vps;
 
 	/*
-	 *	Add the request to the list of outstanding requests.
-	 *	Note that request->proxy->id is a 16 bits value,
-	 *	while rad_send sends only the 8 least significant
-	 *	bits of that same value.
-	 */
-	request->proxy->id = (proxy_id++) & 0xff;
-	proxy_id &= 0xffff;
-
-	/*
-	 *	Add PROXY_STATE attribute.
+	 *	Add PROXY_STATE attribute, before pre-proxy stage,
+	 *	so the pre-proxy modules have access to it.
+	 *
+	 *	Note that, at this point, the proxied request HAS NOT
+	 *	been assigned a RADIUS Id.
 	 */
 	proxy_addinfo(request);
 
-	/*
-	 *	If there is no PW_CHAP_CHALLENGE attribute but there
-	 *	is a PW_CHAP_PASSWORD we need to add it since we can't
-	 *	use the request authenticator anymore - we changed it.
-	 */
-	if (pairfind(vps, PW_CHAP_PASSWORD) &&
-	    pairfind(vps, PW_CHAP_CHALLENGE) == NULL) {
-		vp = paircreate(PW_CHAP_CHALLENGE, PW_TYPE_STRING);
-		if (!vp) {
-			radlog(L_ERR|L_CONS, "no memory");
-			exit(1);
-		}
-		vp->length = AUTH_VECTOR_LEN;
-		memcpy(vp->strvalue, request->packet->vector, AUTH_VECTOR_LEN);
-		pairadd(&vps, vp);
-	}
-
 	/*
 	 *	Set up for sending the request.
 	 */
 	memcpy(request->proxysecret, realm->secret, sizeof(request->proxysecret));
-	request->proxy_is_replicate = replicating;
 	request->proxy_try_count = mainconfig.proxy_retry_count - 1;
 	request->proxy_next_try = request->timestamp + mainconfig.proxy_retry_delay;
-	delaypair = pairfind(vps, PW_ACCT_DELAY_TIME);
+	delaypair = pairfind(request->proxy->vps, PW_ACCT_DELAY_TIME);
 	request->proxy->timestamp = request->timestamp - (delaypair ? delaypair->lvalue : 0);
 
 	/*
@@ -440,8 +435,8 @@ int proxy_send(REQUEST *request)
 	rcode = module_pre_proxy(request);
 
 	/*
-	 *	Do NOT free proxy->vps, the pairs are needed for the
-	 *	retries! --Pac.
+	 *	Do NOT free request->proxy->vps, the pairs are needed
+	 *	for the retries! --Pac.
 	 */
 
 	/*
@@ -457,12 +452,32 @@ int proxy_send(REQUEST *request)
 	if ((rcode == RLM_MODULE_OK) ||
 	    (rcode == RLM_MODULE_NOOP) ||
 	    (rcode == RLM_MODULE_UPDATED)) {
-		rad_send(request->proxy, NULL, (char *)request->proxysecret);
-		if (!replicating) {
-			rcode = RLM_MODULE_HANDLED; /* caller doesn't reply */
-		} else {
-			rcode = RLM_MODULE_NOOP; /* caller replies */
+		request->options |= RAD_REQUEST_OPTION_PROXIED;
+
+		/*
+		 *	IF it's a fake request, don't send the proxy
+		 *	packet.  The outer tunnel session will take
+		 *	care of doing that.
+		 */
+		if ((request->options & RAD_REQUEST_OPTION_FAKE_REQUEST) == 0) {
+			/*
+			 *	Add the proxied request to the
+			 *	list of outstanding proxied
+			 *	requests, BEFORE we send it, so
+			 *	we have fewer problems with race
+			 *	conditions when the responses come
+			 *	back very quickly.
+			 */
+			if (!rl_add_proxy(request)) {
+				DEBUG("ERROR: Failed to proxy request %d",
+				      request->number);
+				return RLM_MODULE_FAIL; /* caller doesn't reply */
+			}
+
+			rad_send(request->proxy, NULL,
+				 (char *)request->proxysecret);
 		}
+		rcode = RLM_MODULE_HANDLED; /* caller doesn't reply */
 	} else {
 		rcode = RLM_MODULE_FAIL; /* caller doesn't reply */
 	}
