@@ -1,7 +1,7 @@
-/* 
+/*
  * radius_snmp.c	Radius SNMP support
  *
- * Version:	$Id: radius_snmp.c,v 1.14 2001/05/04 16:58:04 aland Exp $
+ * Version:	$Id: radius_snmp.c,v 1.17 2004/02/26 19:04:24 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 static const char rcsid[] =
-"$Id: radius_snmp.c,v 1.14 2001/05/04 16:58:04 aland Exp $";
+"$Id: radius_snmp.c,v 1.17 2004/02/26 19:04:24 aland Exp $";
 
 #include "autoconf.h"
 
@@ -30,7 +30,7 @@ static const char rcsid[] =
 
 #include "libradius.h"
 
-#if HAVE_NETINET_IN_H
+#ifdef HAVE_NETINET_IN_H
 #	include <netinet/in.h>
 #endif
 
@@ -40,7 +40,6 @@ static const char rcsid[] =
 #include "radiusd.h"
 #include "conffile.h"
 
-extern RADCLIENT *clients;
 extern int need_reload;
 
 /*
@@ -142,7 +141,7 @@ static const unsigned char *radAuthEntry(struct variable *vp,
 	size_t  *var_len,
 	WriteMethod **write_method);
 
-static struct variable radiusacc_variables[] = 
+static struct variable radiusacc_variables[] =
 {
 	{RADIUSACCSERVIDENT, STRING, RONLY, radAccServ, 1, {1}},
 	{RADIUSACCSERVUPTIME, TIMETICKS, RONLY, radAccServ, 1, {2}},
@@ -207,7 +206,7 @@ get_client(struct variable *v, oid objid[], size_t *objid_len, int exact) {
 
 	len = *objid_len - v->namelen;
 
-	if (!clients)
+	if (!mainconfig.clients)
 		return NULL;
 
 	if (exact) {
@@ -218,7 +217,7 @@ get_client(struct variable *v, oid objid[], size_t *objid_len, int exact) {
 			return NULL;
 
 		i = objid[v->namelen]-1;
-		c = clients;
+		c = mainconfig.clients;
 		while (i && c)
 		{
 			c = c->next;
@@ -232,9 +231,9 @@ get_client(struct variable *v, oid objid[], size_t *objid_len, int exact) {
 	*objid_len = v->namelen + 1;
 	if (!len || (objid[v->namelen] == 0)) {
 		objid[v->namelen]=1;
-		return clients;
+		return mainconfig.clients;
 	}
-	c = clients->next;
+	c = mainconfig.clients->next;
 	while (i && c) {
 		c = c->next;
 		i--;
@@ -247,7 +246,7 @@ get_client(struct variable *v, oid objid[], size_t *objid_len, int exact) {
 }
 
 static int
-radServReset (int action, u_char *var_val, u_char var_val_type, 
+radServReset (int action, u_char *var_val, u_char var_val_type,
 		size_t var_val_len, const unsigned char *statP, oid *name,
 		size_t name_len) {
 
@@ -256,13 +255,13 @@ radServReset (int action, u_char *var_val, u_char var_val_type,
 
 	switch (action) {
 		case RESERVE1:
-			if (var_val_type != INTEGER) 
+			if (var_val_type != INTEGER)
 				return SNMP_ERR_WRONGTYPE;
-			if (var_val_len != sizeof (long)) 
+			if (var_val_len != sizeof (long))
 				return SNMP_ERR_WRONGLENGTH;
 			if (! asn_parse_int(var_val, &big, &var_val_type, &i, sizeof(long)))
 				return SNMP_ERR_WRONGENCODING;
-			if (i != 2) 
+			if (i != 2)
 				return SNMP_ERR_WRONGVALUE;
 			break;
 		case COMMIT:
@@ -277,7 +276,7 @@ radServReset (int action, u_char *var_val, u_char var_val_type,
 }
 
 static const unsigned char *
-radAccServ(struct variable *vp, oid *name, size_t *length, int exact, 
+radAccServ(struct variable *vp, oid *name, size_t *length, int exact,
 		size_t *var_len, WriteMethod **write_method) {
 
 	static int result;
@@ -351,7 +350,7 @@ radAccServ(struct variable *vp, oid *name, size_t *length, int exact,
 }
 
 static const unsigned char *
-radAccEntry(struct variable *vp, oid *name, size_t *length, int exact, 
+radAccEntry(struct variable *vp, oid *name, size_t *length, int exact,
 		size_t *var_len, WriteMethod **write_method) {
 
 	RADCLIENT *c;
@@ -405,7 +404,7 @@ radAccEntry(struct variable *vp, oid *name, size_t *length, int exact,
 }
 
 static const unsigned char *
-radAuthServ(struct variable *vp, oid *name, size_t *length, int exact, 
+radAuthServ(struct variable *vp, oid *name, size_t *length, int exact,
 		size_t *var_len, WriteMethod **write_method) {
 
 	static int result;
@@ -485,7 +484,7 @@ radAuthServ(struct variable *vp, oid *name, size_t *length, int exact,
 }
 
 static const unsigned char *
-radAuthEntry(struct variable *vp, oid	 *name, size_t *length, int exact, 
+radAuthEntry(struct variable *vp, oid	 *name, size_t *length, int exact,
 		size_t *var_len, WriteMethod **write_method) {
 
 	RADCLIENT *c;
@@ -583,7 +582,7 @@ radius_snmp_init (void) {
 	 *  Parse the SNMP configuration information.
 	 */
 	cs = cf_section_find(NULL);
-	if (cs != NULL) 
+	if (cs != NULL)
 		cf_section_parse(cs, NULL, snmp_config);
 
 	/*
