@@ -3,7 +3,7 @@
  *		Can also be installed as fingerd on the UNIX
  *		machine RADIUS runs on.
  *
- * Version:	$Id: radwho.c,v 1.42 2002/10/30 20:38:18 aland Exp $
+ * Version:	$Id: radwho.c,v 1.44 2004/04/06 19:10:59 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
  */
 
 static const char rcsid[] =
-"$Id: radwho.c,v 1.42 2002/10/30 20:38:18 aland Exp $";
+"$Id: radwho.c,v 1.44 2004/04/06 19:10:59 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -53,14 +53,14 @@ static const char rcsid[] =
 /*
  *	Header above output and format.
  */
-static const char *hdr1 = 
+static const char *hdr1 =
 "Login      Name              What  TTY  When      From      Location";
 static const char *ufmt1 = "%-10.10s %-17.17s %-5.5s %-4.4s %-9.9s %-9.9s %-.16s%s";
 static const char *ufmt1r = "%s,%s,%s,%s,%s,%s,%s%s";
 static const char *rfmt1 = "%-10.10s %-17.17s %-5.5s %s%-3d %-9.9s %-9.9s %-.19s%s";
 static const char *rfmt1r = "%s,%s,%s,%s%d,%s,%s,%s%s";
 
-static const char *hdr2 = 
+static const char *hdr2 =
 "Login      Port    What      When          From       Location";
 static const char *ufmt2 = "%-10.10s %-6.6d %-7.7s %-13.13s %-10.10s %-.16s%s";
 static const char *ufmt2r = "%s,%d,%s,%s,%s,%s%s";
@@ -80,8 +80,6 @@ const char *radutmp_file = NULL;
 const char *radius_dir = NULL;
 const char *radacct_dir = NULL;
 const char *radlib_dir = NULL;
-int auth_port = 0;
-int acct_port;
 uint32_t myip = INADDR_ANY;
 int log_stripped_names;
 
@@ -135,7 +133,7 @@ static int ffile(const char *arg)
 		fp = safe_popen(fn, "r");
 	} else fp = fopen(fn, "r");
 
-	if (fp == NULL) 
+	if (fp == NULL)
 		return -1;
 
 	while(fgets(fn, 1024, fp)) {
@@ -455,7 +453,7 @@ int main(int argc, char **argv)
 
 	if (showlocal && (fp = fopen(radutmp_file, "r"))) {
 		if (rawoutput == 0)
-		{	
+		{
 			fputs(showname ? hdr1 : hdr2, stdout);
 			fputs(eol, stdout);
 		}
@@ -549,7 +547,7 @@ int main(int argc, char **argv)
 						proto(rt.proto, rt.porttype),
 						dotime(rt.time),
 						nas_name3(nasname, sizeof(nasname), rt.nas_address),
-						hostname(othername, sizeof(othername), rt.framed_address), 	
+						hostname(othername, sizeof(othername), rt.framed_address),
 						eol);
 		}
 	}
