@@ -1,7 +1,7 @@
 /*
  * threads.c	request threading support
  *
- * Version:	$Id: threads.c,v 1.63.2.2 2003/07/30 15:46:22 aland Exp $
+ * Version:	$Id: threads.c,v 1.77.2.1 2004/12/16 18:54:57 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,14 +23,14 @@
 
 #include "libradius.h"
 
-#if HAVE_PTHREAD_H
+#ifdef HAVE_PTHREAD_H
 
 #include <stdlib.h>
 #include <string.h>
 #include <semaphore.h>
 #include <signal.h>
 
-#if HAVE_SYS_WAIT_H
+#ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
 #endif
 
@@ -39,7 +39,7 @@
 #include "conffile.h"
 
 static const char rcsid[] =
-"$Id: threads.c,v 1.63.2.2 2003/07/30 15:46:22 aland Exp $";
+"$Id: threads.c,v 1.77.2.1 2004/12/16 18:54:57 aland Exp $";
 
 #define SEMAPHORE_LOCKED	(0)
 #define SEMAPHORE_UNLOCKED	(1)
@@ -48,11 +48,6 @@ static const char rcsid[] =
 #define THREAD_CANCELLED	(2)
 #define THREAD_EXITED		(3)
 
-/*
- *	Prototype to shut the compiler up.
- */
-int rad_spawn_child(REQUEST *request, RAD_REQUEST_FUNP fun);
-
 /*
  *  A data structure which contains the information about
  *  the current thread.
@@ -63,22 +58,27 @@ int rad_spawn_child(REQUEST *request, RAD_REQUEST_FUNP fun);
  *  status        is the thread running or exited?
  *  request_count the number of requests that this thread has handled
  *  timestamp     when the thread started executing.
- *  request       the current request that the thread is processing.
- *  fun           the function which is handling the request.
  */
 typedef struct THREAD_HANDLE {
 	struct THREAD_HANDLE *prev;
 	struct THREAD_HANDLE *next;
 	pthread_t            pthread_id;
 	int                  thread_num;
-	sem_t                semaphore;
 	int                  status;
 	unsigned int         request_count;
 	time_t               timestamp;
-	REQUEST              *request;
-	RAD_REQUEST_FUNP     fun;
+	REQUEST		     *request;
 } THREAD_HANDLE;
 
+/*
+ *	For the request queue.
+ */
+typedef struct request_queue_t {
+	REQUEST	    	  *request;
+	RAD_REQUEST_FUNP  fun;
+} request_queue_t;
+
+
 /*
  *	A data structure to manage the thread pool.  There's no real
  *	need for a data structure, but it makes things conceptually
@@ -88,18 +88,33 @@ typedef struct THREAD_POOL {
 	THREAD_HANDLE *head;
 	THREAD_HANDLE *tail;
 
-	THREAD_HANDLE *next_pick;
-	
 	int total_threads;
 	int max_thread_num;
 	int start_threads;
 	int max_threads;
 	int min_spare_threads;
 	int max_spare_threads;
-	int max_requests_per_thread;
+	unsigned int max_requests_per_thread;
 	unsigned long request_count;
 	time_t time_last_spawned;
 	int cleanup_delay;
+
+	/*
+	 *	All threads wait on this semaphore, for requests
+	 *	to enter the queue.
+	 */
+	sem_t		semaphore;
+
+	/*
+	 *	To ensure only one thread at a time touches the queue.
+	 */
+	pthread_mutex_t	mutex;
+
+	int		active_threads;
+	int		queue_head; /* first filled entry */
+	int		queue_tail; /* first empty entry */
+	int		queue_size;
+	request_queue_t *queue;
 } THREAD_POOL;
 
 static THREAD_POOL thread_pool;
@@ -138,15 +153,232 @@ static pthread_mutex_t fork_mutex;
  *	A mapping of configuration file names to internal integers
  */
 static const CONF_PARSER thread_config[] = {
-	{ "start_servers",           PW_TYPE_INTEGER, 0, &thread_pool.start_threads,           "5" }, 
-	{ "max_servers",             PW_TYPE_INTEGER, 0, &thread_pool.max_threads,             "32" }, 
-	{ "min_spare_servers",       PW_TYPE_INTEGER, 0, &thread_pool.min_spare_threads,       "3" }, 
-	{ "max_spare_servers",       PW_TYPE_INTEGER, 0, &thread_pool.max_spare_threads,       "10" }, 
-	{ "max_requests_per_server", PW_TYPE_INTEGER, 0, &thread_pool.max_requests_per_thread, "0" }, 
-	{ "cleanup_delay",           PW_TYPE_INTEGER, 0, &thread_pool.cleanup_delay,           "5" }, 
+	{ "start_servers",           PW_TYPE_INTEGER, 0, &thread_pool.start_threads,           "5" },
+	{ "max_servers",             PW_TYPE_INTEGER, 0, &thread_pool.max_threads,             "32" },
+	{ "min_spare_servers",       PW_TYPE_INTEGER, 0, &thread_pool.min_spare_threads,       "3" },
+	{ "max_spare_servers",       PW_TYPE_INTEGER, 0, &thread_pool.max_spare_threads,       "10" },
+	{ "max_requests_per_server", PW_TYPE_INTEGER, 0, &thread_pool.max_requests_per_thread, "0" },
+	{ "cleanup_delay",           PW_TYPE_INTEGER, 0, &thread_pool.cleanup_delay,           "5" },
 	{ NULL, -1, 0, NULL, NULL }
 };
 
+
+/*
+ *	Add a request to the list of waiting requests.
+ *	This function gets called ONLY from the main handler thread...
+ *
+ *	This function should never fail.
+ */
+static void request_enqueue(REQUEST *request, RAD_REQUEST_FUNP fun)
+{
+	int num_entries;
+
+	pthread_mutex_lock(&thread_pool.mutex);
+
+	thread_pool.request_count++;
+
+	/*
+	 *	If the queue is empty, re-set the indices to zero,
+	 *	for no particular reason...
+	 */
+	if ((thread_pool.queue_head == thread_pool.queue_tail) &&
+	    (thread_pool.queue_head != 0)) {
+		thread_pool.queue_head = thread_pool.queue_tail = 0;
+	}
+
+	/*
+	 *	If the queue is full, die.
+	 *
+	 *	The math is to take into account the fact that it's a
+	 *	circular queue.
+	 */
+	num_entries = ((thread_pool.queue_tail + thread_pool.queue_size) -
+		       thread_pool.queue_head) % thread_pool.queue_size;
+	if (num_entries == (thread_pool.queue_size - 1)) {
+		request_queue_t *new_queue;
+
+		/*
+		 *	If the queue becomes larger than 65536,
+		 *	there's a serious problem.
+		 */
+		if (thread_pool.queue_size >= 65536) {
+			pthread_mutex_unlock(&thread_pool.mutex);
+
+			/*
+			 *	Mark the request as done.
+			 */
+			radlog(L_ERR|L_CONS, "!!! ERROR !!! The server is blocked: discarding new request %d", request->number);
+			request->finished = TRUE;
+			return;
+		}
+
+		/*
+		 *	Malloc a new queue, doubled in size, copy the
+		 *	data from the current queue over to it, zero
+		 *	out the second half of the queue, free the old
+		 *	one, and replace thread_pool.queue with the
+		 *	new one.
+		 */
+		new_queue = rad_malloc(sizeof(*new_queue) * thread_pool.queue_size * 2);
+		memcpy(new_queue, thread_pool.queue,
+		       sizeof(*new_queue) * thread_pool.queue_size);
+		memset(new_queue + sizeof(*new_queue) * thread_pool.queue_size,
+		       0, sizeof(*new_queue) * thread_pool.queue_size);
+
+		free(thread_pool.queue);
+		thread_pool.queue = new_queue;
+		thread_pool.queue_size *= 2;
+	}
+
+	/*
+	 *	Add the data to the queue tail, increment the tail,
+	 *	and signal the semaphore that there's another request
+	 *	in the queue.
+	 */
+	thread_pool.queue[thread_pool.queue_tail].request = request;
+	thread_pool.queue[thread_pool.queue_tail].fun = fun;
+	thread_pool.queue_tail++;
+	thread_pool.queue_tail &= (thread_pool.queue_size - 1);
+
+	pthread_mutex_unlock(&thread_pool.mutex);
+
+	/*
+	 *	There's one more request in the queue.
+	 *
+	 *	Note that we're not touching the queue any more, so
+	 *	the semaphore post is outside of the mutex.  This also
+	 *	means that when the thread wakes up and tries to lock
+	 *	the mutex, it will be unlocked, and there won't be
+	 *	contention.
+	 */
+
+	sem_post(&thread_pool.semaphore);
+
+	return;
+}
+
+/*
+ *	Remove a request from the queue.
+ */
+static void request_dequeue(REQUEST **request, RAD_REQUEST_FUNP *fun)
+{
+	pthread_mutex_lock(&thread_pool.mutex);
+
+	/*
+	 *	Head & tail are the same.  There's nothing in
+	 *	the queue.
+	 */
+	if (thread_pool.queue_head == thread_pool.queue_tail) {
+		pthread_mutex_unlock(&thread_pool.mutex);
+		*request = NULL;
+		*fun = NULL;
+		return;
+	}
+
+	*request = thread_pool.queue[thread_pool.queue_head].request;
+	*fun = thread_pool.queue[thread_pool.queue_head].fun;
+
+	rad_assert(*request != NULL);
+	rad_assert((*request)->magic == REQUEST_MAGIC);
+	rad_assert(*fun != NULL);
+
+	thread_pool.queue_head++;
+	thread_pool.queue_head &= (thread_pool.queue_size - 1);
+
+	/*
+	 *	FIXME: Check the request timestamp.  If it's more than
+	 *	"clean_delay" seconds old, then discard the request,
+	 *	log an error, and try to de-queue another request.
+	 *
+	 *	The main clean-up code won't delete the request from
+	 *	the request list, because it's not marked "finished"
+	 */
+
+	/*
+	 *	The thread is currently processing a request.
+	 */
+	thread_pool.active_threads++;
+
+	pthread_mutex_unlock(&thread_pool.mutex);
+
+	/*
+	 *	If the request is currently being processed, then that
+	 *	MAY be OK, if it's a proxy reply.  In that case,
+	 *	sending the packet may result in a reply being
+	 *	received before that thread clears the child_pid.
+	 *
+	 *	In that case, we busy-wait for the request to be free.
+	 *
+	 *	We COULD push it onto the queue and try to grab
+	 *	another request, but what if this is the only request?
+	 *	What if there are multiple such packets with race
+	 *	conditions?  We don't want to thrash the queue...
+	 *
+	 *	This busy-wait is less than optimal, but it's simple,
+	 *	fail-safe, and it works.
+	 */
+	if ((*request)->child_pid != NO_SUCH_CHILD_PID) {
+		int count, ok;
+		struct timeval tv;
+#ifdef HAVE_PTHREAD_SIGMASK
+		sigset_t set, old_set;
+
+		/*
+		 *	Block a large number of signals which could
+		 *	cause the select to return EINTR
+		 */
+		sigemptyset(&set);
+		sigaddset(&set, SIGPIPE);
+		sigaddset(&set, SIGCONT);
+		sigaddset(&set, SIGSTOP);
+		sigaddset(&set, SIGCHLD);
+		pthread_sigmask(SIG_BLOCK, &set, &old_set);
+#endif
+
+		rad_assert((*request)->proxy_reply != NULL);
+
+		ok = FALSE;
+
+		/*
+		 *	Sleep for 100 milliseconds.  If the other thread
+		 *	doesn't get serviced in this time, to clear
+		 *	the "child_pid" entry, then the server is too
+		 *	busy, so we die.
+		 */
+		for (count = 0; count < 10; count++) {
+			tv.tv_sec = 0;
+			tv.tv_usec = 10000; /* sleep for 10 milliseconds */
+
+			/*
+			 *	Portable sleep that's thread-safe.
+			 *
+			 *	Don't worry about interrupts, as they're
+			 *	blocked above.
+			 */
+			select(0, NULL, NULL, NULL, &tv);
+			if ((*request)->child_pid == NO_SUCH_CHILD_PID) {
+				ok = TRUE;
+				break;
+			}
+		}
+
+#ifdef HAVE_PTHREAD_SIGMASK
+		/*
+		 *	Restore the original thread signal mask.
+		 */
+		pthread_sigmask(SIG_SETMASK, &old_set, NULL);
+#endif
+
+		if (!ok) {
+			radlog(L_ERR, "FATAL!  Server is too busy to process requests");
+			exit(1);
+		}
+	}
+
+	return;
+}
+
+
 /*
  *	The main thread handler for requests.
  *
@@ -154,8 +386,9 @@ static const CONF_PARSER thread_config[] = {
  */
 static void *request_handler_thread(void *arg)
 {
-	THREAD_HANDLE	*self = (THREAD_HANDLE *) arg;
-#if HAVE_PTHREAD_SIGMASK
+	RAD_REQUEST_FUNP  fun;
+	THREAD_HANDLE	  *self = (THREAD_HANDLE *) arg;
+#ifdef HAVE_PTHREAD_SIGMASK
 	sigset_t set;
 
 	/*
@@ -175,24 +408,25 @@ static void *request_handler_thread(void *arg)
 	sigaddset(&set, SIGTERM);
 	pthread_sigmask(SIG_BLOCK, &set, NULL);
 #endif
-	
+
 	/*
 	 *	Loop forever, until told to exit.
 	 */
-	for (;;) {
+	do {
 		/*
 		 *	Wait to be signalled.
 		 */
 		DEBUG2("Thread %d waiting to be assigned a request",
 		       self->thread_num);
 	re_wait:
-		if (sem_wait(&self->semaphore) != 0) {
+		if (sem_wait(&thread_pool.semaphore) != 0) {
 			/*
 			 *	Interrupted system call.  Go back to
 			 *	waiting, but DON'T print out any more
 			 *	text.
 			 */
 			if (errno == EINTR) {
+				DEBUG2("Re-wait %d", self->thread_num);
 				goto re_wait;
 			}
 			radlog(L_ERR, "Thread %d failed waiting for semaphore: %s: Exiting\n",
@@ -200,36 +434,38 @@ static void *request_handler_thread(void *arg)
 			break;
 		}
 
-		/*
-		 *	If we've been told to kill ourselves,
-		 *	then exit politely.
-		 */
-		if (self->status == THREAD_CANCELLED) {
-			DEBUG2("Thread %d exiting on request from parent.",
-					self->thread_num);
-			break;
-		}
+		DEBUG2("Thread %d got semaphore", self->thread_num);
 
 		/*
-		 *	Stupid implementations of sem_wait return on
-		 *	signals, but don't return -1.
+		 *	Try to grab a request from the queue.
+		 *
+		 *	It may be empty, in which case we fail
+		 *	gracefully.
 		 */
-		if (!self->request) {
-			continue;
-		}
+		request_dequeue(&self->request, &fun);
+		if (!self->request) continue;
+
+		self->request->child_pid = self->pthread_id;
+		self->request_count++;
 
 		DEBUG2("Thread %d handling request %d, (%d handled so far)",
-				self->thread_num, self->request->number,
-				self->request_count);
-		
-		/*
-		 *  This responds, and resets request->child_pid
-		 */
-		rad_respond(self->request, self->fun);
+		       self->thread_num, self->request->number,
+		       self->request_count);
 
-		self->fun = NULL;
+		/*
+		 *	Respond, and reset request->child_pid
+		 */
+		rad_respond(self->request, fun);
 		self->request = NULL;
-	}
+
+		/*
+		 *	Update the active threads.
+		 */
+		pthread_mutex_lock(&thread_pool.mutex);
+		rad_assert(thread_pool.active_threads > 0);
+		thread_pool.active_threads--;
+		pthread_mutex_unlock(&thread_pool.mutex);
+	} while (self->status != THREAD_CANCELLED);
 
 	DEBUG2("Thread %d exiting...", self->thread_num);
 
@@ -267,7 +503,7 @@ static void delete_thread(THREAD_HANDLE *handle)
 	} else {
 		prev->next = next;
 	}
-  
+
 	if (next == NULL) {
 		rad_assert(thread_pool.tail == handle);
 		thread_pool.tail = prev;
@@ -275,20 +511,12 @@ static void delete_thread(THREAD_HANDLE *handle)
 		next->prev = prev;
 	}
 
-	/*
-	 *	Ensure that deleted threads aren't picked.
-	 */
-	if (thread_pool.next_pick == handle) {
-		thread_pool.next_pick = next;
-	}
-
 	DEBUG2("Deleting thread %d", handle->thread_num);
 
 	/*
 	 *	This thread has exited.  Delete any additional
-	 *	resources associated with it (semaphore, etc).
+	 *	resources associated with it.
 	 */
-	sem_destroy(&handle->semaphore);
 
 	/*
 	 *	Free the memory, now that we're sure the thread
@@ -331,24 +559,6 @@ static THREAD_HANDLE *spawn_thread(time_t now)
 	handle->status = THREAD_RUNNING;
 	handle->timestamp = time(NULL);
 
-	/*
-	 *	Initialize the semaphore to be for this process only,
-	 *	and locked, so that the thread block until the server
-	 *	kicks it.
-	 */
-	rcode = sem_init(&handle->semaphore, 0, SEMAPHORE_LOCKED);
-	if (rcode != 0) {
-		radlog(L_ERR|L_CONS, "FATAL: Failed to initialize semaphore: %s",
-				strerror(errno));
-		exit(1);
-	}
-
-	/*
-	 *	The thread isn't currently handling a request.
-	 */
-	handle->request = NULL;
-	handle->fun = NULL;
-
 	/*
 	 *	Initialize the thread's attributes to detached.
 	 *
@@ -358,7 +568,7 @@ static THREAD_HANDLE *spawn_thread(time_t now)
 	 */
 	pthread_attr_init(&attr);
 	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
-	
+
 	/*
 	 *	Create the thread detached, so that it cleans up it's
 	 *	own memory when it exits.
@@ -410,11 +620,11 @@ static THREAD_HANDLE *spawn_thread(time_t now)
  *      until all threads are finished handling requests.  This returns
  *      the number of active threads to 'radiusd.c'.
  */
-int total_active_threads(void) 
+int total_active_threads(void)
 {
         int rcode = 0;
 	THREAD_HANDLE *handle;
-	
+
 	for (handle = thread_pool.head; handle != NULL; handle = handle->next){
 		if (handle->request != NULL) {
 			rcode ++;
@@ -431,8 +641,7 @@ int total_active_threads(void)
  */
 int thread_pool_init(void)
 {
-	int i;
-	THREAD_HANDLE	*handle;
+	int		i, rcode;
 	CONF_SECTION	*pool_cf;
 	time_t		now;
 
@@ -449,7 +658,6 @@ int thread_pool_init(void)
 		memset(&thread_pool, 0, sizeof(THREAD_POOL));
 		thread_pool.head = NULL;
 		thread_pool.tail = NULL;
-		thread_pool.next_pick = NULL;
 		thread_pool.total_threads = 0;
 		thread_pool.max_thread_num = 1;
 		thread_pool.cleanup_delay = 5;
@@ -479,133 +687,79 @@ int thread_pool_init(void)
 		return 0;
 	}
 
+	/*
+	 *	Initialize the queue of requests.
+	 */
+	rcode = sem_init(&thread_pool.semaphore, 0, SEMAPHORE_LOCKED);
+	if (rcode != 0) {
+		radlog(L_ERR|L_CONS, "FATAL: Failed to initialize semaphore: %s",
+		       strerror(errno));
+		exit(1);
+	}
+
+	rcode = pthread_mutex_init(&thread_pool.mutex,NULL);
+	if (rcode != 0) {
+		radlog(L_ERR, "FATAL: Failed to initialize mutex: %s",
+		       strerror(errno));
+		exit(1);
+	}
+
+	/*
+	 *	Queue head & tail are set to zero by the memset,
+	 *	above.
+	 *
+	 *	Allocate an initial queue, always as a power of 2.
+	 */
+	thread_pool.queue_size = 256;
+	thread_pool.queue = rad_malloc(sizeof(*thread_pool.queue) *
+				       thread_pool.queue_size);
+	memset(thread_pool.queue, 0, (sizeof(*thread_pool.queue) *
+				      thread_pool.queue_size));
+
 	/*
 	 *	Create a number of waiting threads.
 	 *
 	 *	If we fail while creating them, do something intelligent.
 	 */
 	for (i = 0; i < thread_pool.start_threads; i++) {
-		handle = spawn_thread(now);
-		if (handle == NULL) {
+		if (spawn_thread(now) == NULL) {
 			return -1;
 		}
 	}
 
+	DEBUG2("Thread pool initialized");
 	pool_initialized = TRUE;
 	return 0;
 }
 
+
 /*
  *	Assign a new request to a free thread.
  *
  *	If there isn't a free thread, then try to create a new one,
  *	up to the configured limits.
  */
-int rad_spawn_child(REQUEST *request, RAD_REQUEST_FUNP fun)
+int thread_pool_addrequest(REQUEST *request, RAD_REQUEST_FUNP fun)
 {
-	THREAD_HANDLE *handle;
-	THREAD_HANDLE *found;
-	THREAD_HANDLE *start;
-
 	/*
-	 *	Loop over the active thread pool, looking for a
-	 *	waiting thread.
+	 *	If the thread pool is busy handling requests, then
+	 *	try to spawn another one.
 	 */
-	found = NULL;
-
-	/*
-	 *	Minimize the time spent searching for an empty thread,
-	 *	by starting off from the last one we found.  This also
-	 *	helps to span requests across multiple threads.
-	 */
-	start = thread_pool.next_pick;
-	if (!start) start = thread_pool.head;
-	
-	handle = start;
-	while (handle) {
-		/*
-		 *	Ignore threads which aren't running.
-		 */
-		if (handle->status != THREAD_RUNNING) {
-			goto next_handle;
-		}
-
-		/*
-		 *	If we haven't found a free thread yet, then
-		 *	check it's semaphore lock.  We don't lock it,
-		 *	so if it's locked, then the thread MUST be the
-		 *	one locking it, waiting for us to unlock it.
-		 */
-		if (handle->request == NULL) {
-			found = handle;
-			break;
-		}
-
-	next_handle:
-		/*
-		 *	There is a 'next', use it.
-		 *
-		 *	Otherwise, go back to the head of the list.
-		 */
-		if (handle->next) {
-			handle = handle->next;
-		} else {
-			handle = thread_pool.head;
-		}
-
-		/*
-		 *	Stop if we've looped back to the beginning.
-		 */
-		if (handle == start) break;
-	} /* loop over all of the threads */
-
-	/*
-	 *	If we haven't found an active thread, then spawn a new one.
-	 *
-	 *	If we can't spawn a new one, complain, and exit.
-	 */
-	if (!found) {
-		found = spawn_thread(request->timestamp);
-		if (found == NULL) {
-			radlog(L_INFO, 
-					"The maximum number of threads (%d) are active, cannot spawn new thread to handle request", 
-					thread_pool.max_threads);
-			return -1;
+	if (thread_pool.active_threads == thread_pool.total_threads) {
+		if (spawn_thread(request->timestamp) == NULL) {
+			radlog(L_INFO,
+			       "The maximum number of threads (%d) are active, cannot spawn new thread to handle request",
+			       thread_pool.max_threads);
+			return 0;
 		}
 	}
 
 	/*
-	 *	Remember where we left off, so that we can start the
-	 *	search from here the next time.
+	 *	Add the new request to the queue.
 	 */
-	thread_pool.next_pick = found->next;
+	request_enqueue(request, fun);
 
-	/*
-	 *	OK, now 'handle' points to a waiting thread.  We move
-	 *	it to the tail of the thread pool, so that we can
-	 *	cycle among the threads.
-	 *
-	 *	We then give it the request, signal its semaphore, and
-	 *	return.  The thread eventually wakes up, and handles
-	 *	the request.
-	 */
-	DEBUG2("Thread %d assigned request %d", found->thread_num, request->number);
-	request->child_pid = found->pthread_id;
-
-	/*
-	 *	Tell the child thread what to do.
-	 */
-	found->request = request;
-	found->fun = fun;
-	found->request_count++;
-	thread_pool.request_count++;
-
-	/*
-	 * 	Now that the child has things to do, kick it.
-	 */
-	sem_post(&found->semaphore);
-
-	return 0;
+	return 1;
 }
 
 /*
@@ -620,25 +774,15 @@ int thread_pool_clean(time_t now)
 	int i, total;
 	THREAD_HANDLE *handle, *next;
 	int active_threads;
+	static time_t last_cleaned = 0;
 
 	/*
-	 *	Loop over the thread pool, incrementing the 'active'
-	 *	counter if the thread is currently processing a request.
-	 *
-	 *	At this point, we also delete 'exited' threads.
+	 *	Loop over the thread pool deleting exited threads.
 	 */
-	active_threads = 0;
 	for (handle = thread_pool.head; handle; handle = next) {
 		next = handle->next;
 
-		if (handle->request != NULL) {
-			active_threads++;
-			continue;
-		}
-
 		/*
-		 *	Handle is NULL, so it's not processing anything.
-		 *
 		 *	Maybe we've asked the thread to exit, and it
 		 *	has agreed.
 		 */
@@ -647,6 +791,13 @@ int thread_pool_clean(time_t now)
 		}
 	}
 
+	/*
+	 *	We don't need a mutex lock here, as we're reading
+	 *	the location, and not modifying it.  We want a close
+	 *	approximation of the number of active threads, and this
+	 *	is good enough.
+	 */
+	active_threads = thread_pool.active_threads;
 	spare = thread_pool.total_threads - active_threads;
 	if (debug_flag) {
 		static int old_total = -1;
@@ -684,6 +835,15 @@ int thread_pool_clean(time_t now)
 		return 0;
 	}
 
+	/*
+	 *	Only delete spare threads if we haven't already done
+	 *	so this second.
+	 */
+	if (now == last_cleaned) {
+		return 0;
+	}
+	last_cleaned = now;
+
 	/*
 	 *	Only delete the spare threads if sufficient time has
 	 *	passed since we last created one.  This helps to minimize
@@ -708,24 +868,27 @@ int thread_pool_clean(time_t now)
 
 		/*
 		 *	Walk through the thread pool, deleting the
-		 *	first N idle threads we come across.
+		 *	first idle thread we come across.
 		 */
 		for (handle = thread_pool.head; (handle != NULL) && (spare > 0) ; handle = next) {
 			next = handle->next;
 
 			/*
 			 *	If the thread is not handling a
-			 *	request, then tell it to exit.
+			 *	request, but still live, then tell it
+			 *	to exit.
 			 *
-			 *	Note that we delete it from the thread
-			 *	pool BEFORE telling it to kill itself,
-			 *	as the child thread can free the 'handle'
-			 *	structure, without anyone else using it.
+			 *	It will eventually wake up, and realize
+			 *	it's been told to commit suicide.
 			 */
-			if (handle->request == NULL) {
+			if ((handle->request == NULL) &&
+			    (handle->status == THREAD_RUNNING)) {
 				handle->status = THREAD_CANCELLED;
-
-				sem_post(&handle->semaphore);
+				/*
+				 *	Post an extra semaphore, as a
+				 *	signal to wake up, and exit.
+				 */
+				sem_post(&thread_pool.semaphore);
 				spare--;
 				break;
 			}
@@ -741,16 +904,18 @@ int thread_pool_clean(time_t now)
 			next = handle->next;
 
 			/*
-			 *	Not handling a request, we can kill it.
+			 *	Not handling a request, but otherwise
+			 *	live, we can kill it.
 			 */
 			if ((handle->request == NULL) &&
+			    (handle->status == THREAD_RUNNING) &&
 			    (handle->request_count > thread_pool.max_requests_per_thread)) {
 				handle->status = THREAD_CANCELLED;
-				sem_post(&handle->semaphore);
+				sem_post(&thread_pool.semaphore);
 			}
 		}
 	}
-  
+
 	/*
 	 *	Otherwise everything's kosher.  There are not too few,
 	 *	or too many spare threads.  Exit happily.
@@ -766,12 +931,12 @@ static int exec_initialized = FALSE;
 void rad_exec_init(void)
 {
 	int i;
-	
+
 	/*
 	 *	Initialize the mutex used to remember calls to fork.
 	 */
 	pthread_mutex_init(&fork_mutex, NULL);
-	
+
 	/*
 	 *	Initialize the data structure where we remember the
 	 *	mappings of thread ID && child PID to exit status.
@@ -798,7 +963,7 @@ void rad_exec_init(void)
 pid_t rad_fork(int exec_wait)
 {
 	sigset_t set;
-	pid_t child_pid;	
+	pid_t child_pid;
 
 	/*
 	 *	The thread is NOT interested in waiting for the exit
@@ -844,7 +1009,7 @@ pid_t rad_fork(int exec_wait)
 		 */
 		i = PID_2_ARRAY(child_pid);
 		found = -1;
-		
+
 		/*
 		 *	We may have multiple threads trying to find an
 		 *	empty position, so we lock the array until
@@ -883,13 +1048,14 @@ pid_t rad_fork(int exec_wait)
 			i++;
 			i &= (NUM_FORKERS - 1);
 		} while (i != PID_2_ARRAY(child_pid));
-		pthread_mutex_unlock(&fork_mutex);
 
 		/*
 		 *	Arg.  We did a fork, and there was nowhere to
 		 *	put the answer.
 		 */
 		if (found < 0) {
+			sigprocmask(SIG_UNBLOCK, &set, NULL);
+			pthread_mutex_unlock(&fork_mutex);
 			return (pid_t) -1;
 		}
 
@@ -899,9 +1065,10 @@ pid_t rad_fork(int exec_wait)
 		 */
 		forkers[found].status = -1;
 		forkers[found].child_pid = child_pid;
-		forkers[i].thread_id = pthread_self();
-		forkers[i].time_forked = now;
+		forkers[found].thread_id = pthread_self();
+		forkers[found].time_forked = now;
 		sem_init(&forkers[found].child_done, 0, SEMAPHORE_LOCKED);
+		pthread_mutex_unlock(&fork_mutex);
 	}
 
 	/*
@@ -1041,7 +1208,7 @@ int rad_savepid(pid_t pid, int status)
 			 *	That is, we've forked, and the forker
 			 *	is waiting nearly forever
 			 */
-			return 0;		
+			return 0;
 		}
 
 		i++;
