@@ -2,7 +2,7 @@
  * valuepair.c	Valuepair functions that are radiusd-specific
  *		and as such do not belong in the library.
  *
- * Version:	$Id: valuepair.c,v 1.60.2.3 2004/09/09 14:31:06 aland Exp $
+ * Version:	$Id: valuepair.c,v 1.60.2.4 2005/02/07 20:04:47 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-static const char rcsid[] = "$Id: valuepair.c,v 1.60.2.3 2004/09/09 14:31:06 aland Exp $";
+static const char rcsid[] = "$Id: valuepair.c,v 1.60.2.4 2005/02/07 20:04:47 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -570,13 +570,17 @@ static int presufcmp(void *instance,
 	if (ret != 0)
 		return ret;
 
-	if (pairfind(check_pairs, PW_STRIP_USER_NAME)) {
-		/*
-		 *	I don't think we want to update the User-Name
-		 *	attribute in place... - atd
-		 */
-		strcpy((char *)request->strvalue, rest);
-		request->length = strlen(rest);
+	if ((vp = pairfind(check_pairs, PW_STRIP_USER_NAME)) != NULL) {
+		if (vp->lvalue == 1) {
+			/*
+			 *	I don't think we want to update the User-Name
+			 *	attribute in place... - atd
+			 */
+			strcpy((char *)request->strvalue, rest);
+			request->length = strlen(rest);
+		} else {
+			return ret;
+		}
 	} else {
 		if ((vp = pairfind(check_pairs, PW_STRIPPED_USER_NAME)) != NULL){
 			strcpy((char *)vp->strvalue, rest);
