@@ -1,18 +1,19 @@
 /*
  * rlm_always.c
  *
- *  This program is is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License, version 2 if the
- *  License as published by the Free Software Foundation.
- * 
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *  
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 2 of the License, or
+ *   (at your option) any later version.
+ *
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
+ *
+ *   You should have received a copy of the GNU General Public License
+ *   along with this program; if not, write to the Free Software
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * Copyright 2000  The FreeRADIUS server project
  */
@@ -28,7 +29,7 @@
 #include "modules.h"
 #include "conffile.h"
 
-static const char rcsid[] = "$Id: rlm_always.c,v 1.9 2002/11/12 17:09:14 aland Exp $";
+static const char rcsid[] = "$Id: rlm_always.c,v 1.12 2004/02/26 19:04:27 aland Exp $";
 
 /*
  *	The instance data for rlm_always is the list of fake values we are
@@ -91,11 +92,15 @@ static int str2rcode(const char *s)
 static int always_instantiate(CONF_SECTION *conf, void **instance)
 {
 	rlm_always_t *data;
-	
+
 	/*
 	 *	Set up a storage area for instance data
 	 */
 	data = rad_malloc(sizeof(*data));
+	if (!data) {
+		return -1;
+	}
+	memset(data, 0, sizeof(*data));
 
 	/*
 	 *	If the configuration parameters can't be parsed, then
@@ -105,7 +110,7 @@ static int always_instantiate(CONF_SECTION *conf, void **instance)
 		free(data);
 		return -1;
 	}
-	
+
 	/*
 	 *	Convert the rcode string to an int, and get rid of it
 	 */
@@ -156,7 +161,7 @@ static int always_detach(void *instance)
 }
 
 module_t rlm_always = {
-	"always",	
+	"always",
 	RLM_TYPE_THREAD_SAFE,		/* type */
 	NULL,				/* initialization */
 	always_instantiate,		/* instantiation */
