@@ -1,7 +1,7 @@
 /*
  * rlm_attr_rewrite.c
  *
- * Version:  $Id: rlm_attr_rewrite.c,v 1.10 2002/10/17 13:01:23 kkalev Exp $
+ * Version:  $Id: rlm_attr_rewrite.c,v 1.21 2004/05/02 12:57:33 kkalev Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include "config.h"
-#if HAVE_REGEX_H
+#ifdef HAVE_REGEX_H
 #	include <regex.h>
 #endif
 
@@ -40,19 +40,21 @@
 #define RLM_REGEX_INPACKET 0
 #define RLM_REGEX_INCONFIG 1
 #define RLM_REGEX_INREPLY  2
+#define RLM_REGEX_INPROXY 3
+#define RLM_REGEX_INPROXYREPLY 4
 
-static const char rcsid[] = "$Id: rlm_attr_rewrite.c,v 1.10 2002/10/17 13:01:23 kkalev Exp $";
+static const char rcsid[] = "$Id: rlm_attr_rewrite.c,v 1.21 2004/05/02 12:57:33 kkalev Exp $";
 
 typedef struct rlm_attr_rewrite_t {
 	char *attribute;	/* The attribute to search for */
 	int  attr_num;		/* The attribute number */
 	char *search;		/* The pattern to search for */
 	int search_len;		/* The length of the search pattern */
-	char *searchin_str;	/* The VALUE_PAIR list to search in. Can be either packet,reply or config */
+	char *searchin_str;	/* The VALUE_PAIR list to search in. Can be either packet,reply,proxy,proxy_reply or config */
 	char searchin;		/* The same as above just coded as a number for speed */
 	char *replace;		/* The replacement */
 	int replace_len;	/* The length of the replacement string */
-	int  append;		/* Switch to control append mode (1,0) */ 
+	int  append;		/* Switch to control append mode (1,0) */
 	int  nocase;		/* Ignore case */
 	int  new_attr;		/* Boolean. Do we create a new attribute or not? */
 	int  num_matches;	/* Maximum number of matches */
@@ -78,11 +80,15 @@ static int attr_rewrite_instantiate(CONF_SECTION *conf, void **instance)
 	rlm_attr_rewrite_t *data;
 	DICT_ATTR *dattr;
 	char *instance_name = NULL;
-	
+
 	/*
 	 *	Set up a storage area for instance data
 	 */
 	data = rad_malloc(sizeof(*data));
+	if (!data) {
+		return -1;
+	}
+	memset(data, 0, sizeof(*data));
 
 	/*
 	 *	If the configuration parameters can't be parsed, then
@@ -94,7 +100,7 @@ static int attr_rewrite_instantiate(CONF_SECTION *conf, void **instance)
 	}
 
 	/*
-	 *	Discover the attribute number of the key. 
+	 *	Discover the attribute number of the key.
 	 */
 	if (data->attribute == NULL) {
 		radlog(L_ERR, "rlm_attr_rewrite: 'attribute' must be set.");
@@ -127,6 +133,10 @@ static int attr_rewrite_instantiate(CONF_SECTION *conf, void **instance)
 			data->searchin = RLM_REGEX_INCONFIG;
 		else if (strcmp(data->searchin_str, "reply") == 0)
 			data->searchin = RLM_REGEX_INREPLY;
+		else if (strcmp(data->searchin_str, "proxy") == 0)
+			data->searchin = RLM_REGEX_INPROXY;
+		else if (strcmp(data->searchin_str, "proxy_reply") == 0)
+			data->searchin = RLM_REGEX_INPROXYREPLY;
 		else {
 			radlog(L_ERR, "rlm_attr_rewrite: Illegal searchin directive given. Assuming packet.");
 			data->searchin = RLM_REGEX_INPACKET;
@@ -145,10 +155,10 @@ static int attr_rewrite_instantiate(CONF_SECTION *conf, void **instance)
 	instance_name = cf_section_name2(conf);
 	if (instance_name != NULL)
 		data->name = strdup(instance_name);
-	
-	
+
+
 	*instance = data;
-	
+
 	return 0;
 }
 
@@ -159,7 +169,7 @@ static int do_attr_rewrite(void *instance, REQUEST *request)
 	VALUE_PAIR *attr_vp = NULL;
 	VALUE_PAIR *tmp = NULL;
 	regex_t preg;
-	regmatch_t pmatch;
+	regmatch_t pmatch[9];
 	int cflags = 0;
 	int err = 0;
 	char done_xlat = 0;
@@ -194,6 +204,16 @@ static int do_attr_rewrite(void *instance, REQUEST *request)
 			case RLM_REGEX_INREPLY:
 				tmp = request->reply->vps;
 				break;
+			case RLM_REGEX_INPROXYREPLY:
+				if (!request->proxy_reply)
+					return RLM_MODULE_NOOP;
+				tmp = request->proxy_reply->vps;
+				break;
+			case RLM_REGEX_INPROXY:
+				if (!request->proxy)
+					return RLM_MODULE_NOOP;
+				tmp = request->proxy->vps;
+				break;
 			default:
 				radlog(L_ERR, "rlm_attr_rewrite: Illegal value for searchin. Changing to packet.");
 				data->searchin = RLM_REGEX_INPACKET;
@@ -229,6 +249,8 @@ do_again:
 	}
 
 	if (!data->new_attr){
+		unsigned int j = 0;
+
 		if ((err = regcomp(&preg,search_STR,cflags))) {
 			regerror(err, &preg, err_msg, MAX_STRING_LEN);
 			DEBUG2("rlm_attr_rewrite: regcomp() returned error: %s",err_msg);
@@ -238,14 +260,14 @@ do_again:
 		ptr2 = attr_vp->strvalue;
 		counter = 0;
 
-		for ( i = 0 ;i < data->num_matches; i++) {
-			err = regexec(&preg, ptr2, 1, &pmatch, 0);
+		for ( i = 0 ;i < (unsigned)data->num_matches; i++) {
+			err = regexec(&preg, ptr2, 9, pmatch, 0);
 			if (err == REG_NOMATCH) {
 				if (i == 0) {
 					DEBUG2("rlm_attr_rewrite: No match found for attribute %s with value '%s'",
 							data->attribute, attr_vp->strvalue);
 					regfree(&preg);
-					return RLM_MODULE_OK;
+					goto to_do_again;
 				} else
 					break;
 			}
@@ -255,23 +277,55 @@ do_again:
 						data->attribute, attr_vp->strvalue);
 				return ret;
 			}
-			if (pmatch.rm_so == -1)
+			if (pmatch[0].rm_so == -1)
 				break;
-			len = pmatch.rm_so;
+			len = pmatch[0].rm_so;
 			if (data->append) {
-				len = len + (pmatch.rm_eo - pmatch.rm_so);
-			} 
+				len = len + (pmatch[0].rm_eo - pmatch[0].rm_so);
+			}
 			counter += len;
 			if (counter >= MAX_STRING_LEN) {
 				regfree(&preg);
 				DEBUG2("rlm_attr_rewrite: Replacement out of limits for attribute %s with value '%s'",
-						data->attribute, attr_vp->strvalue);	
+						data->attribute, attr_vp->strvalue);
 				return ret;
 			}
 
 			strncpy(ptr, ptr2,len);
 			ptr += len;
-			ptr2 += pmatch.rm_eo;
+			ptr2 += pmatch[0].rm_eo;
+
+			if (i == 0){
+				/*
+				 * We only run on the first match, sorry
+				 */
+				for(j = 0; j <= 8; j++){
+					char *p;
+					char buffer[sizeof(attr_vp->strvalue)];
+
+					/*
+				   	 * Stolen from src/main/valuepair.c, paircmp()
+				 	 */
+				
+					/*
+					 * Delete old matches if the corresponding match does not
+					 * exist in the current regex
+					 */
+					if (pmatch[j].rm_so == -1){
+						p = request_data_get(request,request,REQUEST_DATA_REGEX | j);
+						if (p){
+							free(p);
+							continue;
+						}
+						break;
+					}
+					memcpy(buffer,
+						attr_vp->strvalue + pmatch[j].rm_so,pmatch[j].rm_eo);
+					buffer[pmatch[j].rm_eo] = '\0';
+					p = strdup(buffer);
+					request_data_add(request,request,REQUEST_DATA_REGEX | j,p,free);
+				}
+			}
 
 			if (!done_xlat){
 				if (data->replace_len != 0 &&
@@ -287,12 +341,12 @@ do_again:
 			if (counter >= MAX_STRING_LEN) {
 				regfree(&preg);
 				DEBUG2("rlm_attr_rewrite: Replacement out of limits for attribute %s with value '%s'",
-						data->attribute, attr_vp->strvalue);	
+						data->attribute, attr_vp->strvalue);
 				return ret;
 			}
 			if (replace_len){
 				strncpy(ptr, replace_STR, replace_len);
-				ptr += replace_len;	
+				ptr += replace_len;
 			}
 		}
 		regfree(&preg);
@@ -300,7 +354,7 @@ do_again:
 		counter += len;
 		if (counter >= MAX_STRING_LEN){
 			DEBUG2("rlm_attr_rewrite: Replacement out of limits for attribute %s with value '%s'",
-					data->attribute, attr_vp->strvalue);	
+					data->attribute, attr_vp->strvalue);
 			return ret;
 		}
 		strncpy(ptr, ptr2, len);
@@ -310,6 +364,7 @@ do_again:
 		attr_vp->length = strlen(new_str);
 		strncpy(attr_vp->strvalue, new_str, (attr_vp->length + 1));
 
+to_do_again:
 		ret = RLM_MODULE_OK;
 
 		if (tmp != NULL){
@@ -320,6 +375,11 @@ do_again:
 	}
 	else{
 		attr_vp = pairmake(data->attribute,replace_STR,0);
+		if (attr_vp == NULL){
+			DEBUG2("rlm_attr_rewrite: Could not add new attribute %s with value '%s'",
+				data->attribute,replace_STR);
+			return ret;
+		}
 		switch(data->searchin){
 			case RLM_REGEX_INPACKET:
 				pairadd(&request->packet->vps,attr_vp);
@@ -336,10 +396,10 @@ do_again:
 				pairadd(&request->packet->vps,attr_vp);
 				break;
 		}
-		DEBUG2("rlm_attr_rewrite: Added attribute %s with value '%s'",data->attribute,attr_vp->strvalue);
+		DEBUG2("rlm_attr_rewrite: Added attribute %s with value '%s'",data->attribute,replace_STR);
 		ret = RLM_MODULE_OK;
 	}
-				
+
 
 	return ret;
 }
@@ -390,7 +450,7 @@ static int attr_rewrite_detach(void *instance)
 		free(data->attribute);
 	if (data->search)
 		free(data->search);
-	if (data->replace)	
+	if (data->replace)
 		free(data->replace);
 	if (data->name)
 		free(data->name);
@@ -409,7 +469,7 @@ static int attr_rewrite_detach(void *instance)
  *	is single-threaded.
  */
 module_t rlm_attr_rewrite = {
-	"attr_rewrite",	
+	"attr_rewrite",
 	RLM_TYPE_THREAD_UNSAFE,		/* type */
 	NULL,				/* initialization */
 	attr_rewrite_instantiate,		/* instantiation */
