@@ -1,7 +1,7 @@
 /*
  * rlm_detail.c	accounting:    Write the "detail" files.
  *
- * Version:	$Id: rlm_detail.c,v 1.30.2.1 2003/09/12 19:18:04 phampson Exp $
+ * Version:	$Id: rlm_detail.c,v 1.37 2004/02/26 19:04:28 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  * Copyright 2000  The FreeRADIUS server project
  */
 
-static const char rcsid[] = "$Id: rlm_detail.c,v 1.30.2.1 2003/09/12 19:18:04 phampson Exp $";
+static const char rcsid[] = "$Id: rlm_detail.c,v 1.37 2004/02/26 19:04:28 aland Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -37,6 +37,22 @@ static const char rcsid[] = "$Id: rlm_detail.c,v 1.30.2.1 2003/09/12 19:18:04 ph
 #include	"modules.h"
 #define 	DIRLEN	8192
 
+static const char *packet_codes[] = {
+  "",
+  "Access-Request",
+  "Access-Accept",
+  "Access-Reject",
+  "Accounting-Request",
+  "Accounting-Response",
+  "Accounting-Status",
+  "Password-Request",
+  "Password-Accept",
+  "Password-Reject",
+  "Accounting-Message",
+  "Access-Challenge"
+};
+
+
 struct detail_instance {
 	/* detail file */
 	char *detailfile;
@@ -90,22 +106,33 @@ static int detail_instantiate(CONF_SECTION *conf, void **instance)
 	return 0;
 }
 
-static int do_detail(void *instance, REQUEST *request, VALUE_PAIR *pair)
+/*
+ *	Do detail, compatible with old accounting
+ */
+static int do_detail(void *instance, REQUEST *request, RADIUS_PACKET *packet,
+		     int compat)
 {
 	int		outfd;
 	FILE		*outfp;
 	char		buffer[DIRLEN];
 	char		*p;
-	int		ret = RLM_MODULE_OK;
 	struct stat	st;
 	int		locked;
 	int		lock_count;
 	struct timeval	tv;
 	REALM		*proxy_realm;
 	char		proxy_buffer[16];
+	VALUE_PAIR	*pair = packet->vps;
 
 	struct detail_instance *inst = instance;
 
+	/*
+	 *	Nothing to log: don't do anything.
+	 */
+	if (!packet) {
+		return RLM_MODULE_NOOP;
+	}
+
 	/*
 	 *	Create a directory for this nas.
 	 *
@@ -128,7 +155,7 @@ static int do_detail(void *instance, REQUEST *request, VALUE_PAIR *pair)
 	 *	we prolly must create it the dir(s)
 	 */
 	if ((p) && (stat(buffer, &st) < 0)) {
-		*p = '\0';	
+		*p = '\0';
 		/*
 		 *	NO previously cached directory name, so we've
 		 *	got to create a new one.
@@ -140,8 +167,8 @@ static int do_detail(void *instance, REQUEST *request, VALUE_PAIR *pair)
 		 *	so we've got to create a new one.
 		 */
 		if ((inst->last_made_directory == NULL) ||
-		    (strcmp(inst->last_made_directory, buffer) != 0)) { 
-			
+		    (strcmp(inst->last_made_directory, buffer) != 0)) {
+
 			/*
 			 *	Free any previously cached name.
 			 */
@@ -149,7 +176,7 @@ static int do_detail(void *instance, REQUEST *request, VALUE_PAIR *pair)
 				free((char *) inst->last_made_directory);
 				inst->last_made_directory = NULL;
 			}
-			
+
 			/*
 			 *	Go create possibly multiple directories.
 			 */
@@ -159,13 +186,21 @@ static int do_detail(void *instance, REQUEST *request, VALUE_PAIR *pair)
 			}
 			inst->last_made_directory = strdup(buffer);
 		}
-		
-		*p = '/';	
+
+		*p = '/';
 	} /* else there was no directory delimiter. */
 
 	/*
 	 *	Open & create the file, with the given permissions.
-	 *
+	 */
+	if ((outfd = open(buffer, O_WRONLY | O_APPEND | O_CREAT,
+			  inst->detailperm)) < 0) {
+		radlog(L_ERR, "rlm_detail: Couldn't open file %s: %s",
+		       buffer, strerror(errno));
+		return RLM_MODULE_FAIL;
+	}
+
+	/*
 	 *	If we're not using locking, we'll just pass straight though
 	 *	the while loop.
 	 *	If we fail to aquire the filelock in 80 tries (approximately
@@ -174,13 +209,6 @@ static int do_detail(void *instance, REQUEST *request, VALUE_PAIR *pair)
 	locked = 0;
 	lock_count = 0;
 	do {
-		if ((outfd = open(buffer, O_WRONLY | O_APPEND | O_CREAT,
-				  inst->detailperm)) < 0) {
-			radlog(L_ERR, "rlm_detail: Couldn't open file %s: %s",
-			       buffer, strerror(errno));
-			ret = RLM_MODULE_FAIL;
-			break;
-		}
 		if (inst->locking) {
 			lseek(outfd, 0L, SEEK_SET);
 			if (rad_lockfd_nonblock(outfd, 0) < 0) {
@@ -190,7 +218,7 @@ static int do_detail(void *instance, REQUEST *request, VALUE_PAIR *pair)
 				select(0, NULL, NULL, NULL, &tv);
 				lock_count++;
 			} else {
-				DEBUG("rlm_detail: Aquired filelock, tried %d time(s)",
+				DEBUG("rlm_detail: Acquired filelock, tried %d time(s)",
 				      lock_count + 1);
 				locked = 1;
 			}
@@ -200,75 +228,108 @@ static int do_detail(void *instance, REQUEST *request, VALUE_PAIR *pair)
 	if (!locked && inst->locking && lock_count >= 80) {
 		radlog(L_ERR, "rlm_detail: Failed to aquire filelock for %s, giving up",
 		       buffer);
-		outfd = -1;
-		ret = RLM_MODULE_FAIL;
+		return RLM_MODULE_FAIL;
 	}
 
-	outfp = NULL;
-	if (outfd > -1) {
-		if ((outfp = fdopen(outfd, "a")) == NULL) {
-			radlog(L_ERR, "rlm_detail: Couldn't open file %s: %s",
-			       buffer, strerror(errno));
-			ret = RLM_MODULE_FAIL;
-			if (inst->locking) {
-				lseek(outfd, 0L, SEEK_SET);
-				rad_unlockfd(outfd, 0);
-				DEBUG("rlm_detail: Released filelock");
-			}
-			close(outfd);
+	/*
+	 *	Convert the FD to FP.  The FD is no longer valid
+	 *	after this operation.
+	 */
+	if ((outfp = fdopen(outfd, "a")) == NULL) {
+		radlog(L_ERR, "rlm_detail: Couldn't open file %s: %s",
+		       buffer, strerror(errno));
+		if (inst->locking) {
+			lseek(outfd, 0L, SEEK_SET);
+			rad_unlockfd(outfd, 0);
+			DEBUG("rlm_detail: Released filelock");
+		}
+		close(outfd);
+
+		return RLM_MODULE_FAIL;
+	}
+
+	/*
+	 *	Write the information to the file.
+	 */
+	if (!compat) {
+		/*
+		 *	Print out names, if they're OK.
+		 *	Numbers, if not.
+		 */
+		if ((packet->code > 0) &&
+		    (packet->code <= PW_ACCESS_CHALLENGE)) {
+			fprintf(outfp, "Packet-Type = %s\n",
+				packet_codes[packet->code]);
+		} else {
+			fprintf(outfp, "Packet-Type = %d\n", packet->code);
 		}
 	}
 
-	if (outfd > -1 && outfp) {
-		/* Post a timestamp */
-		fseek(outfp, 0L, SEEK_END);
-		fputs(ctime_r(&request->timestamp, buffer), outfp);
+	/*
+	 *	Post a timestamp
+	 */
+	fseek(outfp, 0L, SEEK_END);
+	fputs(CTIME_R(&request->timestamp, buffer, DIRLEN), outfp);
 
-		/* Write each attribute/value to the log file */
-		while (pair) {
-			if (pair->attribute != PW_PASSWORD) {
-				fputs("\t", outfp);
-				vp_print(outfp, pair);
-				fputs("\n", outfp);
-			}
+	/* Write each attribute/value to the log file */
+	while (pair) {
+		/*
+		 *	Don't print passwords in old format...
+		 */
+		if (compat && (pair->attribute == PW_PASSWORD)) {
 			pair = pair->next;
+			continue;
 		}
 
 		/*
-		 *	Add non-protocol attibutes.
+		 *	Print all of the attributes.
 		 */
-		if ((pair = pairfind(request->config_items, PW_PROXY_TO_REALM))
-		    != NULL) {
+		fputs("\t", outfp);
+		vp_print(outfp, pair);
+		fputs("\n", outfp);
+		pair = pair->next;
+	}
+
+	/*
+	 *	Add non-protocol attibutes.
+	 */
+	if (compat) {
+		if ((pair = pairfind(request->config_items,
+				     PW_PROXY_TO_REALM)) != NULL) {
 			proxy_realm = realm_find(pair->strvalue, TRUE);
 			if (proxy_realm) {
 				memset((char *) proxy_buffer, 0, 16);
 				ip_ntoa(proxy_buffer, proxy_realm->acct_ipaddr);
 				fprintf(outfp, "\tFreeradius-Proxied-To = %s\n",
-				        proxy_buffer);
+					proxy_buffer);
 				DEBUG("rlm_detail: Freeradius-Proxied-To set to %s",
 				      proxy_buffer);
 			}
 		}
-		fprintf(outfp, "\tTimestamp = %ld\n", request->timestamp);
+		fprintf(outfp, "\tTimestamp = %ld\n",
+			(unsigned long) request->timestamp);
+
 		if (request->packet->verified == 2)
 			fputs("\tRequest-Authenticator = Verified\n", outfp);
 		else if (request->packet->verified == 1)
 			fputs("\tRequest-Authenticator = None\n", outfp);
-		pair = NULL;
-
-		fputs("\n", outfp);
-
-		if (inst->locking) {
-			fflush(outfp);
-			lseek(outfd, 0L, SEEK_SET);
-			rad_unlockfd(outfd, 0);
-			DEBUG("rlm_detail: Released filelock");
-		}
-	
-		fclose(outfp);
 	}
 
-	return ret;
+	fputs("\n", outfp);
+
+	if (inst->locking) {
+		fflush(outfp);
+		lseek(outfd, 0L, SEEK_SET);
+		rad_unlockfd(outfd, 0);
+		DEBUG("rlm_detail: Released filelock");
+	}
+
+	fclose(outfp);
+
+	/*
+	 *	And everything is fine.
+	 */
+	return RLM_MODULE_OK;
 }
 
 /*
@@ -277,7 +338,7 @@ static int do_detail(void *instance, REQUEST *request, VALUE_PAIR *pair)
 static int detail_accounting(void *instance, REQUEST *request)
 {
 
-	return do_detail(instance,request,request->packet->vps);
+	return do_detail(instance,request,request->packet, TRUE);
 }
 
 /*
@@ -285,7 +346,7 @@ static int detail_accounting(void *instance, REQUEST *request)
  */
 static int detail_authorize(void *instance, REQUEST *request)
 {
-	return do_detail(instance,request,request->packet->vps);
+	return do_detail(instance,request,request->packet, FALSE);
 }
 
 /*
@@ -293,7 +354,35 @@ static int detail_authorize(void *instance, REQUEST *request)
  */
 static int detail_postauth(void *instance, REQUEST *request)
 {
-	return do_detail(instance,request,request->reply->vps);
+	return do_detail(instance,request,request->reply, FALSE);
+}
+
+
+/*
+ *	Outgoing Access-Request to home server - write the detail files.
+ */
+static int detail_pre_proxy(void *instance, REQUEST *request)
+{
+	if (request->proxy &&
+	    request->proxy->vps) {
+		return do_detail(instance,request,request->proxy, FALSE);
+	}
+
+	return RLM_MODULE_NOOP;
+}
+
+
+/*
+ *	Outgoing Access-Request Reply - write the detail files.
+ */
+static int detail_post_proxy(void *instance, REQUEST *request)
+{
+	if (request->proxy_reply &&
+	    request->proxy_reply->vps) {
+		return do_detail(instance,request,request->proxy_reply, FALSE);
+	}
+
+	return RLM_MODULE_NOOP;
 }
 
 
@@ -315,7 +404,7 @@ static int detail_detach(void *instance)
 /* globally exported name */
 module_t rlm_detail = {
 	"detail",
-	0,				/* type: reserved */
+	RLM_TYPE_THREAD_UNSAFE,        /* type: reserved */
 	NULL,				/* initialization */
 	detail_instantiate,		/* instantiation */
 	{
@@ -324,8 +413,8 @@ module_t rlm_detail = {
 		NULL,			/* preaccounting */
 		detail_accounting,	/* accounting */
 		NULL,			/* checksimul */
-		NULL,			/* pre-proxy */
-		NULL,			/* post-proxy */
+		detail_pre_proxy,      	/* pre-proxy */
+		detail_post_proxy,	/* post-proxy */
 		detail_postauth		/* post-auth */
 	},
 	detail_detach,			/* detach */
