@@ -1,7 +1,7 @@
 /*
- * cb.c 
+ * cb.c
  *
- * Version:     $Id: cb.c,v 1.8 2003/05/08 10:37:29 wichert Exp $
+ * Version:     $Id: cb.c,v 1.16 2004/05/27 22:10:17 mgriego Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -29,17 +29,17 @@ void cbtls_info(const SSL *s, int where, int ret)
 	int w;
 
 	w = where & ~SSL_ST_MASK;
-	if (w & SSL_ST_CONNECT) str="TLS_connect";
-	else if (w & SSL_ST_ACCEPT) str="TLS_accept";
-	else str="undefined";
+	if (w & SSL_ST_CONNECT) str="    TLS_connect";
+	else if (w & SSL_ST_ACCEPT) str="    TLS_accept";
+	else str="    (other)";
 
 	state = SSL_state_string_long(s);
 	state = state ? state : "NULL";
 
 	if (where & SSL_CB_LOOP) {
-		radlog(L_INFO, "%s: %s\n", str, state);
+		if (debug_flag) radlog(L_INFO, "%s: %s\n", str, state);
 	} else if (where & SSL_CB_HANDSHAKE_START) {
-		radlog(L_INFO, "%s: %s\n", str, state);
+		if (debug_flag) radlog(L_INFO, "%s: %s\n", str, state);
 	} else if (where & SSL_CB_HANDSHAKE_DONE) {
 		radlog(L_INFO, "%s: %s\n", str, state);
 	} else if (where & SSL_CB_ALERT) {
@@ -56,66 +56,72 @@ void cbtls_info(const SSL *s, int where, int ret)
 }
 
 /*
- * Before trusting a certificate, you must make sure that the certificate is
-   'valid'. There are several steps that your application can take in 
-   determining if a certificate is valid. Commonly used steps are: 
-
-  1.Verifying the certificate's signature, and verifying that the certificate
-    has been issued by a trusted Certificate Authority. 
-
-  2.Verifying that the certificate is valid for the present date (i.e. it is
-    being presented within its validity dates). 
-
-  3.Verifying that the certificate has not been revoked by its issuing
-    Certificate Authority, by checking with respect to a Certificate
-    Revocation List (CRL). 
-
-  4.Verifying that the credentials presented by the certificate fulfill
-    additional requirements specific to the application, such as with respect
-    to access control lists or with respect to OCSP (Online Certificate Status
-    Processing). 
- */
-/*
- * NOTE: This callback will be called multiple times based on the 
- * depth of the root certificate chain
+ *	Before trusting a certificate, you must make sure that the
+ *	certificate is 'valid'. There are several steps that your
+ *	application can take in determining if a certificate is
+ *	valid. Commonly used steps are:
+ *
+ *	1.Verifying the certificate's signature, and verifying that
+ *	the certificate has been issued by a trusted Certificate
+ *	Authority.
+ *
+ *	2.Verifying that the certificate is valid for the present date
+ *	(i.e. it is being presented within its validity dates).
+ *
+ *	3.Verifying that the certificate has not been revoked by its
+ *	issuing Certificate Authority, by checking with respect to a
+ *	Certificate Revocation List (CRL).
+ *
+ *	4.Verifying that the credentials presented by the certificate
+ *	fulfill additional requirements specific to the application,
+ *	such as with respect to access control lists or with respect
+ *	to OCSP (Online Certificate Status Processing).
+ *
+ *	NOTE: This callback will be called multiple times based on the
+ *	depth of the root certificate chain
  */
 int cbtls_verify(int ok, X509_STORE_CTX *ctx)
 {
 	char subject[256]; /* Used for the subject name */
 	char issuer[256]; /* Used for the issuer name */
-	char buf[256]; 
-	char *user_name = NULL; /* User-Name */
+	char buf[256];
+	char cn_str[256];
+	EAP_HANDLER *handler = NULL;
 	X509 *client_cert;
 	SSL *ssl;
 	int err, depth;
-	int data_index = 0;
+	EAP_TLS_CONF *conf;
+	int my_ok = ok;
 
 	client_cert = X509_STORE_CTX_get_current_cert(ctx);
 	err = X509_STORE_CTX_get_error(ctx);
 	depth = X509_STORE_CTX_get_error_depth(ctx);
 
-	if(!ok)
+	if(!my_ok)
 		radlog(L_ERR,"--> verify error:num=%d:%s\n",err,
 			X509_verify_cert_error_string(err));
 	/*
-	Catch too long Certificate chains
-	*/
+	 *	Catch too long Certificate chains
+	 */
 
 	/*
 	 * Retrieve the pointer to the SSL of the connection currently treated
 	 * and the application specific data stored into the SSL object.
 	 */
 	ssl = X509_STORE_CTX_get_ex_data(ctx, SSL_get_ex_data_X509_STORE_CTX_idx());
-	user_name = (char *)SSL_get_ex_data(ssl, data_index);
+	handler = (EAP_HANDLER *)SSL_get_ex_data(ssl, 0);
+	conf = (EAP_TLS_CONF *)SSL_get_ex_data(ssl, 1);
 
 	/*
-	 * Get the Subject & Issuer
+	 *	Get the Subject & Issuer
 	 */
 	subject[0] = issuer[0] = '\0';
 	X509_NAME_oneline(X509_get_subject_name(client_cert), subject, 256);
 	X509_NAME_oneline(X509_get_issuer_name(ctx->current_cert), issuer, 256);
 
-	/* Get the Common Name */
+	/*
+	 *	Get the Common Name
+	 */
 	X509_NAME_get_text_by_NID(X509_get_subject_name(client_cert),
              NID_commonName, buf, 256);
 
@@ -127,32 +133,63 @@ int cbtls_verify(int ok, X509_STORE_CTX *ctx)
 	case X509_V_ERR_CERT_NOT_YET_VALID:
 	case X509_V_ERR_ERROR_IN_CERT_NOT_BEFORE_FIELD:
 		radlog(L_ERR, "notBefore=");
-		//ASN1_TIME_print(bio_err, X509_get_notBefore(ctx->current_cert));
+#if 0
+		ASN1_TIME_print(bio_err, X509_get_notBefore(ctx->current_cert));
+#endif
 		break;
 	case X509_V_ERR_CERT_HAS_EXPIRED:
 	case X509_V_ERR_ERROR_IN_CERT_NOT_AFTER_FIELD:
 		radlog(L_ERR, "notAfter=");
-		//ASN1_TIME_print(bio_err, X509_get_notAfter(ctx->current_cert));
+#if 0
+		ASN1_TIME_print(bio_err, X509_get_notAfter(ctx->current_cert));
+#endif
 		break;
 	}
 
-	radlog(L_INFO, "chain-depth=%d, ", depth);
 	/*
-	if (depth > 0) {
-		return ok;
+	 *	If we're at the actual client cert and the conf tells
+	 *	us to, check the CN in the cert against the xlat'ed
+	 *	value
+	 */
+	if (depth == 0 && conf->check_cert_cn != NULL) {
+		if (!radius_xlat(cn_str, sizeof(cn_str), conf->check_cert_cn, handler->request, NULL)) {
+			radlog(L_ERR, "rlm_eap_tls (%s): xlat failed.",
+                               conf->check_cert_cn);
+			/* if this fails, fail the verification */
+			my_ok = 0;
+		}
+		DEBUG2("    rlm_eap_tls: checking certificate CN (%s) with xlat'ed value (%s)", buf, cn_str);
+		if (strncmp(cn_str, buf, sizeof(buf)) != 0) {
+			my_ok = 0;
+			radlog(L_AUTH, "rlm_eap_tls: Certificate CN (%s) does not match specified value (%s)!", buf, cn_str);
+		}
 	}
-	*/
-	radlog(L_INFO, "error=%d", err);
 
-	radlog(L_INFO, "--> User-Name = %s", user_name);
-	radlog(L_INFO, "--> BUF-Name = %s", buf);
-	radlog(L_INFO, "--> subject = %s", subject);
-	radlog(L_INFO, "--> issuer  = %s", issuer);
-	radlog(L_INFO, "--> verify return:%d", ok);
-	return ok;
+	if (debug_flag > 0) {
+		radlog(L_INFO, "chain-depth=%d, ", depth);
+		/*
+		  if (depth > 0) {
+		  return ok;
+		  }
+		*/
+		radlog(L_INFO, "error=%d", err);
+
+		radlog(L_INFO, "--> User-Name = %s", handler->identity);
+		radlog(L_INFO, "--> BUF-Name = %s", buf);
+		radlog(L_INFO, "--> subject = %s", subject);
+		radlog(L_INFO, "--> issuer  = %s", issuer);
+		radlog(L_INFO, "--> verify return:%d", my_ok);
+	}
+	return my_ok;
 }
 
-void cbtls_msg(int write_p, int msg_version, int content_type, const void *buf, size_t len, SSL *ssl, void *arg)
+
+/*
+ *	Fill in our 'info' with TLS data.
+ */
+void cbtls_msg(int write_p, int msg_version, int content_type,
+	       const void *buf, size_t len,
+	       SSL *ssl UNUSED, void *arg)
 {
 	tls_session_t *state = (tls_session_t *)arg;
 
@@ -160,13 +197,14 @@ void cbtls_msg(int write_p, int msg_version, int content_type, const void *buf,
 	state->info.content_type = (unsigned char)content_type;
 	state->info.record_len = len;
 	state->info.version = msg_version;
+	state->info.initialized = 1;
 
-	if (content_type == 21) {
+	if (content_type == SSL3_RT_ALERT) {
 		state->info.alert_level = ((const unsigned char*)buf)[0];
 		state->info.alert_description = ((const unsigned char*)buf)[1];
 		state->info.handshake_type = 0x00;
-	
-	} else if (content_type == 22) {
+
+	} else if (content_type == SSL3_RT_HANDSHAKE) {
 		state->info.handshake_type = ((const unsigned char*)buf)[0];
 		state->info.alert_level = 0x00;
 		state->info.alert_description = 0x00;
@@ -174,18 +212,20 @@ void cbtls_msg(int write_p, int msg_version, int content_type, const void *buf,
 	tls_session_information(state);
 }
 
-int cbtls_password(char *buf, int num, int rwflag, void *userdata)
+int cbtls_password(char *buf,
+		   int num UNUSED,
+		   int rwflag UNUSED,
+		   void *userdata)
 {
 	strcpy(buf, (char *)userdata);
 	return(strlen((char *)userdata));
 }
 
-RSA *cbtls_rsa(SSL *s, int is_export, int keylength)
+RSA *cbtls_rsa(SSL *s UNUSED, int is_export UNUSED, int keylength)
 {
 	static RSA *rsa_tmp=NULL;
 
-	if (rsa_tmp == NULL)
-	{
+	if (rsa_tmp == NULL) {
 		radlog(L_INFO, "Generating temp (%d bit) RSA key...", keylength);
 		rsa_tmp=RSA_generate_key(keylength, RSA_F4, NULL, NULL);
 	}
@@ -193,4 +233,3 @@ RSA *cbtls_rsa(SSL *s, int is_export, int keylength)
 }
 
 #endif /* !defined(NO_OPENSSL) */
-
