@@ -1,7 +1,7 @@
 /*
  * eap_tls.c
  *
- * Version:     $Id: eap_tls.c,v 1.13 2003/04/04 20:06:12 aland Exp $
+ * Version:     $Id: eap_tls.c,v 1.22.2.1 2004/09/01 19:30:48 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -18,11 +18,12 @@
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * Copyright 2001  hereUare Communications, Inc. <raghud@hereuare.com>
+ * Copyright 2003  Alan DeKok <aland@freeradius.org>
  */
 
 /*
  *
- *  TLS Packet Format in EAP 
+ *  TLS Packet Format in EAP
  *  --- ------ ------ -- ---
  * 0                   1                   2                   3
  * 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
@@ -77,14 +78,14 @@ void eaptls_free(EAPTLS_PACKET **eaptls_packet_ptr)
    The S flag is set only within the EAP-TLS start message
    sent from the EAP server to the peer.
 */
-int eaptls_start(EAP_DS *eap_ds)
+int eaptls_start(EAP_DS *eap_ds, int peap_flag)
 {
 	EAPTLS_PACKET 	reply;
 
 	reply.code = EAPTLS_START;
 	reply.length = TLS_HEADER_LEN + 1/*flags*/;
 
-	reply.flags = 0x00;
+	reply.flags = peap_flag;
 	reply.flags = SET_START(reply.flags);
 
 	reply.data = NULL;
@@ -95,13 +96,13 @@ int eaptls_start(EAP_DS *eap_ds)
 	return 1;
 }
 
-int eaptls_success(EAP_DS *eap_ds)
+int eaptls_success(EAP_DS *eap_ds, int peap_flag)
 {
 	EAPTLS_PACKET	reply;
 
 	reply.code = EAPTLS_SUCCESS;
 	reply.length = TLS_HEADER_LEN;
-	reply.flags = 0x00;
+	reply.flags = peap_flag;
 	reply.data = NULL;
 	reply.dlen = 0;
 
@@ -110,13 +111,13 @@ int eaptls_success(EAP_DS *eap_ds)
 	return 1;
 }
 
-int eaptls_fail(EAP_DS *eap_ds)
+int eaptls_fail(EAP_DS *eap_ds, int peap_flag)
 {
 	EAPTLS_PACKET	reply;
 
 	reply.code = EAPTLS_FAIL;
 	reply.length = TLS_HEADER_LEN;
-	reply.flags = 0x00;
+	reply.flags = peap_flag;
 	reply.data = NULL;
 	reply.dlen = 0;
 
@@ -128,13 +129,14 @@ int eaptls_fail(EAP_DS *eap_ds)
 /*
    A single TLS record may be up to 16384 octets in length, but a TLS
    message may span multiple TLS records, and a TLS certificate message
-   may in principle be as long as 16MB. 
+   may in principle be as long as 16MB.
 */
+
 /*
- * Frame the Dirty data that needs to be send to the client in an EAP-Request.
- * We always embed the TLS-length in all EAP-TLS packets 
- * that we send, for easy reference purpose.
- * Handles fragmentation and sending the next fragment etc.,
+ *	Frame the Dirty data that needs to be send to the client in an
+ *	EAP-Request.  We always embed the TLS-length in all EAP-TLS
+ *	packets that we send, for easy reference purpose.  Handle
+ *	fragmentation and sending the next fragment etc.
  */
 int eaptls_request(EAP_DS *eap_ds, tls_session_t *ssn)
 {
@@ -143,17 +145,17 @@ int eaptls_request(EAP_DS *eap_ds, tls_session_t *ssn)
 	unsigned int 	nlen;
 	unsigned int 	lbit = 0;
 
-	/* This value determines whether we set (L)ength flag for 
-		EVERY packet we send and add corresponding 
+	/* This value determines whether we set (L)ength flag for
+		EVERY packet we send and add corresponding
 		"TLS Message Length" field.
 
 	length_flag = TRUE;
 		This means we include L flag and "TLS Msg Len" in EVERY
 		packet we send out.
-       
+
 	length_flag = FALSE;
 		This means we include L flag and "TLS Msg Len" **ONLY**
-		in First packet of a fragment series. We do not use 
+		in First packet of a fragment series. We do not use
 		it anywhere else.
 
 		Having L flag in every packet is prefered.
@@ -167,7 +169,7 @@ int eaptls_request(EAP_DS *eap_ds, tls_session_t *ssn)
 	}
 
 	reply.code = EAPTLS_REQUEST;
-	reply.flags = 0x00;
+	reply.flags = ssn->peap_flag;
 
 	/* Send data, NOT more than the FRAGMENT size */
 	if (ssn->dirty_out.used > ssn->offset) {
@@ -192,7 +194,7 @@ int eaptls_request(EAP_DS *eap_ds, tls_session_t *ssn)
 		memcpy(reply.data, &nlen, lbit);
 		reply.flags = SET_LENGTH_INCLUDED(reply.flags);
 	}
-	record_minus(&ssn->dirty_out, reply.data+lbit, size);
+	record_minus(&ssn->dirty_out, reply.data + lbit, size);
 
 	eaptls_compose(eap_ds, &reply);
 	free(reply.data);
@@ -206,72 +208,75 @@ int eaptls_request(EAP_DS *eap_ds, tls_session_t *ssn)
  * 1. Handshake completed Message, so now send, EAP-Success
  * 2. Alert Message, now send, EAP-Failure
  * 3. Fragment Message, now send, next Fragment
- *
- * After Success/Failure, We are done with the Session, free all the resources
  */
-eaptls_status_t eaptls_ack_handler(EAP_HANDLER *handler)
+static eaptls_status_t eaptls_ack_handler(EAP_HANDLER *handler)
 {
 	tls_session_t *tls_session;
 
 	tls_session = (tls_session_t *)handler->opaque;
-	if ((tls_session == NULL) || (tls_session->info.origin == 0)) {
-
+	if (tls_session == NULL){
 		radlog(L_ERR, "rlm_eap_tls: Unexpected ACK received");
-		return EAPTLS_NOOP;
+		return EAPTLS_FAIL;
+	}
+	if (tls_session->info.initialized == 0) {
+		DEBUG("  rlm_eap_tls: No SSL info available. Waiting for more SSL data.");
+		return EAPTLS_REQUEST;
+	}
+	if (tls_session->info.origin == 0) {
+		radlog(L_ERR, "rlm_eap_tls: Unexpected ACK received");
+		return EAPTLS_FAIL;
 	}
 
 	switch (tls_session->info.content_type) {
-
 	case alert:
-		eaptls_fail(handler->eap_ds);
-		session_free(&handler->opaque);
+		DEBUG2("  rlm_eap_tls: ack alert");
+		eaptls_fail(handler->eap_ds, tls_session->peap_flag);
 		return EAPTLS_FAIL;
 
 	case handshake:
 		if (tls_session->info.handshake_type == finished) {
-			eaptls_success(handler->eap_ds);
-			eaptls_gen_mppe_keys(handler->reply_vps, 
-					     tls_session->ssl);
-			session_free(&handler->opaque);
+			DEBUG2("  rlm_eap_tls: ack handshake is finished");
 			return EAPTLS_SUCCESS;
-		} else if (tls_session->fragment > 0) {
-			/* Fragmentation handler, send next fragment */
-			eaptls_request(handler->eap_ds, tls_session);
-			return EAPTLS_REQUEST;
 		}
-		/*
-		 * For the rest of the conditions,
-		 * switch over to the default section below.
-		 */
 
+		DEBUG2("  rlm_eap_tls: ack handshake fragment handler");
+		/* Fragmentation handler, send next fragment */
+		return EAPTLS_REQUEST;
+
+		/*
+		 *	For the rest of the conditions, switch over
+		 *	to the default section below.
+		 */
 	default:
-		radlog(L_ERR, "rlm_eap_tls: Invalid ACK received");
-		session_free(&handler->opaque);
-		return EAPTLS_NOOP;
+		DEBUG2("  rlm_eap_tls: ack default");
+		radlog(L_ERR, "rlm_eap_tls: Invalid ACK received: %d",
+		       tls_session->info.content_type);
+		return EAPTLS_FAIL;
 	}
 }
 
 /*
-   Similarly, when the EAP server receives an EAP-Response with the M
-   bit set, it MUST respond with an EAP-Request with EAP-Type=EAP-TLS
-   and no data. This serves as a fragment ACK. 
-
-   In order to prevent errors in the processing of fragments, the EAP
-   server MUST use increment the Identifier value for each fragment ACK
-   contained within an EAP-Request, and the peer MUST include this
-   Identifier value in the subsequent fragment contained within an EAP-
-   Reponse.
-
- * EAP server sends an ACK when it determines there are More fragments 
- * to receive to make the complete TLS-record/TLS-Message
+ *	Similarly, when the EAP server receives an EAP-Response with
+ *	the M bit set, it MUST respond with an EAP-Request with
+ *	EAP-Type=EAP-TLS and no data. This serves as a fragment ACK.
+ *
+ *	In order to prevent errors in the processing of fragments, the
+ *	EAP server MUST use increment the Identifier value for each
+ *	fragment ACK contained within an EAP-Request, and the peer
+ *	MUST include this Identifier value in the subsequent fragment
+ *	contained within an EAP- Reponse.
+ *
+ *	EAP server sends an ACK when it determines there are More
+ *	fragments to receive to make the complete
+ *	TLS-record/TLS-Message
  */
-int eaptls_send_ack(EAP_DS *eap_ds)
+static int eaptls_send_ack(EAP_DS *eap_ds, int peap_flag)
 {
 	EAPTLS_PACKET 	reply;
 
 	reply.code = EAPTLS_ACK;
 	reply.length = TLS_HEADER_LEN + 1/*flags*/;
-	reply.flags = 0x00;
+	reply.flags = peap_flag;
 	reply.data = NULL;
 	reply.dlen = 0;
 
@@ -281,64 +286,83 @@ int eaptls_send_ack(EAP_DS *eap_ds)
 }
 
 /*
-   The S flag is set only within the EAP-TLS start message
-   sent from the EAP server to the peer.
-
-   Similarly, when the EAP server receives an EAP-Response with the M
-   bit set, it MUST respond with an EAP-Request with EAP-Type=EAP-TLS
-   and no data. This serves as a fragment ACK. The EAP peer MUST wait
+ *	The S flag is set only within the EAP-TLS start message sent
+ *	from the EAP server to the peer.
+ *
+ *	Similarly, when the EAP server receives an EAP-Response with
+ *	the M bit set, it MUST respond with an EAP-Request with
+ *	EAP-Type=EAP-TLS and no data. This serves as a fragment
+ *	ACK. The EAP peer MUST wait.
  */
-eaptls_status_t eaptls_verify(EAP_DS *eap_ds, EAP_DS *prev_eap_ds)
+static eaptls_status_t eaptls_verify(EAP_HANDLER *handler)
 {
+	EAP_DS *eap_ds = handler->eap_ds;
+	EAP_DS *prev_eap_ds = handler->prev_eapds;
 	eaptls_packet_t	*eaptls_packet, *eaptls_prev = NULL;
 
-	if ((eap_ds == NULL) 					|| 
-		(eap_ds->response == NULL)			|| 
-		(eap_ds->response->code != PW_EAP_RESPONSE)	||
-		(eap_ds->response->length <= EAP_HEADER_LEN + 1/*EAP-Type*/)	||
-		(eap_ds->response->type.type != PW_EAP_TLS)) {
-
-		radlog(L_ERR, "rlm_eap_tls: corrupted data");
-		return EAPTLS_INVALID;
-	}
-
+	/*
+	 *	We don't check ANY of the input parameters.  It's all
+	 *	code which works together, so if something is wrong,
+	 *	we SHOULD core dump.
+	 *
+	 *	e.g. if eap_ds is NULL, of if eap_ds->response is
+	 *	NULL, of if it's NOT an EAP-Response, or if the packet
+	 *	is too short.  See eap_validation()., in ../../eap.c
+	 *
+	 *	Also, eaptype_select() takes care of selecting the
+	 *	appropriate type, so we don't need to check
+	 *	eap_ds->response->type.type == PW_EAP_TLS, or anything
+	 *	else.
+	 */
 	eaptls_packet = (eaptls_packet_t *)eap_ds->response->type.data;
 	if (prev_eap_ds && prev_eap_ds->response)
 		eaptls_prev = (eaptls_packet_t *)prev_eap_ds->response->type.data;
 
 	/*
-	 * check for ACK
-	 * 1. Find if this is a reply to the previous request sent
-	 * 2. If typedata[0] == NULL && length == EAP_HEADER_LEN && ALLTLSFLAGS == 0
+	 *	check for ACK
+	 *
+	 *	If there's no TLS data, or there's 1 byte of TLS data,
+	 *	with the flags set to zero, then it's an ACK.
+	 *
+	 *	Find if this is a reply to the previous request sent
 	 */
-	if ((eap_ds->response->length == EAP_HEADER_LEN + 2/*EAPtype+flags*/) && 
-		((eaptls_packet != NULL) && (eaptls_packet->flags == 0x00))) {
+	if ((eaptls_packet == NULL) ||
+	    ((eap_ds->response->length == EAP_HEADER_LEN + 2) &&
+	     ((eaptls_packet->flags & 0xc0) == 0x00))) {
 
 		if (prev_eap_ds->request->id == eap_ds->response->id) {
+			/*
+			 *	Run the ACK handler directly from here.
+			 */
 			radlog(L_INFO, "rlm_eap_tls: Received EAP-TLS ACK message");
-			return EAPTLS_ACK;
+			return eaptls_ack_handler(handler);
 		} else {
 			radlog(L_ERR, "rlm_eap_tls: Received Invalid EAP-TLS ACK message");
 			return EAPTLS_INVALID;
 		}
 	}
 
-	/* We donot receive a TLS_START but we send it.  */
-	if (TLS_START(eaptls_packet->flags) == 1) {
-		radlog(L_ERR, "rlm_eap_tls:  Received EAP-TLS Start message");
-		//return EAPTLS_START;
+	/*
+	 *	We send TLS_START, but do not receive it.
+	 */
+	if (TLS_START(eaptls_packet->flags)) {
+		radlog(L_ERR, "rlm_eap_tls:  Received unexpected EAP-TLS Start message");
 		return EAPTLS_INVALID;
 	}
 
 	/*
-      The L bit (length included) is set to indicate the presence of the
-      four octet TLS Message Length field, and MUST be set for the first
-      fragment of a fragmented TLS message or set of messages. The M bit
-      (more fragments) is set on all but the last fragment. The S bit
-      (EAP-TLS start) is set in an EAP-TLS Start message. This
-      differentiates the EAP-TLS Start message from a fragment
-      acknowledgement.
-	*/
+	 *	The L bit (length included) is set to indicate the
+	 *	presence of the four octet TLS Message Length field,
+	 *	and MUST be set for the first fragment of a fragmented
+	 *	TLS message or set of messages.
+	 *
+	 *	The M bit (more fragments) is set on all but the last
+	 *	fragment.
+	 *
+	 *	The S bit (EAP-TLS start) is set in an EAP-TLS Start
+	 *	message. This differentiates the EAP-TLS Start message
+	 *	from a fragment acknowledgement.
+	 */
 	if (TLS_LENGTH_INCLUDED(eaptls_packet->flags)) {
 		if (TLS_MORE_FRAGMENTS(eaptls_packet->flags)) {
 			/*
@@ -348,8 +372,8 @@ eaptls_status_t eaptls_verify(EAP_DS *eap_ds, EAP_DS *prev_eap_ds)
 			 * 	(It is because Last fragment will not have M bit set)
 			 */
 			if ((prev_eap_ds->response == NULL) ||
-					(eaptls_prev == NULL) ||
-					(TLS_MORE_FRAGMENTS(eaptls_prev->flags) == 0)) {
+			    (eaptls_prev == NULL) ||
+			    !TLS_MORE_FRAGMENTS(eaptls_prev->flags)) {
 
 				radlog(L_INFO, "rlm_eap_tls:  Received EAP-TLS First Fragment of the message");
 				return EAPTLS_FIRST_FRAGMENT;
@@ -370,7 +394,10 @@ eaptls_status_t eaptls_verify(EAP_DS *eap_ds, EAP_DS *prev_eap_ds)
 		return EAPTLS_MORE_FRAGMENTS;
 	}
 
-	/* None of the flags are set & But is valid EAPTLS packet */
+	/*
+	 *	None of the flags are set, but it's still a valid
+	 *	EAPTLS packet.
+	 */
 	return EAPTLS_OK;
 }
 
@@ -408,7 +435,7 @@ eaptls_status_t eaptls_verify(EAP_DS *eap_ds, EAP_DS *prev_eap_ds)
  *  packet including the Code, Identifir, Length, Type, and TLS data
  *  fields.
  */
-EAPTLS_PACKET *eaptls_extract(EAP_DS *eap_ds, eaptls_status_t status)
+static EAPTLS_PACKET *eaptls_extract(EAP_DS *eap_ds, eaptls_status_t status)
 {
 	EAPTLS_PACKET	*tlspacket;
 	uint32_t	data_len = 0;
@@ -419,24 +446,29 @@ EAPTLS_PACKET *eaptls_extract(EAP_DS *eap_ds, eaptls_status_t status)
 		return NULL;
 
 	/*
-	 *	We have a 'type' after the EAP header, but no more data.
+	 *	The main EAP code & eaptls_verify() take care of
+	 *	ensuring that the packet is OK, and that we can
+	 *	extract the various fields we want.
 	 *
-	 *	For TLS, we always have 'flags'
+	 *	e.g. a TLS packet with zero data is allowed as an ACK,
+	 *	but we will never see it here, as we will simply
+	 *	send another fragment, instead of trying to extract
+	 *	the data.
+	 *
+	 *	MUST have TLS type octet, followed by flags, followed
+	 *	by data.
 	 */
-	if (eap_ds->response->length <= 2) {
-		radlog(L_ERR, "rlm_eap_tls: Invalid EAP-TLS packet received.  (No data.)");
-		return NULL;
-	}
+	rad_assert(eap_ds->response->length > 2);
 
 	tlspacket = eaptls_alloc();
 	if (tlspacket == NULL) return NULL;
 
 	/*
-	 * Code & id for EAPTLS & EAP are same
-	 * but eaptls_length = eap_length - 1(EAP-Type = 1 octet)
+	 *	Code & id for EAPTLS & EAP are same
+	 *	but eaptls_length = eap_length - 1(EAP-Type = 1 octet)
 	 *
-	 * length = code + id + length + type + tlsdata
-	 *        =  1   +  1 +   2    +  1    +  X
+	 *	length = code + id + length + type + tlsdata
+	 *	       =  1   +  1 +   2    +  1    +  X
 	 */
 	tlspacket->code = eap_ds->response->code;
 	tlspacket->id = eap_ds->response->id;
@@ -447,42 +479,66 @@ EAPTLS_PACKET *eaptls_extract(EAP_DS *eap_ds, eaptls_status_t status)
 	 *	A quick sanity check of the flags.  If we've been told
 	 *	that there's a length, and there isn't one, then stop.
 	 */
-	if ((tlspacket->flags & 0x80) &&
+	if (TLS_LENGTH_INCLUDED(tlspacket->flags) &&
 	    (tlspacket->length < 5)) { /* flags + TLS message length */
 		radlog(L_ERR, "rlm_eap_tls: Invalid EAP-TLS packet received.  (Length bit is set, but no length was found.)");
 		eaptls_free(&tlspacket);
 		return NULL;
 	}
 
+	/*
+	 *	If the final TLS packet is larger than we can handle, die
+	 *	now.
+	 *
+	 *	Likewise, if the EAP packet says N bytes, and the TLS
+	 *	packet says there's fewer bytes, it's a problem.
+	 *
+	 *	FIXME: Try to ensure that the claimed length is
+	 *	consistent across multiple TLS fragments.
+	 */
+	if (TLS_LENGTH_INCLUDED(tlspacket->flags)) {
+		memcpy(&data_len, &eap_ds->response->type.data[1], 4);
+		data_len = ntohl(data_len);
+		if (data_len > MAX_RECORD_SIZE) {
+			radlog(L_ERR, "rlm_eap_tls: The EAP-TLS packet will contain more data than we can process.");
+			eaptls_free(&tlspacket);
+			return NULL;
+		}
+
+#if 0
+		DEBUG2(" TLS: %d %d\n", data_len, tlspacket->length);
+
+		if (data_len < tlspacket->length) {
+			radlog(L_ERR, "rlm_eap_tls: EAP-TLS packet claims to be smaller than the encapsulating EAP packet.");
+			eaptls_free(&tlspacket);
+			return NULL;
+		}
+#endif
+	}
+
 	switch (status) {
 	/*
-	   The TLS Message Length field is
-	   four octets, and provides the total length of the TLS message or set
-	   of messages that is being fragmented; this simplifies buffer
-	   allocation.
-	 * For now we ignore TLS-Length field and hence the Total Length.
-	 */
-	/*
-	 * Start storing this fragment 
-	 * ???: How to interpret the Length ?
-	 *	1. Is it is the total length of all the TLS records ?
-	 *	2. Is it is the total length of one TLS record/current EAP data ?
-
-	 * Dynamic allocation of buffers as & when we know the length
-	 * should solve the problem.
+	 *	The TLS Message Length field is four octets, and
+	 *	provides the total length of the TLS message or set of
+	 *	messages that is being fragmented; this simplifies
+	 *	buffer allocation.
+	 *
+	 *	Dynamic allocation of buffers as & when we know the
+	 *	length should solve the problem.
 	 */
 	case EAPTLS_FIRST_FRAGMENT:
 	case EAPTLS_LENGTH_INCLUDED:
 	case EAPTLS_MORE_FRAGMENTS_WITH_LENGTH:
 		if (tlspacket->length < 5) { /* flags + TLS message length */
-			radlog(L_ERR, "rlm_eap_tls: Invalid EAP-TLS packet received.  (Excepted length, got none.)");
+			radlog(L_ERR, "rlm_eap_tls: Invalid EAP-TLS packet received.  (Expected length, got none.)");
 			eaptls_free(&tlspacket);
 			return NULL;
 		}
 
 		/*
-		 * Extract all the TLS fragments from the previous eap_ds
-		 * Start appending this fragment to the above ds
+		 *	Extract all the TLS fragments from the
+		 *	previous eap_ds Start appending this
+		 *	fragment to the above ds
 		 */
 		memcpy(&data_len, &eap_ds->response->type.data[1], sizeof(uint32_t));
 		data_len = ntohl(data_len);
@@ -493,7 +549,6 @@ EAPTLS_PACKET *eaptls_extract(EAP_DS *eap_ds, eaptls_status_t status)
 		 *	Hmm... this should be an error, too.
 		 */
 		if (data_len > len) {
-			radlog(L_INFO, "Total Length Included");
 			data_len = len;
 		}
 		break;
@@ -527,27 +582,222 @@ EAPTLS_PACKET *eaptls_extract(EAP_DS *eap_ds, eaptls_status_t status)
 	return tlspacket;
 }
 
+
+
 /*
- * compose the TLS reply packet in the EAP reply typedata
+ * To process the TLS,
+ *  INCOMING DATA:
+ * 	1. EAP-TLS should get the compelete TLS data from the peer.
+ * 	2. Store that data in a data structure with any other required info
+ *	3. Handle that data structure to the TLS module.
+ *	4. TLS module will perform its operations on the data and
+ *	handle back to EAP-TLS
+ *
+ *  OUTGOING DATA:
+ * 	1. EAP-TLS if necessary will fragment it and send it to the
+ * 	destination.
+ *
+ *	During EAP-TLS initialization, TLS Context object will be
+ *	initialized and stored.  For every new authentication
+ *	requests, TLS will open a new session object and that session
+ *	object should be maintained even after the session is
+ *	completed for session resumption. (Probably later as a feature
+ *	as we donot know who maintains these session objects ie,
+ *	SSL_CTX (internally) or TLS module(explicitly). If TLS module,
+ *	then how to let SSL API know about these sessions.)
+ */
+static void eaptls_operation(EAPTLS_PACKET *eaptls_packet UNUSED,
+			     eaptls_status_t status, EAP_HANDLER *handler)
+{
+	tls_session_t *tls_session;
+
+	tls_session = (tls_session_t *)handler->opaque;
+
+	if ((status == EAPTLS_MORE_FRAGMENTS) ||
+	    (status == EAPTLS_MORE_FRAGMENTS_WITH_LENGTH) ||
+	    (status == EAPTLS_FIRST_FRAGMENT)) {
+		/*
+		 * Send the ACK.
+		 */
+		eaptls_send_ack(handler->eap_ds, tls_session->peap_flag);
+	} else {
+		/*
+		 *	We have the complete TLS-data or TLS-message.
+		 *
+		 *	Clean the dirty message.
+		 *
+		 *	Authenticate the user and send
+		 *	Success/Failure.
+		 *
+		 *	If more info
+		 *	is required then send another request.  */
+		if (tls_handshake_recv(tls_session)) {
+			/*
+			 *	FIXME: return success/fail.
+			 *
+			 *	TLS proper can decide what to do, then.
+			 */
+			eaptls_request(handler->eap_ds, tls_session);
+		} else {
+			eaptls_fail(handler->eap_ds, tls_session->peap_flag);
+		}
+	}
+	return;
+}
+
+
+/*
+ * In the actual authentication first verify the packet and then create the data structure
+ */
+/*
+ * To process the TLS,
+ *  INCOMING DATA:
+ * 	1. EAP-TLS should get the compelete TLS data from the peer.
+ * 	2. Store that data in a data structure with any other required info
+ *	3. Hand this data structure to the TLS module.
+ *	4. TLS module will perform its operations on the data and hands back to EAP-TLS
+ *  OUTGOING DATA:
+ * 	1. EAP-TLS if necessary will fragment it and send it to the destination.
+ *
+ *	During EAP-TLS initialization, TLS Context object will be
+ *	initialized and stored.  For every new authentication
+ *	requests, TLS will open a new session object and that
+ *	session object SHOULD be maintained even after the session
+ *	is completed, for session resumption. (Probably later as a
+ *	feature, as we do not know who maintains these session
+ *	objects ie, SSL_CTX (internally) or TLS module (explicitly). If
+ *	TLS module, then how to let SSL API know about these
+ *	sessions.)
+ */
+
+/*
+ *	Process an EAP request
+ */
+eaptls_status_t eaptls_process(EAP_HANDLER *handler)
+{
+	tls_session_t *tls_session = (tls_session_t *) handler->opaque;
+	EAPTLS_PACKET	*tlspacket;
+	eaptls_status_t	status;
+
+	DEBUG2("  rlm_eap_tls: processing TLS");
+
+	/* This case is when SSL generates Alert then we
+	 * send that alert to the client and then send the EAP-Failure
+	 */
+	status = eaptls_verify(handler);
+	DEBUG2("  eaptls_verify returned %d\n", status);
+
+	switch (status) {
+	default:
+	case EAPTLS_INVALID:
+	case EAPTLS_FAIL:
+
+		/*
+		 *	Success means that we're done the initial
+		 *	handshake.  For TTLS, this means send stuff
+		 *	back to the client, and the client sends us
+		 *	more tunneled data.
+		 */
+	case EAPTLS_SUCCESS:
+		return status;
+		break;
+
+		/*
+		 *	Normal TLS request, continue with the "get rest
+		 *	of fragments" phase.
+		 */
+	case EAPTLS_REQUEST:
+		eaptls_request(handler->eap_ds, tls_session);
+		return EAPTLS_HANDLED;
+		break;
+
+		/*
+		 *	The handshake is done, and we're in the "tunnel
+		 *	data" phase.
+		 */
+	case EAPTLS_OK:
+		DEBUG2("  rlm_eap_tls: Done initial handshake");
+
+		/*
+		 *	Get the rest of the fragments.
+		 */
+	case EAPTLS_FIRST_FRAGMENT:
+	case EAPTLS_MORE_FRAGMENTS:
+	case EAPTLS_LENGTH_INCLUDED:
+	case EAPTLS_MORE_FRAGMENTS_WITH_LENGTH:
+		break;
+	}
+
+	/*
+	 *	Extract the TLS packet from the buffer.
+	 */
+	if ((tlspacket = eaptls_extract(handler->eap_ds, status)) == NULL)
+		return EAPTLS_FAIL;
+
+	/*
+	 *	Get the session struct from the handler
+	 *
+	 *	update the dirty_in buffer
+	 *
+	 *	NOTE: This buffer will contain partial data when M bit is set.
+	 *
+	 * 	CAUTION while reinitializing this buffer, it should be
+	 * 	reinitialized only when this M bit is NOT set.
+	 */
+	if (tlspacket->dlen !=
+	    record_plus(&tls_session->dirty_in, tlspacket->data, tlspacket->dlen)) {
+		eaptls_free(&tlspacket);
+		radlog(L_ERR, "rlm_eap_tls: Exceeded maximum record size");
+		return EAPTLS_FAIL;
+	}
+
+	/*
+	 *	SSL initalization is done.  Return.
+	 *
+	 *	The TLS data will be in the tls_session structure.
+	 */
+	if (SSL_is_init_finished(tls_session->ssl)) {
+		eaptls_free(&tlspacket);
+		return EAPTLS_OK;
+	}
+
+	/*
+	 *	Continue the handshake.
+	 */
+	eaptls_operation(tlspacket, status, handler);
+
+	eaptls_free(&tlspacket);
+	return EAPTLS_HANDLED;
+}
+
+
+/*
+ *	compose the TLS reply packet in the EAP reply typedata
  */
 int eaptls_compose(EAP_DS *eap_ds, EAPTLS_PACKET *reply)
 {
 	uint8_t *ptr;
 
-	eap_ds->request->type.type = PW_EAP_TLS;
+	/*
+	 *	Don't set eap_ds->request->type.type, as the main EAP
+	 *	handler will do that for us.  This allows the TLS
+	 *	module to be called from TTLS & PEAP.
+	 */
 
-/*
-   Similarly, when the EAP server receives an EAP-Response with the M
-   bit set, it MUST respond with an EAP-Request with EAP-Type=EAP-TLS
-   and no data. This serves as a fragment ACK. The EAP peer MUST wait
-   until it receives the EAP-Request before sending another fragment.
-
-   In order to prevent errors in the processing of fragments, the EAP
-   server MUST use increment the Identifier value for each fragment ACK
-   contained within an EAP-Request, and the peer MUST include this
-   Identifier value in the subsequent fragment contained within an EAP-
-   Reponse.
-*/
+	/*
+	 * 	When the EAP server receives an EAP-Response with the
+	 * 	M bit set, it MUST respond with an EAP-Request with
+	 * 	EAP-Type=EAP-TLS and no data. This serves as a
+	 * 	fragment ACK. The EAP peer MUST wait until it receives
+	 * 	the EAP-Request before sending another fragment.
+	 *
+	 *	In order to prevent errors in the processing of
+	 *	fragments, the EAP server MUST use increment the
+	 *	Identifier value for each fragment ACK contained
+	 *	within an EAP-Request, and the peer MUST include this
+	 *	Identifier value in the subsequent fragment contained
+	 *	within an EAP- Reponse.
+	 */
 	eap_ds->request->type.data = malloc(reply->length - TLS_HEADER_LEN + 1);
 	if (eap_ds->request->type.data == NULL) {
 		radlog(L_ERR, "rlm_eap_tls: out of memory");
@@ -582,62 +832,3 @@ int eaptls_compose(EAP_DS *eap_ds, EAPTLS_PACKET *reply)
 
 	return 1;
 }
-
-/*
- * To process the TLS,
- *  INCOMING DATA:
- * 	1. EAP-TLS should get the compelete TLS data from the peer.
- * 	2. Store that data in a data structure with any other required info
- *	3. Handle that data structure to the TLS module.
- *	4. TLS module will perform its operations on the data and handle back to EAP-TLS
- *  OUTGOING DATA:
- * 	1. EAP-TLS if necessary will fragment it and send it to the destination.
- *
- * During EAP-TLS initialization, TLS Context object will be initialized and stored.
- * For every new authentication requests, TLS will open a new session object and that
- * session object should be maintained even after the session is completed for
- * session resumption. (Probably later as a feature as we donot know who maintains these
- * session objects ie, SSL_CTX (internally) or TLS module(explicitly). If TLS module, then
- * how to let SSL API know about these sessions.)
- */
-void eaptls_operation(EAPTLS_PACKET *eaptls_packet, eaptls_status_t status, EAP_HANDLER *handler)
-{
-	tls_session_t *tls_session;
-
-	tls_session = (tls_session_t *)handler->opaque;
-	/*
-	 * Get the session struct from the handler 
-	 * update the dirty_in buffer
-	 * NOTE: This buffer will contain partial data when M bit is set.
-	 * 	CAUTION while reinitializing this buffer.
-	 * 	It should be reinitialized only when this M bit is NOT set.
-	 */
-	if (eaptls_packet->dlen != 
-			record_plus(&tls_session->dirty_in, eaptls_packet->data, eaptls_packet->dlen)) {
-			radlog(L_ERR, "rlm_eap_tls: Exceeded maximum record size");
-			return;
-	}
-
-	if ((status == EAPTLS_MORE_FRAGMENTS) || 
-		(status == EAPTLS_MORE_FRAGMENTS_WITH_LENGTH) || 
-		(status == EAPTLS_FIRST_FRAGMENT)) {
-		/*
-		 * Send the ACK.
-		 */
-		eaptls_send_ack(handler->eap_ds);
-	} else {
-		/*
-		 * We have the complete TLS-data or TLS-message
-		 * Clean the dirty message
-		 * Authenticate the user and send Success/Failure
-		 * If more info is required then send another request.
-		 */
-		if (tls_handshake_recv(tls_session)) {
-			eaptls_request(handler->eap_ds, tls_session);
-		} else {
-			eaptls_fail(handler->eap_ds);
-		}
-		//record_init(&tls_session->dirty_in);
-	}
-	return;
-}
