@@ -1,7 +1,7 @@
 /*
- * rlm_mschap.c	
+ * rlm_mschap.c
  *
- * Version:	$Id: rlm_mschap.c,v 1.41.2.1 2003/09/16 18:40:56 phampson Exp $
+ * Version:	$Id: rlm_mschap.c,v 1.59.2.1 2004/09/21 14:32:02 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@
  *  aka
  *  ZARAZA		3APA3A@security.nnov.ru
  */
- 
+
 /*  MPPE support from Takahiro Wagatsuma <waga@sic.shibaura-it.ac.jp> */
 
 #include	"autoconf.h"
@@ -58,9 +58,9 @@
 #include	"sha1.h"
 #include	"rad_assert.h"
 
-#define PW_MSCHAP_RESPONSE	((311 << 16) | 1)
-#define PW_MSCHAP_CHALLENGE	((311 << 16) | 11)
-#define PW_MSCHAP2_RESPONSE	((311 << 16) | 25)
+#include	"smbdes.h"
+
+static const char rcsid[] = "$Id: rlm_mschap.c,v 1.59.2.1 2004/09/21 14:32:02 aland Exp $";
 
 static const char *letters = "0123456789ABCDEF";
 
@@ -71,7 +71,7 @@ static int hex2bin (const char *szHex, unsigned char* szBin, int len)
 {
 	char * c1, * c2;
 	int i;
-   
+
    	for (i = 0; i < len; i++) {
 		if( !(c1 = memchr(letters, toupper((int) szHex[i << 1]), 16)) ||
 		    !(c2 = memchr(letters, toupper((int) szHex[(i << 1) + 1]), 16)))
@@ -84,7 +84,7 @@ static int hex2bin (const char *szHex, unsigned char* szBin, int len)
 /*
  *	bin2hex creates hexadecimal presentation
  *	of binary data
- */ 
+ */
 static void bin2hex (const unsigned char *szBin, char *szHex, int len)
 {
 	int i;
@@ -127,7 +127,7 @@ static int pdb_decode_acct_ctrl(const char *p)
 			  break;
 
 			case 'D':  /* 'D'isabled. */
-			  acct_ctrl |= ACB_DISABLED ; 
+			  acct_ctrl |= ACB_DISABLED ;
 			  break;
 
 			case 'H':  /* 'H'omedir required. */
@@ -171,7 +171,7 @@ static int pdb_decode_acct_ctrl(const char *p)
 
 			case ':':
 			case '\n':
-			case '\0': 
+			case '\0':
 			case ']':
 			default:
 			  finished = 1;
@@ -190,7 +190,7 @@ static int pdb_decode_acct_ctrl(const char *p)
 static void ntpwdhash (char *szHash, const char *szPassword)
 {
 	char szUnicodePass[513];
-	char nPasswordLen;
+	int nPasswordLen;
 	int i;
 
 	/*
@@ -219,7 +219,7 @@ static void challenge_hash( const char *peer_challenge,
 {
 	SHA1_CTX Context;
 	char hash[20];
-	
+
 	SHA1Init(&Context);
 	SHA1Update(&Context, peer_challenge, 16);
 	SHA1Update(&Context, auth_challenge, 16);
@@ -228,36 +228,24 @@ static void challenge_hash( const char *peer_challenge,
 	memcpy(challenge, hash, 8);
 }
 
-static void mschap2(const char *peer_challenge, const char *auth_challenge,
-		    const char *user_name, const char *nt_password,
-		    char *response)
-{
-	char challenge[8];
-	
-	challenge_hash(peer_challenge, auth_challenge, user_name,
-		       challenge);
-
-	lrad_mschap(nt_password, challenge, response);
-}
-
 /*
  *	auth_response() generates MS-CHAP v2 SUCCESS response
  *	according to RFC 2759 GenerateAuthenticatorResponse()
  *	returns 42-octet response string
  */
-static void auth_response(const char *username, const char *nt_password,
+static void auth_response(const char *username,
+			  const char *nt_hash_hash,
 			  char *ntresponse,
 			  char *peer_challenge, char *auth_challenge,
 			  char *response)
 {
 	SHA1_CTX Context;
-	char hashhash[16];
 	const char magic1[39] =
 	{0x4D, 0x61, 0x67, 0x69, 0x63, 0x20, 0x73, 0x65, 0x72, 0x76,
 	 0x65, 0x72, 0x20, 0x74, 0x6F, 0x20, 0x63, 0x6C, 0x69, 0x65,
 	 0x6E, 0x74, 0x20, 0x73, 0x69, 0x67, 0x6E, 0x69, 0x6E, 0x67,
 	 0x20, 0x63, 0x6F, 0x6E, 0x73, 0x74, 0x61, 0x6E, 0x74};
-                                             
+
 	const char magic2[41] =
 	{0x50, 0x61, 0x64, 0x20, 0x74, 0x6F, 0x20, 0x6D, 0x61, 0x6B,
 	 0x65, 0x20, 0x69, 0x74, 0x20, 0x64, 0x6F, 0x20, 0x6D, 0x6F,
@@ -268,14 +256,8 @@ static void auth_response(const char *username, const char *nt_password,
         char challenge[8];
         char digest[20];
 
-	/*
-	 * Hash password hash into hashhash
-	 */
-
-	md4_calc(hashhash, nt_password, 16);
-
 	SHA1Init(&Context);
-	SHA1Update(&Context, hashhash, 16);
+	SHA1Update(&Context, nt_hash_hash, 16);
 	SHA1Update(&Context, ntresponse, 24);
 	SHA1Update(&Context, magic1, 39);
 	SHA1Final(digest, &Context);
@@ -287,40 +269,289 @@ static void auth_response(const char *username, const char *nt_password,
 	SHA1Final(digest, &Context);
 
 	/*
-	 * Encode the value of 'Digest' as "S=" followed by
-	 * 40 ASCII hexadecimal digits and return it in
-	 * AuthenticatorResponse.
-	 * For example,
-	 *   "S=0123456789ABCDEF0123456789ABCDEF01234567"
+	 *	Encode the value of 'Digest' as "S=" followed by
+	 *	40 ASCII hexadecimal digits and return it in
+	 *	AuthenticatorResponse.
+	 *	For example,
+	 *	"S=0123456789ABCDEF0123456789ABCDEF01234567"
 	 */
  	response[0] = 'S';
 	response[1] = '=';
 	bin2hex(digest, response + 2, 20);
 }
 
-struct mschap_instance {
+
+typedef struct rlm_mschap_t {
 	int use_mppe;
 	int require_encryption;
-	int require_strong;
+        int require_strong;
+        int with_ntdomain_hack;	/* this should be in another module */
 	char *passwd_file;
-	char *auth_type;
-};
+	char *xlat_name;
+	char *auth_type;	/* I don't think this is needed... */
+	char *ntlm_auth;
+} rlm_mschap_t;
+
+
+/*
+ *	Does dynamic translation of strings.
+ *
+ *	Pulls NT-Response, LM-Response, or Challenge from MSCHAP
+ *	attributes.
+ */
+static int mschap_xlat(void *instance, REQUEST *request,
+		       char *fmt, char *out, size_t outlen,
+		       RADIUS_ESCAPE_STRING func)
+{
+	int		i, data_len;
+	uint8_t		*data = NULL;
+	uint8_t		buffer[8];
+	VALUE_PAIR	*user_name;
+	VALUE_PAIR	*chap_challenge, *response;
+	rlm_mschap_t	*inst = instance;
+
+	chap_challenge = response = NULL;
+
+	func = func;		/* -Wunused */
+
+	/*
+	 *	Challenge means MS-CHAPv1 challenge, or
+	 *	hash of MS-CHAPv2 challenge, and peer challenge.
+	 */
+	if (strcasecmp(fmt, "Challenge") == 0) {
+		chap_challenge = pairfind(request->packet->vps,
+					  PW_MSCHAP_CHALLENGE);
+		if (!chap_challenge) {
+			DEBUG2("  rlm_mschap: No MS-CHAP-Challenge in the request.");
+			return 0;
+		}
+
+		/*
+		 *	MS-CHAP-Challenges are 8 octets,
+		 *	for MS-CHAPv2
+		 */
+		if (chap_challenge->length == 8) {
+			DEBUG2(" mschap1: %02x", chap_challenge->strvalue[0]);
+			data = chap_challenge->strvalue;
+			data_len = 8;
+
+			/*
+			 *	MS-CHAP-Challenges are 16 octets,
+			 *	for MS-CHAPv2.
+			 */
+		} else if (chap_challenge->length == 16) {
+			char *username_string;
+
+			DEBUG2(" mschap2: %02x", chap_challenge->strvalue[0]);
+			response = pairfind(request->packet->vps,
+					    PW_MSCHAP2_RESPONSE);
+			if (!response) {
+				DEBUG2("  rlm_mschap: MS-CHAP2-Response is required to calculate MS-CHAPv1 challenge.");
+				return 0;
+			}
+
+			/*
+			 *	Responses are 50 octets.
+			 */
+			if (response->length < 50) {
+				radlog(L_AUTH, "rlm_mschap: MS-CHAP-Response has the wrong format.");
+				return 0;
+			}
+
+			user_name = pairfind(request->packet->vps,
+					     PW_USER_NAME);
+			if (!user_name) {
+				DEBUG2("  rlm_mschap: User-Name is required to calculateMS-CHAPv1 Challenge.");
+				return 0;
+			}
+
+			/*
+			 *	with_ntdomain_hack moved here, too.
+			 */
+			if ((username_string = strchr(user_name->strvalue, '\\')) != NULL) {
+				if (inst->with_ntdomain_hack) {
+					username_string++;
+				} else {
+					DEBUG2("  rlm_mschap: NT Domain delimeter found, should we have enabled with_ntdomain_hack?");
+					username_string = user_name->strvalue;
+				}
+			} else {
+				username_string = user_name->strvalue;
+			}
+
+			/*
+			 *	Get the MS-CHAPv1 challenge
+			 *	from the MS-CHAPv2 peer challenge,
+			 *	our challenge, and the user name.
+			 */
+			challenge_hash(response->strvalue + 2,
+				       chap_challenge->strvalue,
+				       username_string, buffer);
+			data = buffer;
+			data_len = 8;
+		} else {
+			DEBUG2("  rlm_mschap: Invalid MS-CHAP challenge length");
+			return 0;
+		}
+		
+		/*
+		 *	Get the MS-CHAPv1 response, or the MS-CHAPv2
+		 *	response.
+		 */
+	} else if (strcasecmp(fmt, "NT-Response") == 0) {
+		response = pairfind(request->packet->vps,
+				    PW_MSCHAP_RESPONSE);
+		if (!response) response = pairfind(request->packet->vps,
+						   PW_MSCHAP2_RESPONSE);
+		if (!response) {
+			DEBUG2("  rlm_mschap: No MS-CHAP-Response or MS-CHAP2-Response was found in the request.");
+			return 0;
+		}
+
+		/*
+		 *	For MS-CHAPv1, the NT-Response exists only
+		 *	if the second octet says so.
+		 */
+		if ((response->attribute == PW_MSCHAP_RESPONSE) &&
+		    ((response->strvalue[1] & 0x01) == 0)) {
+			DEBUG2("  rlm_mschap: No NT-Response in MS-CHAP-Response");
+			return 0;
+		}
+
+		/*
+		 *	MS-CHAP-Response and MS-CHAP2-Response have
+		 *	the NT-Response at the same offset, and are
+		 *	the same length.
+		 */
+		data = response->strvalue + 26;
+		data_len = 24;
+		
+		/*
+		 *	LM-Response is deprecated, and exists only
+		 *	in MS-CHAPv1, and not often there.
+		 */
+	} else if (strcasecmp(fmt, "LM-Response") == 0) {
+		response = pairfind(request->packet->vps,
+				    PW_MSCHAP_RESPONSE);
+		if (!response) {
+			DEBUG2("  rlm_mschap: No MS-CHAP-Response was found in the request.");
+			return 0;
+		}
+
+		/*
+		 *	For MS-CHAPv1, the NT-Response exists only
+		 *	if the second octet says so.
+		 */
+		if ((response->strvalue[1] & 0x01) != 0) {
+			DEBUG2("  rlm_mschap: No LM-Response in MS-CHAP-Response");
+			return 0;
+		}
+		data = response->strvalue + 2;
+		data_len = 24;
+
+		/*
+		 *	Pull the NT-Domain out of the User-Name, if it exists.
+		 */
+	} else if (strcasecmp(fmt, "NT-Domain") == 0) {
+		char *p;
+
+		user_name = pairfind(request->packet->vps, PW_USER_NAME);
+		if (!user_name) {
+			DEBUG2("  rlm_mschap: No User-Name was found in the request.");
+			return 0;
+		}
+		
+		p = strchr(user_name->strvalue, '\\');
+		if (!p) {
+			DEBUG2("  rlm_mschap: No NT-Domain was found in the User-Name.");
+			return 0;
+		}
+
+		/*
+		 *	Hack.  This is simpler than the alternatives.
+		 */
+		*p = '\0';
+		strNcpy(out, user_name->strvalue, outlen);
+		*p = '\\';
+
+		return strlen(out);
+
+		/*
+		 *	Pull the User-Name out of the User-Name...
+		 */
+	} else if (strcasecmp(fmt, "User-Name") == 0) {
+		char *p;
+
+		user_name = pairfind(request->packet->vps, PW_USER_NAME);
+		if (!user_name) {
+			DEBUG2("  rlm_mschap: No User-Name was found in the request.");
+			return 0;
+		}
+		
+		p = strchr(user_name->strvalue, '\\');
+		if (p) {
+			p++;	/* skip the backslash */
+		} else {
+			p = user_name->strvalue; /* use the whole User-Name */
+		}
+
+		strNcpy(out, p, outlen);
+		return strlen(out);
+
+	} else {
+		DEBUG2("  rlm_mschap: Unknown expansion string \"%s\"",
+		       fmt);
+		return 0;
+	}
+
+	if (outlen == 0) return 0; /* nowhere to go, don't do anything */
+
+	/*
+	 *	Didn't set anything: this is bad.
+	 */
+	if (!data) {
+		DEBUG2("  rlm_mschap: Failed to do anything intelligent");
+		return 0;
+	}
+
+	/*
+	 *	Check the output length.
+	 */
+	if (outlen < ((data_len * 2) + 1)) {
+		data_len = (outlen - 1) / 2;
+	}
+
+	/*
+	 *	
+	 */
+	for (i = 0; i < data_len; i++) {
+		sprintf(out + (2 * i), "%02x", data[i]);
+	}
+	out[data_len * 2] = '\0';
+	
+	return data_len * 2;
+}
+
 
 static CONF_PARSER module_config[] = {
 	/*
 	 *	Cache the password by default.
 	 */
 	{ "use_mppe",    PW_TYPE_BOOLEAN,
-	  offsetof(struct mschap_instance,use_mppe), NULL, "yes" },
+	  offsetof(rlm_mschap_t,use_mppe), NULL, "yes" },
 	{ "require_encryption",    PW_TYPE_BOOLEAN,
-	  offsetof(struct mschap_instance,require_encryption), NULL, "no" },
+	  offsetof(rlm_mschap_t,require_encryption), NULL, "no" },
 	{ "require_strong",    PW_TYPE_BOOLEAN,
-	  offsetof(struct mschap_instance,require_strong), NULL, "no" },
+	  offsetof(rlm_mschap_t,require_strong), NULL, "no" },
+	{ "with_ntdomain_hack",     PW_TYPE_BOOLEAN,
+	  offsetof(rlm_mschap_t,with_ntdomain_hack), NULL, "no" },
 	{ "passwd",   PW_TYPE_STRING_PTR,
-	  offsetof(struct mschap_instance, passwd_file), NULL,  NULL },
+	  offsetof(rlm_mschap_t, passwd_file), NULL,  NULL },
 	{ "authtype",   PW_TYPE_STRING_PTR,
-	  offsetof(struct mschap_instance, auth_type), NULL,  NULL },
-	
+	  offsetof(rlm_mschap_t, auth_type), NULL,  NULL },
+	{ "ntlm_auth",   PW_TYPE_STRING_PTR,
+	  offsetof(rlm_mschap_t, ntlm_auth), NULL,  NULL },
+
 	{ NULL, -1, 0, NULL, NULL }		/* end the list */
 };
 
@@ -329,21 +560,27 @@ static CONF_PARSER module_config[] = {
  *	mschap_instantiate()
  */
 static int mschap_detach(void *instance){
-#define inst ((struct mschap_instance *)instance)
+#define inst ((rlm_mschap_t *)instance)
 	if (inst->passwd_file) free(inst->passwd_file);
 	if (inst->auth_type) free(inst->auth_type);
+	if (inst->ntlm_auth) free(inst->ntlm_auth);
+	if (inst->xlat_name) {
+		xlat_unregister(inst->xlat_name, mschap_xlat);
+		free(inst->xlat_name);
+	}
 	free(instance);
 	return 0;
 #undef inst
 }
-	
+
 /*
  *	Create instance for our module. Allocate space for
  *	instance structure and read configuration parameters
  */
 static int mschap_instantiate(CONF_SECTION *conf, void **instance)
 {
-	struct mschap_instance *inst;
+	const char *xlat_name;
+	rlm_mschap_t *inst;
 
 	inst = *instance = rad_malloc(sizeof(*inst));
 	if (!inst) {
@@ -368,6 +605,17 @@ static int mschap_instantiate(CONF_SECTION *conf, void **instance)
 		return -1;
 	}
 
+	/*
+	 *	Create the dynamic translation.
+	 */
+	xlat_name = cf_section_name2(conf);
+	if (xlat_name == NULL)
+		xlat_name = cf_section_name1(conf);
+	if (xlat_name){
+		inst->xlat_name = strdup(xlat_name);
+		xlat_register(xlat_name, mschap_xlat, inst);
+	}
+
 	return 0;
 }
 
@@ -394,7 +642,7 @@ static void add_reply(VALUE_PAIR** vp, unsigned char ident,
 /*
  *	Add MPPE attributes to the reply.
  */
-static void mppe_add_reply(VALUE_PAIR** vp,
+static void mppe_add_reply(VALUE_PAIR **vp,
 			   const char* name, const char* value, int len)
 {
        VALUE_PAIR *reply_attr;
@@ -409,6 +657,128 @@ static void mppe_add_reply(VALUE_PAIR** vp,
        pairadd(vp, reply_attr);
 }
 
+
+/*
+ *	Do the MS-CHAP stuff.
+ *
+ *	This function is here so that all of the MS-CHAP related
+ *	authentication is in one place, and we can perhaps later replace
+ *	it with code to call winbindd, or something similar.
+ */
+static int do_mschap(rlm_mschap_t *inst,
+		     REQUEST *request, VALUE_PAIR *password,
+		     uint8_t *challenge, uint8_t *response,
+		     uint8_t *nthashhash)
+{
+	int		do_ntlm_auth = 0;
+	uint8_t		calculated[24];
+	VALUE_PAIR	*vp = NULL;
+
+	/*
+	 *	If we have ntlm_auth configured, use it unless told
+	 *	otherwise
+	 */
+	if (inst->ntlm_auth) do_ntlm_auth = 1;
+
+	/*
+	 *	If we have an ntlm_auth configuration, then we may
+	 *	want to use it.
+	 */
+	vp = pairfind(request->config_items,
+		      PW_MS_CHAP_USE_NTLM_AUTH);
+	if (vp) do_ntlm_auth = vp->lvalue;
+
+	/*
+	 *	No ntlm_auth configured, attribute to tell us to
+	 *	use it exists, and we're told to use it.  We don't
+	 *	know what to do...
+	 */
+	if (!inst->ntlm_auth && do_ntlm_auth) {
+		DEBUG2("  rlm_mschap: Asked to use ntlm_auth, but it was not configured in the mschap{} section.");
+		return -1;
+	}
+
+	/*
+	 *	Do normal authentication.
+	 */
+	if (!do_ntlm_auth) {
+		/*
+		 *	No password: can't do authentication.
+		 */
+		if (!password) {
+			DEBUG2("  rlm_mschap: FAILED: No NT/LM-Password.  Cannot perform authentication.");
+			return -1;
+		}
+		
+		smbdes_mschap(password->strvalue, challenge, calculated);
+		if (memcmp(response, calculated, 24) != 0) {
+			return -1;
+		}
+		
+		/*
+		 *	If the password exists, and is an NT-Password,
+		 *	then calculate the hash of the NT hash.  Doing this
+		 *	here minimizes work for later.
+		 */
+		if (password && (password->attribute == PW_NT_PASSWORD)) {
+			md4_calc(nthashhash, password->strvalue, 16);
+		} else {
+			memset(nthashhash, 0, 16);
+		}
+	} else {		/* run ntlm_auth */
+		int	result;
+		char	buffer[256];
+
+		memset(nthashhash, 0, 16);
+
+		/*
+		 *	Run the program, and expect that we get 16 
+		 */
+		result = radius_exec_program(inst->ntlm_auth, request,
+					     TRUE, /* wait */
+					     buffer, sizeof(buffer),
+					     NULL, NULL);
+		if (result != 0) {
+			DEBUG2("  rlm_mschap: External script failed.");
+			return -1;
+		}
+
+		/*
+		 *	Parse the answer as an nthashhash.
+		 *
+		 *	ntlm_auth currently returns:
+		 *	NT_KEY: 000102030405060708090a0b0c0d0e0f
+		 */
+		if (memcmp(buffer, "NT_KEY: ", 8) != 0) {
+			DEBUG2("  rlm_mschap: Invalid output from ntlm_auth: expecting NT_KEY");
+			return -1;
+		}
+
+		/*
+		 *	Check the length.  It should be at least 32,
+		 *	with an LF at the end.
+		 */
+		if (strlen(buffer + 8) < 32) {
+			DEBUG2("  rlm_mschap: Invalid output from ntlm_auth: NT_KEY has unexpected length");
+			return -1;
+		}
+
+		/*
+		 *	Update the NT hash hash, from the NT key.
+		 */
+		if (hex2bin(buffer + 8, nthashhash, 16) != 16) {
+			DEBUG2("  rlm_mschap: Invalid output from ntlm_auth: NT_KEY has non-hex values");
+			return -1;
+		}
+	}
+
+	return 0;
+}
+
+
+/*
+ *	Data for the hashes.
+ */
 static const uint8_t SHSpad1[40] =
                { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -492,7 +862,7 @@ static void mppe_GetAsymmetricStartKey(uint8_t *masterkey,uint8_t *sesskey,
 
 
 static void mppe_chap2_get_keys128(uint8_t *nt_hashhash,uint8_t *nt_response,
-                               uint8_t *sendkey,uint8_t *recvkey)
+				   uint8_t *sendkey,uint8_t *recvkey)
 {
        uint8_t masterkey[16];
 
@@ -505,16 +875,11 @@ static void mppe_chap2_get_keys128(uint8_t *nt_hashhash,uint8_t *nt_response,
 /*
  *	Generate MPPE keys.
  */
-static void mppe_chap2_gen_keys128(uint8_t *secret,uint8_t *vector,
-                               uint8_t *nt_hash,uint8_t *response,
-                               uint8_t *sendkey,uint8_t *recvkey)
+static void mppe_chap2_gen_keys128(uint8_t *nt_hashhash,uint8_t *response,
+				   uint8_t *sendkey,uint8_t *recvkey)
 {
 	uint8_t enckey1[16];
 	uint8_t enckey2[16];
-	/* uint8_t salt[2]; */
-	uint8_t nt_hashhash[16];
-
-	md4_calc(nt_hashhash,nt_hash,16);
 
 	mppe_chap2_get_keys128(nt_hashhash,response,enckey1,enckey2);
 
@@ -536,7 +901,7 @@ static void mppe_chap2_gen_keys128(uint8_t *secret,uint8_t *vector,
  */
 static int mschap_authorize(void * instance, REQUEST *request)
 {
-#define inst ((struct mschap_instance *)instance)
+#define inst ((rlm_mschap_t *)instance)
 	VALUE_PAIR *challenge = NULL, *response = NULL;
 	VALUE_PAIR *vp;
 	const char *authtype_name = "MS-CHAP";
@@ -545,7 +910,7 @@ static int mschap_authorize(void * instance, REQUEST *request)
 	if (!challenge) {
 		return RLM_MODULE_NOOP;
 	}
-		
+
 	response = pairfind(request->packet->vps, PW_MSCHAP_RESPONSE);
 	if (!response)
 		response = pairfind(request->packet->vps, PW_MSCHAP2_RESPONSE);
@@ -565,7 +930,7 @@ static int mschap_authorize(void * instance, REQUEST *request)
 		authtype_name = inst->auth_type;
 	}
 
-	DEBUG2("  rlm_mschap: Found MS-CHAP attributes.  Setting 'Auth-Type := %s'", authtype_name);
+	DEBUG2("  rlm_mschap: Found MS-CHAP attributes.  Setting 'Auth-Type  = %s'", authtype_name);
 
 	/*
 	 *	Set Auth-Type to MS-CHAP.  The authentication code
@@ -573,7 +938,7 @@ static int mschap_authorize(void * instance, REQUEST *request)
 	 *	NT/LM passwords.
 	 */
 	pairdelete(&request->config_items, PW_AUTHTYPE);
-	vp = pairmake("Auth-Type", authtype_name, T_OP_SET);
+	vp = pairmake("Auth-Type", authtype_name, T_OP_EQ);
 	rad_assert(vp != NULL);
 	pairadd(&request->config_items, vp);
 
@@ -588,7 +953,7 @@ static int mschap_authorize(void * instance, REQUEST *request)
  *	or in configured passwd file.
  *	If one is found we will check paraneters given by NAS.
  *
- *	If PW_SMB_ACCOUNT_CTRL is not set to ACB_PWNOTREQ we must have 
+ *	If PW_SMB_ACCOUNT_CTRL is not set to ACB_PWNOTREQ we must have
  *	one of:
  *		PAP:      PW_PASSWORD or
  *		MS-CHAP:  PW_MSCHAP_CHALLENGE and PW_MSCHAP_RESPONSE or
@@ -600,17 +965,17 @@ static int mschap_authorize(void * instance, REQUEST *request)
  */
 static int mschap_authenticate(void * instance, REQUEST *request)
 {
-#define inst ((struct mschap_instance *)instance)
+#define inst ((rlm_mschap_t *)instance)
 	VALUE_PAIR *challenge = NULL, *response = NULL;
 	VALUE_PAIR *password = NULL;
 	VALUE_PAIR *lm_password, *nt_password, *smb_ctrl;
+	VALUE_PAIR *username;
 	VALUE_PAIR *reply_attr;
-	uint8_t calculated[32];
+	uint8_t nthashhash[16];
 	uint8_t msch2resp[42];
-        uint8_t mppe_sendkey[34];
-        uint8_t mppe_recvkey[34];
+	char *username_string;
 	int chap = 0;
-	
+
 	/*
 	 *	Find the SMB-Account-Ctrl attribute, or the
 	 *	SMB-Account-Ctrl-Text attribute.
@@ -673,7 +1038,7 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 		if (!lm_password) {
 			radlog(L_ERR, "No memory");
 		} else {
-			lrad_lmpwdhash(password->strvalue,
+			smbdes_lmpwdhash(password->strvalue,
 				       lm_password->strvalue);
 			lm_password->length = 16;
 			pairadd(&request->config_items, lm_password);
@@ -703,6 +1068,7 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 		nt_password = pairmake("NT-Password", "", T_OP_EQ);
 		if (!nt_password) {
 			radlog(L_ERR, "No memory");
+			return RLM_MODULE_FAIL;
 		} else {
 			ntpwdhash(nt_password->strvalue, password->strvalue);
 			nt_password->length = 16;
@@ -710,44 +1076,6 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 		}
 	}
 
-	/*
-	 *	No NT or LM Passwords, die.
-	 */
-	if (!lm_password && !nt_password) {
-		DEBUG2("  rlm_mschap: No LM-Password or NT-Password attribute found.  Cannot perform MS-CHAP authentication.");
-		return RLM_MODULE_FAIL;
-	}
-	
-	/*
-	 *	We MAY be asked to take a User-Password attribute from
-	 *	the packet, and compare it to passwords retrieved from
-	 *	an SMB Password file.
-	 */
-	password = pairfind(request->packet->vps, PW_PASSWORD);
-	if (password) {
-		if (lm_password) {
-			lrad_lmpwdhash(calculated,
-				       password->strvalue);
-			if (memcmp(calculated,
-				   lm_password->strvalue, 16) == 0) {
-				DEBUG2("  rlm_mschap: User-Password matches LM-Password.");
-				return RLM_MODULE_OK;
-			} else {
-				DEBUG2("  rlm_mschap: FAILED: User-Password does NOT match LM-Password.");
-			}
-		} else if (nt_password) {
-			ntpwdhash(calculated, password->strvalue);
-			if (memcmp(calculated,
-				   nt_password->strvalue, 16) == 0) {
-				DEBUG2("  rlm_mschap: User-Password matches NT-Password.");
-				return RLM_MODULE_OK;
-			} else {
-				DEBUG2("  rlm_mschap: FAILED: User-Password does NOT match NT-Password.");
-			}
-		}
-		return RLM_MODULE_REJECT;
-	} /* compare User-Password in packet to configured NT/LM-Password */
-
 	challenge = pairfind(request->packet->vps, PW_MSCHAP_CHALLENGE);
 	if (!challenge) {
 		DEBUG2("  rlm_mschap: No MS-CHAP-Challenge in the request");
@@ -758,6 +1086,7 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 	 *	We also require an MS-CHAP-Response.
 	 */
 	response = pairfind(request->packet->vps, PW_MSCHAP_RESPONSE);
+
 	/*
 	 *	MS-CHAP-Response, means MS-CHAPv1
 	 */
@@ -771,7 +1100,7 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 			radlog(L_AUTH, "rlm_mschap: MS-CHAP-Challenge has the wrong format.");
 			return RLM_MODULE_INVALID;
 		}
-	
+
 		/*
 		 *	Responses are 50 octets.
 		 */
@@ -779,43 +1108,37 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 			radlog(L_AUTH, "rlm_mschap: MS-CHAP-Response has the wrong format.");
 			return RLM_MODULE_INVALID;
 		}
-	    
+
 		/*
 		 *	We are doing MS-CHAP.  Calculate the MS-CHAP
 		 *	response
 		 */
 		if (response->strvalue[1] & 0x01) {
-			DEBUG2("  rlm_mschap: doing MS-CHAPv1 with NT-Password");
+			DEBUG2("  rlm_mschap: Told to do MS-CHAPv1 with NT-Password");
 			password = nt_password;
 			offset = 26;
 		} else {
-			DEBUG2("  rlm_mschap: doing MS-CHAPv1 with LM-Password");
+			DEBUG2("  rlm_mschap: Told to do MS-CHAPv1 with LM-Password");
 			password = lm_password;
 			offset = 2;
 		}
 
 		/*
-		 *	No password configured.  Die.
+		 *	Do the MS-CHAP authentication.
 		 */
-		if (!password) {
-			DEBUG2("  rlm_mschap: FAILED: No NT/LM-Password");
+		if (do_mschap(inst, request, password, challenge->strvalue,
+			      response->strvalue + offset, nthashhash) < 0) {
+			DEBUG2("  rlm_mschap: MS-CHAP-Response is incorrect.");
+			add_reply(&request->reply->vps, *response->strvalue,
+				  "MS-CHAP-Error", "E=691 R=1", 9);
 			return RLM_MODULE_REJECT;
 		}
-		
-		/*
-		 *	Calculate the expected response.
-		 */
-		lrad_mschap(password->strvalue, challenge->strvalue,
-			    calculated);
-		if (memcmp(response->strvalue + offset,
-			   calculated, 24) != 0) {
-			DEBUG("  rlm_mschap: FAILED: MS-CHAP-Response is incorrect");
-			return RLM_MODULE_FAIL;
-		}
 
 		chap = 1;
 
 	} else if ((response = pairfind(request->packet->vps, PW_MSCHAP2_RESPONSE)) != NULL) {
+		uint8_t	mschapv1_challenge[16];
+
 		/*
 		 *	MS-CHAPv2 challenges are 16 octets.
 		 */
@@ -823,7 +1146,7 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 			radlog(L_AUTH, "rlm_mschap: MS-CHAP-Challenge has the wrong format.");
 			return RLM_MODULE_INVALID;
 		}
-	
+
 		/*
 		 *	Responses are 50 octets.
 		 */
@@ -835,40 +1158,66 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 		/*
 		 *	We also require a User-Name
 		 */
-		if (!request->username) {
+		username = pairfind(request->packet->vps, PW_USER_NAME);
+		if (!username) {
 			radlog(L_AUTH, "rlm_mschap: We require a User-Name for MS-CHAPv2");
 			return RLM_MODULE_INVALID;
 		}
-	    
+
+
 		/*
-		 *	We are doing MS-CHAPv2
-		 *	We need NT hash for it to calculate response
+		 *	with_ntdomain_hack moved here
 		 */
-		if (!nt_password) {
-			DEBUG2("  rlm_mschap: No NT-Password configured.  Cannot perform MS-CHAPv2 authentication.");
-			return RLM_MODULE_INVALID;
+		if ((username_string = strchr(username->strvalue, '\\')) != NULL) {
+		        if (inst->with_ntdomain_hack) {
+			        username_string++;
+			} else {
+				DEBUG2("  rlm_mschap: NT Domain delimeter found, should we have enabled with_ntdomain_hack?");
+				username_string = username->strvalue;
+			}
+		} else {
+		        username_string = username->strvalue;
 		}
 
-		DEBUG2("  rlm_mschap: doing MS-CHAPv2 with NT-Password");
-		mschap2(response->strvalue + 2, challenge->strvalue,
-			request->username->strvalue, nt_password->strvalue,
-			calculated);
-		if (memcmp(response->strvalue + 26, calculated, 24) != 0) {
+		/*
+		 *	The old "mschapv2" function has been moved to
+		 *	here.
+		 *
+		 *	MS-CHAPv2 takes some additional data to create an
+		 *	MS-CHAPv1 challenge, and then does MS-CHAPv1.
+		 */
+		challenge_hash(response->strvalue + 2, /* peer challenge */
+			       challenge->strvalue, /* our challenge */
+			       username_string,	/* user name */
+			       mschapv1_challenge); /* resulting challenge */
+		
+		DEBUG2("  rlm_mschap: Told to do MS-CHAPv2 for %s with NT-Password",
+		       username_string);
+
+		if (do_mschap(inst, request, nt_password, mschapv1_challenge,
+			      response->strvalue + 26, nthashhash) < 0) {
 			DEBUG2("  rlm_mschap: FAILED: MS-CHAP2-Response is incorrect");
 			add_reply(&request->reply->vps, *response->strvalue,
 				  "MS-CHAP-Error", "E=691 R=1", 9);
 			return RLM_MODULE_REJECT;
 		}
 
-		auth_response(request->username->strvalue,
-			      nt_password->strvalue, calculated,
-			      response->strvalue + 2,
-			      challenge->strvalue,
-			      msch2resp);
+		/*
+		 *	Get the NT-hash-hash, if necessary
+		 */
+		if (nt_password) {
+		}
+
+		auth_response(username_string, /* without the domain */
+			      nthashhash, /* nt-hash-hash */
+			      response->strvalue + 26, /* peer response */
+			      response->strvalue + 2, /* peer challenge */
+			      challenge->strvalue, /* our challenge */
+			      msch2resp); /* calculated MPPE key */
 		add_reply( &request->reply->vps, *response->strvalue,
 			   "MS-CHAP2-Success", msch2resp, 42);
 		chap = 2;
-		
+
 	} else {		/* Neither CHAPv1 or CHAPv2 response: die */
 		radlog(L_AUTH, "rlm_mschap: No MS-CHAP response found");
 		return RLM_MODULE_INVALID;
@@ -907,6 +1256,9 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 
 	/* now create MPPE attributes */
 	if (inst->use_mppe) {
+		uint8_t mppe_sendkey[34];
+		uint8_t mppe_recvkey[34];
+
 		if (chap == 1){
 			DEBUG2("rlm_mschap: adding MS-CHAPv1 MPPE keys");
 			memset(mppe_sendkey, 0, 32);
@@ -914,26 +1266,26 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 				memcpy(mppe_sendkey, lm_password->strvalue, 8);
 			}
 
-			if (nt_password) {
-				/* 
-				 *	According to RFC 2548 we
-				 *	should send NT hash.  But in
-				 *	practice it doesn't work.
-				 *	Instead, we should send nthashhash
-				 *
-				 *	This is an error on RFC 2548.
-				 */
-				md4_calc(mppe_sendkey + 8,
-					 nt_password->strvalue, 16);
-				mppe_add_reply(&request->reply->vps,
-					       "MS-CHAP-MPPE-Keys",
-					       mppe_sendkey, 32);
-			}
+			/*
+			 *	According to RFC 2548 we
+			 *	should send NT hash.  But in
+			 *	practice it doesn't work.
+			 *	Instead, we should send nthashhash
+			 *
+			 *	This is an error on RFC 2548.
+			 */
+			/*
+			 *	do_mschap cares to zero nthashhash if NT hash
+			 *	is not available.
+			 */
+			memcpy(mppe_sendkey + 8,
+			       nthashhash, 16);
+			mppe_add_reply(&request->reply->vps,
+				       "MS-CHAP-MPPE-Keys",
+				       mppe_sendkey, 32);
 		} else if (chap == 2) {
 			DEBUG2("rlm_mschap: adding MS-CHAPv2 MPPE keys");
-			mppe_chap2_gen_keys128(request->secret,
-					       request->packet->vector,
-					       nt_password->strvalue,
+			mppe_chap2_gen_keys128(nthashhash,
 					       response->strvalue + 26,
 					       mppe_sendkey, mppe_recvkey);
 			
@@ -943,7 +1295,7 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 			mppe_add_reply(&request->reply->vps,
 				       "MS-MPPE-Send-Key",
 				       mppe_sendkey, 16);
-			
+
 		}
 		reply_attr = pairmake("MS-MPPE-Encryption-Policy",
 				      (inst->require_encryption)? "0x00000002":"0x00000001",
@@ -955,7 +1307,7 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 				      T_OP_EQ);
 		rad_assert(reply_attr != NULL);
 		pairadd(&request->reply->vps, reply_attr);
-		
+
 	} /* else we weren't asked to use MPPE */
 
 	return RLM_MODULE_OK;
