@@ -7,7 +7,7 @@
  *		That, in fact, was again based on the original stuff
  *		from Jeph Blaize <jblaize@kiva.net> done in May 1997.
  *
- * Version:	$Id: rlm_pam.c,v 1.25 2002/01/13 11:12:05 fcusack Exp $
+ * Version:	$Id: rlm_pam.c,v 1.32 2004/02/26 19:04:33 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -31,12 +31,22 @@
 #include	"autoconf.h"
 #include	"libradius.h"
 
+#include	"config.h"
+
 #include	<stdio.h>
 #include	<stdlib.h>
 #include	<string.h>
 
+#ifdef HAVE_SECURITY_PAM_APPL_H
 #include	<security/pam_appl.h>
-#if HAVE_SYSLOG_H
+#endif
+
+#ifdef HAVE_PAM_PAM_APPL_H
+#include	<pam/pam_appl.h>
+#endif
+
+
+#ifdef HAVE_SYSLOG_H
 #include	<syslog.h>
 #endif
 
@@ -61,6 +71,10 @@ static int pam_instantiate(CONF_SECTION *conf, void **instance)
 	rlm_pam_t *data;
 
 	data = rad_malloc(sizeof(*data));
+	if (!data) {
+		return -1;
+	}
+	memset(data, 0, sizeof(*data));
 
 	if (cf_section_parse(conf, data, module_config) < 0) {
 		free(data);
@@ -111,11 +125,11 @@ static int PAM_conv (int num_msg,
   struct pam_response *reply = NULL;
   int size = sizeof(struct pam_response);
   my_PAM *pam_config = (my_PAM *) appdata_ptr;
-  
+
 #define GET_MEM if (reply) realloc(reply, size); else reply = rad_malloc(size); \
   size += sizeof(struct pam_response)
 #define COPY_STRING(s) ((s) ? strdup(s) : NULL)
-				     
+
   for (count = 0; count < num_msg; count++) {
     switch (msg[count]->msg_style) {
     case PAM_PROMPT_ECHO_ON:
@@ -157,7 +171,7 @@ static int PAM_conv (int num_msg,
  *************************************************************************/
 
 /* cjd 19980706
- * 
+ *
  * for most flexibility, passing a pamauth type to this function
  * allows you to have multiple authentication types (i.e. multiple
  * files associated with radius in /etc/pam.d)
@@ -190,7 +204,7 @@ static int pam_pass(const char *name, const char *passwd, const char *pamauth)
     if (retval != PAM_SUCCESS) {
       DEBUG("pam_pass: function pam_authenticate FAILED for <%s>. Reason: %s",
 	    name, pam_strerror(pamh, retval));
-      pam_end(pamh, 0);
+      pam_end(pamh, retval);
       return -1;
     }
 
@@ -203,13 +217,13 @@ static int pam_pass(const char *name, const char *passwd, const char *pamauth)
     if (retval != PAM_SUCCESS) {
       DEBUG("pam_pass: function pam_acct_mgmt FAILED for <%s>. Reason: %s",
 	    name, pam_strerror(pamh, retval));
-      pam_end(pamh, 0);
+      pam_end(pamh, retval);
       return -1;
     }
 #endif
 
     DEBUG("pam_pass: authentication succeeded for <%s>", name);
-    pam_end(pamh, 0);
+    pam_end(pamh, retval);
     return 0;
 }
 
@@ -260,7 +274,7 @@ static int pam_auth(void *instance, REQUEST *request)
 		     (char *)request->password->strvalue,
 		     pam_auth_string);
 
-#if HAVE_SYSLOG_H
+#ifdef HAVE_SYSLOG_H
 	if (!strcmp(radlog_dir, "syslog")) {
 		openlog(progname, LOG_PID, syslog_facility);
 	}
@@ -274,7 +288,7 @@ static int pam_auth(void *instance, REQUEST *request)
 
 module_t rlm_pam = {
   "Pam",
-  0,				/* type: reserved */
+  RLM_TYPE_THREAD_UNSAFE,	/* The PAM libraries are not thread-safe */
   NULL,				/* initialize */
   pam_instantiate,		/* instantiation */
   {
@@ -282,7 +296,10 @@ module_t rlm_pam = {
 	  NULL,			/* authorize */
 	  NULL,			/* pre-accounting */
 	  NULL,			/* accounting */
-	  NULL			/* checksimul */
+	  NULL,			/* checksimul */
+	  NULL,			/* pre-proxy */
+	  NULL,			/* post-proxy */
+	  NULL			/* post-auth */
   },
   pam_detach,			/* detach */
   NULL,				/* destroy */
