@@ -1,7 +1,7 @@
 /*
  * rlm_pap.c
  *
- * Version:  $Id: rlm_pap.c,v 1.13 2003/07/07 19:08:38 aland Exp $
+ * Version:  $Id: rlm_pap.c,v 1.15 2004/02/29 13:52:50 kkalev Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@
 	free((char *)inst->scheme); \
 	free(inst)
 
-static const char rcsid[] = "$Id: rlm_pap.c,v 1.13 2003/07/07 19:08:38 aland Exp $";
+static const char rcsid[] = "$Id: rlm_pap.c,v 1.15 2004/02/29 13:52:50 kkalev Exp $";
 
 /*
  *      Define a structure for our module configuration.
@@ -85,7 +85,7 @@ static void pap_hexify(char *buffer, char *str, int len)
 	char *pch = str;
 	char ch;
 	int i;
-	
+
 	for(i = 0;i < len; i ++) {
 		ch = pch[i];
 		buffer[2*i] = pap_hextab[(ch>>4) & 15];
@@ -178,14 +178,17 @@ static int pap_authenticate(void *instance, REQUEST *request)
 	}
 
 	if (request->password->length == 0) {
-		radlog(L_ERR, "rlm_pap: empty password supplied");
+		snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: empty password supplied");
+		module_fmsg_vp = pairmake("Module-Failure-Message", module_fmsg, T_OP_EQ);
+		pairadd(&request->packet->vps, module_fmsg_vp);
 		return RLM_MODULE_INVALID;
 	}
 
-	DEBUG("rlm_pap: login attempt by \"%s\" with password %s", 
+	DEBUG("rlm_pap: login attempt by \"%s\" with password %s",
 		request->username->strvalue, request->password->strvalue);
 
-	if (((passwd_item = pairfind(request->config_items, PW_PASSWORD)) == NULL) ||
+	if ((((passwd_item = pairfind(request->config_items, PW_PASSWORD)) == NULL) &&
+		((passwd_item = pairfind(request->config_items, PW_CRYPT_PASSWORD)) == NULL)) ||
 	    (passwd_item->length == 0) || (passwd_item->strvalue[0] == 0)) {
 		DEBUG("rlm_pap: No password (or empty password) to check against for for user %s",request->username->strvalue);
 		snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: User password not available");
@@ -193,10 +196,16 @@ static int pap_authenticate(void *instance, REQUEST *request)
 		pairadd(&request->packet->vps, module_fmsg_vp);
 		return RLM_MODULE_INVALID;
 	}
+	if (passwd_item->attribute == PW_CRYPT_PASSWORD){
+		if (inst->sch != PAP_ENC_CRYPT){
+			radlog(L_ERR, "rlm_pap: Crypt-Password attribute but encryption scheme is not set to CRYPT");
+			return RLM_MODULE_FAIL;
+		}	
+	}
 
 	DEBUG("rlm_pap: Using password \"%s\" for user %s authentication.",
 	      passwd_item->strvalue, request->username->strvalue);
-	
+
 	if (inst->sch == PAP_ENC_INVALID || inst->sch > PAP_MAX_ENC){
 		radlog(L_ERR, "rlm_pap: Wrong password scheme");
 		return RLM_MODULE_FAIL;
@@ -303,7 +312,7 @@ static int pap_detach(void *instance)
  *	is single-threaded.
  */
 module_t rlm_pap = {
-	"PAP",	
+	"PAP",
 	0,				/* type */
 	NULL,				/* initialization */
 	pap_instantiate,		/* instantiation */
