@@ -3,7 +3,7 @@
  *		Contains the functions for the "huntgroups" and "hints"
  *		files.
  *
- * Version:     $Id: rlm_preprocess.c,v 1.52 2004/05/15 15:30:13 mgriego Exp $
+ * Version:     $Id: rlm_preprocess.c,v 1.52.2.1 2004/10/07 20:52:31 pnixon Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-static const char rcsid[] = "$Id: rlm_preprocess.c,v 1.52 2004/05/15 15:30:13 mgriego Exp $";
+static const char rcsid[] = "$Id: rlm_preprocess.c,v 1.52.2.1 2004/10/07 20:52:31 pnixon Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -116,7 +116,7 @@ static void cisco_vsa_hack(VALUE_PAIR *vp)
 
 	for ( ; vp != NULL; vp = vp->next) {
 		vendorcode = (vp->attribute >> 16); /* HACK! */
-		if (vendorcode != 9) continue; /* not a Cisco VSA, continue */
+		if (!((vendorcode == 9) || (vendorcode == 6618))) continue; /* not a Cisco or Quintum VSA, continue */
 
 		if (vp->type != PW_TYPE_STRING) continue;
 
