@@ -25,26 +25,32 @@ typedef struct sql_config {
 	char   *sql_nas_table;
 	char   *sql_dict_table;
 	char   *query_user;
+	char   *default_profile;
 	char   *authorize_check_query;
 	char   *authorize_reply_query;
 	char   *authorize_group_check_query;
 	char   *authorize_group_reply_query;
-	char   *authenticate_query;
 	char   *accounting_onoff_query;
 	char   *accounting_update_query;
+	char   *accounting_update_query_alt;
 	char   *accounting_start_query;
 	char   *accounting_start_query_alt;
 	char   *accounting_stop_query;
 	char   *accounting_stop_query_alt;
 	char   *simul_count_query;
 	char   *simul_verify_query;
-	char   *simul_zap_query;
+	char   *groupmemb_query;
 	int     sqltrace;
+	int	do_clients;
 	char   *tracefile;
 	char   *xlat_name;
 	int     deletestalesessions;
 	int     num_sql_socks;
 	int     connect_failure_retry_delay;
+	int	query_on_not_found;
+	char   *sql_postauth_table;
+	char   *postauth_query;
+	char   *allowed_chars;
 
 	/* individual driver config */
 	void	*localcfg;
