@@ -117,6 +117,19 @@ CREATE TABLE usergroup (
   KEY UserName (UserName(32))
 ) ;
 
+#
+# Table structure for table 'radpostauth'
+#
+
+CREATE TABLE radpostauth (
+  id int(11) NOT NULL auto_increment,
+  user varchar(64) NOT NULL default '',
+  pass varchar(64) NOT NULL default '',
+  reply varchar(32) NOT NULL default '',
+  date timestamp(14) NOT NULL,
+  PRIMARY KEY  (id)
+) ;
+
 ######################################################################
 #
 #  The next two tables are commented out because they are not
@@ -139,16 +152,16 @@ CREATE TABLE usergroup (
 #
 # Table structure for table 'nas'
 #
-#CREATE TABLE nas (
-#  id int(10) DEFAULT '0' NOT NULL auto_increment,
-#  nasname varchar(128),
-#  shortname varchar(32),
-#  ipaddr varchar(15),
-#  type varchar(30),
-#  ports int(5),
-#  secret varchar(60),
-#  community varchar(50),
-#  snmp varchar(10),
-#  PRIMARY KEY (id)
-#);
+CREATE TABLE nas (
+  id int(10) DEFAULT '0' NOT NULL auto_increment,
+  nasname varchar(128) NOT NULL,
+  shortname varchar(32),
+  type varchar(30) DEFAULT 'other',
+  ports int(5),
+  secret varchar(60) DEFAULT 'secret' NOT NULL,
+  community varchar(50),
+  description varchar(200) DEFAULT 'RADIUS Client',
+  PRIMARY KEY (id),
+  KEY nasname (nasname)
+);
 
