@@ -1,7 +1,7 @@
 /*
  * sql_mysql.c		SQL Module
  *
- * Version:	$Id: sql_mysql.c,v 1.14.2.1 2003/07/09 20:46:02 aland Exp $
+ * Version:	$Id: sql_mysql.c,v 1.18 2004/01/22 18:23:20 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -27,11 +27,20 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <mysql/errmsg.h>
-
 #include 	"radiusd.h"
 
-#include	<mysql/mysql.h>
+#include "config.h"
+
+#ifdef HAVE_MYSQL_MYSQL_H
+#include <mysql/errmsg.h>
+#include <mysql/mysql.h>
+#else
+#ifdef HAVE_MYSQL_H
+#include <errmsg.h>
+#include <mysql.h>
+#endif
+#endif
+
 #include	"rlm_sql.h"
 
 typedef struct rlm_sql_mysql_sock {
@@ -52,8 +61,12 @@ static int sql_init_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config)
 {
 	rlm_sql_mysql_sock *mysql_sock;
 
-	sqlsocket->conn = (rlm_sql_mysql_sock *)rad_malloc(sizeof(rlm_sql_mysql_sock));
-
+	if (!sqlsocket->conn) {
+		sqlsocket->conn = (rlm_sql_mysql_sock *)rad_malloc(sizeof(rlm_sql_mysql_sock));
+		if (!sqlsocket->conn) {
+			return -1;
+		}
+	}
 	mysql_sock = sqlsocket->conn;
 	memset(mysql_sock, 0, sizeof(*mysql_sock));
 
@@ -89,9 +102,9 @@ static int sql_init_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config)
  *************************************************************************/
 static int sql_destroy_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config)
 {
-	rlm_sql_mysql_sock *mysql_sock = sqlsocket->conn;
+	free(sqlsocket->conn);
+	sqlsocket->conn = NULL;
 
-	free(mysql_sock);
 	return 0;
 }
 
