@@ -1,5 +1,5 @@
 /*
- * $Id: db_oracle.sql,v 1.4.2.3 2003/07/12 13:53:12 pnixon Exp $
+ * $Id: db_oracle.sql,v 1.5 2003/08/25 18:54:50 phampson Exp $
  *
  * Oracle schema for FreeRADIUS
  *
@@ -7,6 +7,19 @@
  * NOTE: Which columns are NULLable??
  */
 
+/*
+ * Table structure for table 'dictionary'
+ */
+CREATE TABLE dictionary (
+	id		INT PRIMARY KEY,
+	type		VARCHAR(30),
+	attribute	VARCHAR(32),
+	value		VARCHAR(32),
+	format		VARCHAR(20),
+	vendor		VARCHAR(32)
+);
+CREATE SEQUENCE dictionary_seq START WITH 1 INCREMENT BY 1;
+
 /*
  * Table structure for table 'nas'
  * This is not currently used but FreeRADIUS but is usefull for reporting
