@@ -1,7 +1,7 @@
 /*
  * sql_postgresql.c		Postgresql rlm_sql driver
  *
- * Version:	$Id: sql_postgresql.c,v 1.30 2003/06/27 15:17:52 aland Exp $
+ * Version:	$Id: sql_postgresql.c,v 1.38 2004/02/26 19:04:36 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -58,6 +58,9 @@ typedef struct rlm_sql_postgres_sock {
    char            **row;
 } rlm_sql_postgres_sock;
 
+/* Prototypes */
+static int sql_store_result(SQLSOCK * sqlsocket, SQL_CONFIG *config);
+static int sql_num_fields(SQLSOCK * sqlsocket, SQL_CONFIG *config);
 
 /* Internal function. Return true if the postgresql status value
  * indicates successful completion of the query. Return false otherwise
@@ -156,9 +159,15 @@ static int sql_init_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 		port = "";
 	}
 
-	sqlsocket->conn = (rlm_sql_postgres_sock *)rad_malloc(sizeof(rlm_sql_postgres_sock));
+	if (!sqlsocket->conn) {
+		sqlsocket->conn = (rlm_sql_postgres_sock *)rad_malloc(sizeof(rlm_sql_postgres_sock));
+		if (!sqlsocket->conn) {
+			return -1;
+		}
+	}
 
 	pg_sock = sqlsocket->conn;
+	memset(pg_sock, 0, sizeof(*pg_sock));
 
 	snprintf(connstring, sizeof(connstring),
 			"dbname=%s%s%s%s%s user=%s password=%s",
@@ -199,9 +208,16 @@ static int sql_query(SQLSOCK * sqlsocket, SQL_CONFIG *config, char *querystr) {
 	}
 
 	pg_sock->result = PQexec(pg_sock->conn, querystr);
+		/* Returns a result pointer or possibly a NULL pointer.
+		 * A non-NULL pointer will generally be returned except in
+		 * out-of-memory conditions or serious errors such as inability
+		 * to send the command to the backend. If a NULL is returned,
+		 *  it should be treated like a PGRES_FATAL_ERROR result.
+		 * Use PQerrorMessage to get more information about the error.
+		 */
 	if (!pg_sock->result)
 	{
-		radlog(L_ERR, "rlm_sql_postgresql: PostgreSQL Query failed Error: %s", 
+		radlog(L_ERR, "rlm_sql_postgresql: PostgreSQL Query failed Error: %s",
 				PQerrorMessage(pg_sock->conn));
 		return  SQL_DOWN;
 	} else {
@@ -261,6 +277,20 @@ static int sql_store_result(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 }
 
 
+/*************************************************************************
+ *
+ *      Function: sql_destroy_socket
+ *
+ *      Purpose: Free socket and private connection data
+ *
+ *************************************************************************/
+static int sql_destroy_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config)
+{
+        free(sqlsocket->conn);
+	sqlsocket->conn = NULL;
+        return 0;
+}
+
 /*************************************************************************
  *
  *	Function: sql_num_fields
@@ -444,7 +474,7 @@ not_implemented(SQLSOCK * sqlsocket, SQL_CONFIG *config)
 rlm_sql_module_t rlm_sql_postgresql = {
 	"rlm_sql_postgresql",
 	sql_init_socket,
-	not_implemented, /* sql_destroy_socket*/
+	sql_destroy_socket,
 	sql_query,
 	sql_select_query,
 	not_implemented, /* sql_store_result */
