@@ -2,7 +2,7 @@
  * rlm_sql.c		SQL Module
  * 		Main SQL module file. Most ICRADIUS code is located in sql.c
  *
- * Version:	$Id: rlm_sql.c,v 1.131 2004/05/25 06:56:43 phampson Exp $
+ * Version:	$Id: rlm_sql.c,v 1.131.2.1 2004/09/30 14:54:22 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
  */
 
 static const char rcsid[] =
-	"$Id: rlm_sql.c,v 1.131 2004/05/25 06:56:43 phampson Exp $";
+	"$Id: rlm_sql.c,v 1.131.2.1 2004/09/30 14:54:22 aland Exp $";
 
 #include "autoconf.h"
 
@@ -459,7 +459,7 @@ static int sql_set_user(SQL_INST *inst, REQUEST *request, char *sqlusername, con
 	if (username != NULL) {
 		strNcpy(tmpuser, username, MAX_STRING_LEN);
 	} else if (strlen(inst->config->query_user)) {
-		radius_xlat(tmpuser, sizeof(tmpuser), inst->config->query_user, request, sql_escape_func);
+		radius_xlat(tmpuser, sizeof(tmpuser), inst->config->query_user, request, NULL);
 	} else {
 		return 0;
 	}
