@@ -2,7 +2,7 @@
  *  sql.c		rlm_sql - FreeRADIUS SQL Module
  *		Main code directly taken from ICRADIUS
  *
- * Version:	$Id: sql.c,v 1.72.2.4 2003/08/27 16:00:43 phampson Exp $
+ * Version:	$Id: sql.c,v 1.79 2004/05/25 18:35:32 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -48,7 +48,7 @@
 #include	"conffile.h"
 #include	"rlm_sql.h"
 
-#if HAVE_PTHREAD_H
+#ifdef HAVE_PTHREAD_H
 #include	<pthread.h>
 #endif
 
@@ -113,7 +113,7 @@ int sql_init_socketpool(SQL_INST * inst)
 		sqlsocket->id = i;
 		sqlsocket->state = sockunconnected;
 
-#if HAVE_PTHREAD_H
+#ifdef HAVE_PTHREAD_H
 		rcode = pthread_mutex_init(&sqlsocket->mutex,NULL);
 		if (rcode != 0) {
 			radlog(L_ERR, "rlm_sql: Failed to init lock: %s",
@@ -184,7 +184,7 @@ int sql_close_socket(SQL_INST *inst, SQLSOCK * sqlsocket)
 	if (inst->module->sql_destroy_socket) {
 		(inst->module->sql_destroy_socket)(sqlsocket, inst->config);
 	}
-#if HAVE_PTHREAD_H
+#ifdef HAVE_PTHREAD_H
 	pthread_mutex_destroy(&sqlsocket->mutex);
 #endif
 	free(sqlsocket);
@@ -196,7 +196,7 @@ int sql_close_socket(SQL_INST *inst, SQLSOCK * sqlsocket)
  *
  *	Function: sql_get_socket
  *
- *	Purpose: Return a SQL sqlsocket from the connection pool           
+ *	Purpose: Return a SQL sqlsocket from the connection pool
  *
  *************************************************************************/
 SQLSOCK * sql_get_socket(SQL_INST * inst)
@@ -214,7 +214,7 @@ SQLSOCK * sql_get_socket(SQL_INST * inst)
 	cur = start;
 
 	while (cur) {
-#if HAVE_PTHREAD_H
+#ifdef HAVE_PTHREAD_H
 		/*
 		 *	If this socket is in use by another thread,
 		 *	skip it, and try another socket.
@@ -242,7 +242,7 @@ SQLSOCK * sql_get_socket(SQL_INST * inst)
 		if (cur->state == sockunconnected) {
 			radlog(L_DBG, "rlm_sql (%s): Ignoring unconnected handle %d..", inst->config->xlat_name, cur->id);
 		        unconnected++;
-#if HAVE_PTHREAD_H
+#ifdef HAVE_PTHREAD_H
 			pthread_mutex_unlock(&cur->mutex);
 #endif
 			goto next;
@@ -254,7 +254,7 @@ SQLSOCK * sql_get_socket(SQL_INST * inst)
 		if (unconnected != 0 || tried_to_connect != 0) {
 			radlog(L_INFO, "rlm_sql (%s): got socket %d after skipping %d unconnected handles, tried to reconnect %d though", inst->config->xlat_name, cur->id, unconnected, tried_to_connect);
 		}
-		
+
 		/*
 		 *	The socket is returned in the locked
 		 *	state.
@@ -285,7 +285,7 @@ SQLSOCK * sql_get_socket(SQL_INST * inst)
 		}
 
 		/*
-		 *	If we're at the socket we started 
+		 *	If we're at the socket we started
 		 */
 		if (cur == start) {
 			break;
@@ -301,12 +301,12 @@ SQLSOCK * sql_get_socket(SQL_INST * inst)
  *
  *	Function: sql_release_socket
  *
- *	Purpose: Frees a SQL sqlsocket back to the connection pool           
+ *	Purpose: Frees a SQL sqlsocket back to the connection pool
  *
  *************************************************************************/
 int sql_release_socket(SQL_INST * inst, SQLSOCK * sqlsocket)
 {
-#if HAVE_PTHREAD_H
+#ifdef HAVE_PTHREAD_H
 	pthread_mutex_unlock(&sqlsocket->mutex);
 #endif
 
@@ -330,7 +330,8 @@ int sql_userparse(VALUE_PAIR ** first_pair, SQL_ROW row, int querymode)
 	VALUE_PAIR *pair, *check;
 	char *ptr;
 	char buf[128];
-	int pairmode = T_EOL;
+	char value[256];
+	LRAD_TOKEN xlat, pairmode = T_EOL;
 
 	if ((attr = dict_attrbyname(row[2])) == (DICT_ATTR *) NULL) {
 		radlog(L_ERR | L_CONS, "rlm_sql: unknown attribute %s",
@@ -352,8 +353,8 @@ int sql_userparse(VALUE_PAIR ** first_pair, SQL_ROW row, int querymode)
 	if (pairmode <= T_EOL) pairmode = T_OP_CMP_EQ;
 
 	/*
-	 * If attribute is already there, skip it because we checked usercheck first 
-	 * and we want user settings to over ride group settings 
+	 * If attribute is already there, skip it because we checked usercheck first
+	 * and we want user settings to over ride group settings
 	 */
 	if (pairmode != T_OP_ADD && (check = pairfind(*first_pair, attr->attr)) != NULL &&
 #ifdef ASCEND_BINARY
@@ -362,7 +363,47 @@ int sql_userparse(VALUE_PAIR ** first_pair, SQL_ROW row, int querymode)
 			querymode == PW_VP_GROUPDATA)
 		return 0;
 
-	pair = pairmake(row[2], row[3], pairmode);
+	/*
+	 *	If we have a new-style quoted string, where the
+	 *	*entire* string is quoted, do xlat's.
+	 */
+	if (((row[3][0] == '\'') ||
+	     (row[3][0] == '`') ||
+	     (row[3][0] == '"')) &&
+	    (row[3][0] == row[3][strlen(row[3])-1])) {
+
+		ptr = row[3];
+		xlat = gettoken(&ptr, value, sizeof(value));
+		switch (xlat) {
+			/*
+			 *	Make the full pair now.
+			 */
+		default:
+			pair = pairmake(row[2], row[3], pairmode);
+			break;
+
+		case T_SINGLE_QUOTED_STRING:
+		case T_DOUBLE_QUOTED_STRING:
+			pair = pairmake(row[2], value, pairmode);
+			break;
+
+			/*
+			 *	Mark the pair to be allocated later.
+			 */
+		case T_BACK_QUOTED_STRING:
+			pair = pairmake(row[2], NULL, pairmode);
+			if (pair) {
+				pair->flags.do_xlat = 1;
+				strNcpy(pair->strvalue, value, sizeof(pair->strvalue));
+				pair->length = 0;
+			}
+		}
+	} else {
+		/*
+		 * String starts and ends differently. Take it literally
+		 * */
+		pair = pairmake(row[2], row[3], pairmode);
+	}
 	pairadd(first_pair, pair);
 
 	return 0;
@@ -557,7 +598,7 @@ void query_log(REQUEST *request, SQL_INST *inst, char *querystr)
 			       buffer);
 		} else {
 			int fd = fileno(sqlfile);
-			
+
 			rad_lockfd(fd, MAX_QUERY_LEN);
 			fputs(querystr, sqlfile);
 			fputs(";\n", sqlfile);
