@@ -1,7 +1,7 @@
 /*
  * rlm_sqlcounter.c
  *
- * Version:  $Id: rlm_sqlcounter.c,v 1.7 2002/09/08 14:47:09 kkalev Exp $
+ * Version:  $Id: rlm_sqlcounter.c,v 1.11 2004/05/15 15:50:26 mgriego Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -43,20 +43,20 @@
 
 
 /* 	Note: When your counter spans more than 1 period (ie 3 months or 2 weeks), this module
- *	probably does NOT do what you want!  It calculates the range of dates to count across 
+ *	probably does NOT do what you want!  It calculates the range of dates to count across
  *	by first calculating the End of the Current period and then subtracting the number of
  *	periods you specify from that to determine the beginning of the range.
  *
- *	For example, if you specify a 3 month counter and today is June 15th, the end of the current 
- *	period is June 30. Subtracting 3 months from that gives April 1st.  So, the counter will 
+ *	For example, if you specify a 3 month counter and today is June 15th, the end of the current
+ *	period is June 30. Subtracting 3 months from that gives April 1st.  So, the counter will
  *	sum radacct entries from April 1st to June 30. Then, next month, it will sum entries
  *	from May 1st to July 31st.
  *
- *	To fix this behavior, we need to add some way of storing the Next Reset Time 
+ *	To fix this behavior, we need to add some way of storing the Next Reset Time
  */
 
 
-static const char rcsid[] = "$Id: rlm_sqlcounter.c,v 1.7 2002/09/08 14:47:09 kkalev Exp $";
+static const char rcsid[] = "$Id: rlm_sqlcounter.c,v 1.11 2004/05/15 15:50:26 mgriego Exp $";
 
 /*
  *	Define a structure for our module configuration.
@@ -157,7 +157,7 @@ static int find_next_reset(rlm_sqlcounter_t *data, time_t timeval)
 		return -1;
 	}
 	strftime(sNextTime, sizeof(sNextTime),"%Y-%m-%d %H:%M:%S",tm);
-	DEBUG2("rlm_sqlcounter: Current Time: %d [%s], Next reset %d [%s]", 
+	DEBUG2("rlm_sqlcounter: Current Time: %d [%s], Next reset %d [%s]",
 		(int)timeval,sCurrentTime,(int)data->reset_time, sNextTime);
 
 	return ret;
@@ -227,7 +227,7 @@ static int find_prev_reset(rlm_sqlcounter_t *data, time_t timeval)
 		return -1;
 	}
 	strftime(sPrevTime, sizeof(sPrevTime),"%Y-%m-%d %H:%M:%S",tm);
-	DEBUG2("rlm_sqlcounter: Current Time: %d [%s], Prev reset %d [%s]", 
+	DEBUG2("rlm_sqlcounter: Current Time: %d [%s], Prev reset %d [%s]",
 		(int)timeval,sCurrentTime,(int)data->last_reset, sPrevTime);
 
 	return ret;
@@ -264,7 +264,7 @@ static int sqlcounter_expand(char *out, int outlen, const char *fmt, void *insta
 			/*
 			 * We check if we're inside an open brace.  If we are
 			 * then we assume this brace is NOT literal, but is
-			 * a closing brace and apply it 
+			 * a closing brace and apply it
 			 */
 			if((c == '}') && openbraces) {
 				openbraces--;
@@ -295,20 +295,20 @@ static int sqlcounter_expand(char *out, int outlen, const char *fmt, void *insta
 				*q++ = *p;
 			case 'b': /* last_reset */
 				sprintf(tmpdt, "%lu", data->last_reset);
-				strNcpy(q, tmpdt, freespace); 
+				strNcpy(q, tmpdt, freespace);
 				q += strlen(q);
 				break;
 			case 'e': /* reset_time */
 				sprintf(tmpdt, "%lu", data->reset_time);
-				strNcpy(q, tmpdt, freespace); 
+				strNcpy(q, tmpdt, freespace);
 				q += strlen(q);
 				break;
 			case 'k': /* Key Name */
-				strNcpy(q, data->key_name, freespace); 
+				strNcpy(q, data->key_name, freespace);
 				q += strlen(q);
 				break;
 			case 'S': /* SQL module instance */
-				strNcpy(q, data->sqlmod_inst, freespace); 
+				strNcpy(q, data->sqlmod_inst, freespace);
 				q += strlen(q);
 				break;
 			default:
@@ -374,11 +374,15 @@ static int sqlcounter_instantiate(CONF_SECTION *conf, void **instance)
 	DICT_ATTR *dattr;
 	ATTR_FLAGS flags;
 	time_t now;
-	
+
 	/*
 	 *	Set up a storage area for instance data
 	 */
 	data = rad_malloc(sizeof(*data));
+	if (!data) {
+		return -1;
+	}
+	memset(data, 0, sizeof(*data));
 
 	/*
 	 *	If the configuration parameters can't be parsed, then
@@ -390,11 +394,11 @@ static int sqlcounter_instantiate(CONF_SECTION *conf, void **instance)
 	}
 
 	/*
-	 *	Discover the attribute number of the key. 
+	 *	Discover the attribute number of the key.
 	 */
 	if (data->key_name == NULL) {
 		radlog(L_ERR, "rlm_sqlcounter: 'key' must be set.");
-		exit(0);
+		return -1;
 	}
 	dattr = dict_attrbyname(data->key_name);
 	if (dattr == NULL) {
@@ -403,14 +407,14 @@ static int sqlcounter_instantiate(CONF_SECTION *conf, void **instance)
 		return -1;
 	}
 	data->key_attr = dattr->attr;
-	
+
 
 	/*
 	 *  Create a new attribute for the counter.
 	 */
 	if (data->counter_name == NULL) {
 		radlog(L_ERR, "rlm_sqlcounter: 'counter-name' must be set.");
-		exit(0);
+		return -1;
 	}
 
 	memset(&flags, 0, sizeof(flags));
@@ -430,7 +434,7 @@ static int sqlcounter_instantiate(CONF_SECTION *conf, void **instance)
 	 */
 	if (data->check_name == NULL) {
 		radlog(L_ERR, "rlm_sqlcounter: 'check-name' must be set.");
-		exit(0);
+		return -1;
 	}
 	dict_addattr(data->check_name, 0, PW_TYPE_INTEGER, -1, flags);
 	dattr = dict_attrbyname(data->check_name);
@@ -447,7 +451,7 @@ static int sqlcounter_instantiate(CONF_SECTION *conf, void **instance)
 	 */
 	if (data->reset == NULL) {
 		radlog(L_ERR, "rlm_sqlcounter: 'reset' must be set.");
-		exit(0);
+		return -1;
 	}
 	now = time(NULL);
 	data->reset_time = 0;
@@ -470,7 +474,7 @@ static int sqlcounter_instantiate(CONF_SECTION *conf, void **instance)
 	paircompare_register(data->dict_attr, 0, sqlcounter_cmp, data);
 
 	*instance = data;
-	
+
 	return 0;
 }
 
@@ -612,12 +616,12 @@ static int sqlcounter_authorize(void *instance, REQUEST *request)
 
 		snprintf(module_fmsg, sizeof(module_fmsg), "rlm_sqlcounter: Maximum %s usage time reached", data->reset);
 		module_fmsg_vp = pairmake("Module-Failure-Message", module_fmsg, T_OP_EQ);
-		pairadd(&request->packet->vps, module_fmsg_vp);	
+		pairadd(&request->packet->vps, module_fmsg_vp);
 
 		ret=RLM_MODULE_REJECT;
 
 		DEBUG2("rlm_sqlcounter: Rejected user %s, check_item=%d, counter=%d",
-				key_vp->strvalue,check_vp->lvalue,counter); 
+				key_vp->strvalue,check_vp->lvalue,counter);
 	}
 
 	return ret;
@@ -648,7 +652,7 @@ static int sqlcounter_detach(void *instance)
  *	is single-threaded.
  */
 module_t rlm_sqlcounter = {
-	"SQL Counter",	
+	"SQL Counter",
 	RLM_TYPE_THREAD_SAFE,		/* type */
 	NULL,				/* initialization */
 	sqlcounter_instantiate,		/* instantiation */
