@@ -3,7 +3,7 @@
  *		accounting:     Functions to write radwtmp file.
  *		Also contains handler for "Group".
  *
- * Version:	$Id: rlm_unix.c,v 1.59.2.2 2003/08/26 12:37:50 phampson Exp $
+ * Version:	$Id: rlm_unix.c,v 1.63 2004/02/26 19:04:37 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  * Copyright 2000  Jeff Carneal <jeff@apex.net>
  * Copyright 2000  Alan Curry <pacman@world.std.com>
  */
-static const char rcsid[] = "$Id: rlm_unix.c,v 1.59.2.2 2003/08/26 12:37:50 phampson Exp $";
+static const char rcsid[] = "$Id: rlm_unix.c,v 1.63 2004/02/26 19:04:37 aland Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -37,7 +37,7 @@ static const char rcsid[] = "$Id: rlm_unix.c,v 1.59.2.2 2003/08/26 12:37:50 pham
 
 #include "config.h"
 
-#if HAVE_SHADOW_H
+#ifdef HAVE_SHADOW_H
 #  include	<shadow.h>
 #endif
 
@@ -93,7 +93,7 @@ static CONF_PARSER module_config[] = {
 	  offsetof(struct unix_instance,usegroup), NULL,     "no" },
 	{ "cache_reload", PW_TYPE_INTEGER,
 	  offsetof(struct unix_instance,cache_reload), NULL, "600" },
-	
+
 	{ NULL, -1, 0, NULL, NULL }		/* end the list */
 };
 
@@ -111,7 +111,7 @@ static struct unix_instance *group_inst;
  * file) or not ("Group=" was bound to the first instance of rlm_unix */
 static int group_inst_explicit;
 
-#if HAVE_GETSPNAM
+#ifdef HAVE_GETSPNAM
 #if defined(M_UNIX)
 static inline const char *get_shadow_name(shadow_pwd_t *spwd) {
 	if (spwd == NULL) return NULL;
@@ -168,7 +168,7 @@ static struct group *fgetgrnam(const char *fname, const char *name) {
 	return grp;
 }
 
-#if HAVE_GETSPNAM
+#ifdef HAVE_GETSPNAM
 
 static shadow_pwd_t *fgetspnam(const char *fname, const char *name) {
 	FILE		*file = fopen(fname, "ro");
@@ -191,14 +191,16 @@ static shadow_pwd_t *fgetspnam(const char *fname, const char *name) {
 /*
  *	The Group = handler.
  */
-static int groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE_PAIR *check,
-	VALUE_PAIR *check_pairs, VALUE_PAIR **reply_pairs)
+static int groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request,
+		    VALUE_PAIR *check, VALUE_PAIR *check_pairs,
+		    VALUE_PAIR **reply_pairs)
 {
 	struct passwd	*pwd;
 	struct group	*grp;
 	char		**member;
 	char		*username;
 	int		retval;
+	VALUE_PAIR	*vp;
 
 	instance = instance;
 	check_pairs = check_pairs;
@@ -212,10 +214,14 @@ static int groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE_PAI
 	/*
 	 *	No user name, doesn't compare.
 	 */
-	if (!req->username) {
-		return -1;
+	vp = pairfind(request, PW_STRIPPED_USER_NAME);
+	if (!vp) {
+		vp = pairfind(request, PW_USER_NAME);
+		if (!vp) {
+			return -1;
+		}
 	}
-	username = (char *)req->username->strvalue;
+	username = (char *)vp->strvalue;
 
 	if (group_inst->cache_passwd &&
 	    (retval = H_groupcmp(group_inst->cache, check, username)) != -2)
@@ -373,7 +379,7 @@ static int unix_authenticate(void *instance, REQUEST *request)
 	struct passwd	*pwd;
 	const char	*encrypted_pass;
 	int		ret;
-#if HAVE_GETSPNAM
+#ifdef HAVE_GETSPNAM
 	shadow_pwd_t	*spwd = NULL;
 #endif
 #ifdef OSFC2
@@ -511,7 +517,7 @@ static int unix_authenticate(void *instance, REQUEST *request)
 	encrypted_pass = pwd->pw_passwd;
 #endif /* OSFC2 */
 
-#if HAVE_GETSPNAM
+#ifdef HAVE_GETSPNAM
 	/*
 	 *      See if there is a shadow password.
 	 *
@@ -546,7 +552,7 @@ static int unix_authenticate(void *instance, REQUEST *request)
 	}
 #endif
 
-#if HAVE_GETUSERSHELL
+#ifdef HAVE_GETUSERSHELL
 	/*
 	 *	Check /etc/shells for a valid shell. If that file
 	 *	contains /RADIUSD/ANY/SHELL then any shell will do.
@@ -809,7 +815,7 @@ static int unix_accounting(void *instance, REQUEST *request)
 			return RLM_MODULE_FAIL;
 		}
 		fclose(fp);
-	} else 
+	} else
 		return RLM_MODULE_FAIL;
 
 	return RLM_MODULE_OK;
