@@ -1,6 +1,6 @@
 /*
  * x99_pwe.c
- * $Id: x99_pwe.c,v 1.17 2002/07/10 08:25:59 fcusack Exp $
+ * $Id: x99_pwe.c,v 1.19 2002/11/04 04:02:02 fcusack Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -20,14 +20,13 @@
  */
 
 /*
- * This file implements password checking functions for each supported
- * encoding (PAP, CHAP, etc.).  The current libradius interface is not
- * sufficient for x9.9 use.
+ * This file implements passcode (password) checking functions for each
+ * supported encoding (PAP, CHAP, etc.).  The current libradius interface
+ * is not sufficient for X9.9 use.
  */
 
 #ifdef FREERADIUS
 #define _LRAD_MD4_H
-#define _LRAD_MD5_H
 #define _LRAD_SHA1_H
 #include "libradius.h"
 #include "rad_assert.h"
@@ -42,7 +41,7 @@
 
 #include <string.h>
 
-static const char rcsid[] = "$Id: x99_pwe.c,v 1.17 2002/07/10 08:25:59 fcusack Exp $";
+static const char rcsid[] = "$Id: x99_pwe.c,v 1.19 2002/11/04 04:02:02 fcusack Exp $";
 
 
 /* Attribute IDs for supported password encodings. */
@@ -60,7 +59,6 @@ x99_pwe_init(void)
      * Setup known password types.  These are pairs.
      * NB: Increase pwattr array size when adding a type.
      *     It should be sized as (number of password types * 2)
-     * This is temporary until this code becomes modular.
      */
     (void) memset(pwattr, 0, sizeof(pwattr));
 
@@ -314,6 +312,7 @@ x99_pw_valid(const REQUEST *request, x99_token_t *inst,
 	    /* The NT-Key sub-field is MD4(MD4(unicode(password))). */
 	    (void) MD4(nt_keys, 16, &mppe_keys[8]);
 
+#if 0 /* encoding now handled in lib/radius.c:rad_pwencode() */
 	    /* Now we must encode the key as User-Password is encoded. */
 	    secretlen = strlen(request->secret);
 	    (void) memcpy(encode_buf, request->secret, secretlen);
@@ -326,6 +325,7 @@ x99_pw_valid(const REQUEST *request, x99_token_t *inst,
 	    (void) MD5(encode_buf, secretlen + MD5_DIGEST_LENGTH, md5_md);
 	    for (i = 0; i < 16; ++i)
 		mppe_keys[i + 16] ^= md5_md[i];
+#endif /* 0 */
 
 	    /* Whew.  Now stringify it for pairmake(). */
 	    mppe_keys_string[0] = '0';
@@ -746,14 +746,14 @@ static void
 x99_key_from_hash(des_cblock *key, const unsigned char hashbytes[7])
 {
     int i;
-    unsigned char cNext = 0;
-    unsigned char cWorking = 0;
+    unsigned char working;
+    unsigned char next = 0;
 
     for (i = 0; i < 7; ++i) {
-	cWorking = hashbytes[i];
-	(*key)[i] = (cWorking >> i) | cNext;
-	cNext = (cWorking << (7 - i));
+	working = hashbytes[i];
+	(*key)[i] = (working >> i) | next;
+	next = (working << (7 - i));
     }
-    (*key)[i] = cNext;
+    (*key)[i] = next;
 }
 
