@@ -1,6 +1,6 @@
 /*
  * x99_rlm.c
- * $Id: x99_rlm.c,v 1.37 2002/11/13 04:21:59 fcusack Exp $
+ * $Id: x99_rlm.c,v 1.39.2.1 2004/11/03 17:28:25 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
  * An attacker can learn the token's secret by observing two
  * challenge/response pairs.  See ANSI document X9 TG-24-1999
  * <URL:http://www.x9.org/TG24_1999.pdf>.
- * 
+ *
  * Please read the accompanying docs.
  */
 
@@ -53,7 +53,7 @@
 #endif
 #include "x99.h"
 
-static const char rcsid[] = "$Id: x99_rlm.c,v 1.37 2002/11/13 04:21:59 fcusack Exp $";
+static const char rcsid[] = "$Id: x99_rlm.c,v 1.39.2.1 2004/11/03 17:28:25 aland Exp $";
 
 /* Global data */
 static int rnd_fd;			/* fd for random device           */
@@ -143,6 +143,9 @@ x99_token_instantiate(CONF_SECTION *conf, void **instance)
 
     /* Set up a storage area for instance data. */
     data = rad_malloc(sizeof(*data));
+    if (!data)
+	return -1;
+    memset(data, 0, sizeof(*data));
 
     /* If the configuration parameters can't be parsed, then fail. */
     if (cf_section_parse(conf, data, module_config) < 0) {
@@ -541,9 +544,6 @@ x99_token_authenticate(void *instance, REQUEST *request)
 			"auth: bad state for [%s]: expired", username);
 		return RLM_MODULE_REJECT;
 	    }
-good_state:
-	    /* State is good! */
-
 	} else {
 	    /* This should only happen if the authorize code didn't run. */
 	    x99_log(X99_LOG_ERR, "auth: bad state for [%s]: missing "
@@ -553,6 +553,9 @@ good_state:
 	}
     } /* if (!fast_sync) */
 
+good_state:
+	    /* State is good! */
+
     /* Get the time of the last authentication. */
     if (x99_get_last_auth(inst->syncdir, username, &last_auth) != 0) {
 	x99_log(X99_LOG_ERR,
@@ -848,7 +851,7 @@ x99_token_destroy(void)
  *	is single-threaded.
  */
 module_t rlm_x99_token = {
-	"x99_token",	
+	"x99_token",
 	RLM_TYPE_THREAD_SAFE,		/* type */
 	x99_token_init,			/* initialization */
 	x99_token_instantiate,		/* instantiation */
