@@ -1,6 +1,6 @@
 /*
  * x99_site.c
- * $Id: x99_site.c,v 1.2 2002/01/17 04:02:18 fcusack Exp $
+ * $Id: x99_site.c,v 1.6 2002/07/03 09:03:47 fcusack Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -34,11 +34,10 @@
  * IMPORTANT  IMPORTANT  IMPORTANT  IMPORTANT  IMPORTANT  IMPORTANT
  */
 
-#include "autoconf.h"
-#include "libradius.h"
 #include "x99.h"
+#include <string.h>
 
-static const char rcsid[] = "$Id: x99_site.c,v 1.2 2002/01/17 04:02:18 fcusack Exp $";
+static const char rcsid[] = "$Id: x99_site.c,v 1.6 2002/07/03 09:03:47 fcusack Exp $";
 
 
 int
@@ -47,7 +46,7 @@ x99_challenge_transform(const char *username,
 {
     /* ARGSUSED */
 
-    strcpy(challenge, "DISABLED");
+    (void) strcpy(challenge, "DISABLED");
     return 0;
 }
 
