@@ -1,6 +1,6 @@
 /*
  * x99_sync.c
- * $Id: x99_sync.c,v 1.15 2002/08/28 08:33:08 fcusack Exp $
+ * $Id: x99_sync.c,v 1.16 2003/01/10 07:04:08 fcusack Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@
 #include <unistd.h>
 #include <openssl/des.h> /* des_cblock */
 
-static const char rcsid[] = "$Id: x99_sync.c,v 1.15 2002/08/28 08:33:08 fcusack Exp $";
+static const char rcsid[] = "$Id: x99_sync.c,v 1.16 2003/01/10 07:04:08 fcusack Exp $";
 
 
 /*
@@ -465,7 +465,11 @@ x99_acquire_sd_lock(const char *syncdir, const char *username)
 	if ((fd = open(lockfile, O_CREAT|O_EXCL, S_IRUSR|S_IWUSR)) != -1) {
 	    break;
 	}
-	/* FIXME: does usleep() reset/generate SIGALRM on any systems? */
+	/* break stale locks (older than 60s) */
+	if (stat(lockfile, &st) == 0)
+	    if (st.st_ctime  < time(NULL) - 60)
+		(void) unlink(lockfile);
+
 	usleep(500000); /* 0.5 second */
     }
     if (fd == -1) {
