@@ -1,6 +1,6 @@
 /*
  * x99_sync.h
- * $Id: x99_sync.h,v 1.4 2002/07/03 08:42:04 fcusack Exp $
+ * $Id: x99_sync.h,v 1.5 2002/08/28 08:33:08 fcusack Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -29,8 +29,9 @@ static void x99_release_sd_lock(char *lockfile);
 
 static int x99_get_sd(const char *syncdir, const char *username,
 		      char challenge[MAX_CHALLENGE_LEN + 1], int *failures,
-		      time_t *last_async);
+		      time_t *last_async, unsigned *pos);
 static int x99_set_sd(const char *syncdir, const char *username,
-		      const char *challenge, int failures, time_t last_async);
+		      const char *challenge, int failures, time_t last_async,
+		      unsigned pos);
 
 #endif /* X99_SYNC_H */
